/** 
@author CG
@license Adapted from https://observablehq.com/@d3/grouped-bar-chart | ISC
@log 2023-01-04 adapted from Observable
@log 2024-02-11 migrated to AVAA_STYLE
@image png
@chart
@todo fix that damn vertical separator position
@todo wtf filterZ prop?
**/
chart['grouped-bar'] = function(data, {
  id,
  x = (d, i) => i, // given d in data, returns the (ordinal) x-value
  y = d => d, // given d in data, returns the (quantitative) y-value
  z = () => 1, // given d in data, returns the (categorical) z-value
  title, // given d in data, returns the title text
  marginTop = 30, // top margin, in pixels
  marginRight = 0, // right margin, in pixels
  marginBottom = 30, // bottom margin, in pixels
  marginLeft = 40, // left margin, in pixels
  width = 640, // outer width, in pixels
  height = 400, // outer height, in pixels
  xDomain, // the x domain [xmin, xmax] $$ d3.groupSort(data, g => d3.median(g, ${x}), ${x})
  xRange = [marginLeft, width - marginRight], // [xmin, xmax]
  xPadding = 0.1, // amount of x-range to reserve to separate groups
  yType = d3.scaleLinear, // type of y-scale
  yDomain, // [ymin, ymax]
  yRange = [height - marginBottom, marginTop], // [ymin, ymax]
  zDomain, // the z domain $$ d3.groupSort(data, g => -d3.median(g, ${y}), ${z})
  zPadding = 0.05, // amount of x-range to reserve to separate bars
  yFormat, // a format specifier string for the y-axis
  yLabel, // a label for the y-axis
  filterZ = false, // other way
  separatorWidth = 0,
  separatorOffset = 0,
  separatorColor = 'gainsboro',
  colors, // array of colors
} = {}) {
  // Compute values.
  const STYLE = (typeof AVAA_STYLE !== "undefined" ? AVAA_STYLE : null);  
  const X = d3.map(data, x);
  const Y = d3.map(data, y);
  const Z = d3.map(data, z);
	const Z2 = Z.filter(function(item, pos, self) {
    return self.indexOf(item) == pos;
	})
  // Compute default domains, and unique the x- and z-domains.
  if (xDomain === undefined) xDomain = X;
  if (yDomain === undefined) yDomain = [0, d3.max(Y)];
  if (zDomain === undefined) zDomain = Z2;
  //console.info(Z2)
  xDomain = new d3.InternSet(xDomain);
  zDomain = new d3.InternSet(zDomain);
  //console.info(zDomain)
  // Omit any data not present in both the x- and z-domain.
  const I = d3.range(X.length).filter(i => xDomain.has(X[i]) && zDomain.has(Z[i]));	

  //if (colors === undefined) colors = d3.schemeSpectral[zDomain.size];
  if (colors === undefined) colors = d3.quantize(t => d3.interpolateSpectral(t * 0.8 + 0.1), zDomain.size);
  // Construct scales, axes, and formats.
  const xScale = d3.scaleBand(xDomain, xRange).paddingInner(xPadding);
  
  
  //const xzScale = d3.scaleBand(zDomain, [0, xScale.bandwidth()]).padding(zPadding);
  const xzScale = d3.scaleBand(filterZ ? Z2 : zDomain, [0, xScale.bandwidth()]).padding(zPadding);
  
  
  const yScale = yType(yDomain, yRange);
  //var zScale = d3.scaleOrdinal(Z2, colors);  
  const xAxis = d3.axisBottom(xScale).tickSizeOuter(0);
  const yAxis = d3.axisLeft(yScale).ticks(height / 60, yFormat);
	
	/*const defaultColors = colors.slice();	
	const colorMap = window.CHART_COLORS ? window.CHART_COLORS : {};
	const finalColors = [];
	Z2.forEach(ii=>finalColors.push(colorMap[ii]||defaultColors.shift()))
	//console.info(finalColors);
	zScale = d3.scaleOrdinal().domain(Z2).range(finalColors)*/
	
	const defaultColors = colors.slice();	
	//const finalColors = zDomain.map(ii=>(STYLE ? STYLE.colorOf(ii)||defaultColors.shift() : defaultColors.shift()));
	const finalColors = []; // no zDomain.map!?
	zDomain.forEach(ii=>finalColors.push(STYLE ? STYLE.colorOf(ii)||defaultColors.shift() : defaultColors.shift()));	
	const zScale = d3.scaleOrdinal().domain(zDomain).range(finalColors);

  // Compute titles.
  if (title === undefined) {
    const formatValue = yScale.tickFormat(100, yFormat);
    title = i => `${X[i]}\n${STYLE?STYLE.labelOf(Z[i])||Z[i]:Z[i]}\n${formatValue(Y[i])}`;
  } else {
    const O = d3.map(data, d => d);
    const T = title;
    title = i => T(O[i], i, data);
  }

  const svg = d3.select("#"+id)
      .attr("width", width)
      .attr("height", height)
      .attr("viewBox", [0, 0, width, height])
      .attr("style", "max-width: 100%; height: auto; height: intrinsic;");

  svg.append("g")
      .attr("transform", `translate(${marginLeft},0)`)
      .call(yAxis)
      .call(g => g.select(".domain").remove())
      .call(g => g.selectAll(".tick line").clone()
          .attr("x2", width - marginLeft - marginRight)
          .attr("stroke-opacity", 0.1))
      .call(g => g.append("text")
          .attr("x", -marginLeft)
          .attr("y", 10)
          .attr("fill", "currentColor")
          .attr("text-anchor", "start")
          .text(yLabel));

  const bar = svg.append("g")
    .selectAll("rect")
    .data(I)
    .join("rect")
      .attr("x", i => xScale(X[i]) + xzScale(Z[i]))
      .attr("y", i => yScale(Y[i]))
      .attr("width", xzScale.bandwidth())
      .attr("height", i => yScale(0) - yScale(Y[i]))
      .attr("fill", i => zScale(Z[i]));
	
if(separatorWidth){
const separator = svg.append("g")
    .selectAll("rect")
    .data(I)
    .join("line")
      .attr("x1", i => xScale(X[i]) + xzScale(Z[Z.length-1]) + xzScale.bandwidth() + separatorOffset)
	  .attr("x2", i => xScale(X[i]) + xzScale(Z[Z.length-1]) + xzScale.bandwidth() + separatorOffset)
      .attr("y1", i => marginTop - 10)
      .attr("width", 1)
	  .attr("stroke",separatorColor)
	  .attr("stroke-dasharray","10,10")
	  .attr("stroke-width",separatorWidth)
	  .attr("opacity", i => i%(Z.length-1)===0 ? 1 : 1)
      .attr("y2", i =>  height - marginBottom)
      .attr("fill", i => 'none');	
	  
	  let foo = false && bar.append("line")
        .attr("class", "blabla")
        .attr("x1", xScale.bandwidth()+20)
        .attr("x2", xScale.bandwidth()+20)
        .attr("y1", 0)
        .attr("y2",)
        .style("stroke-width", 1)
        .style("stroke", "#000");
}

  if (title) bar.append("title")
      .text(title);

  svg.append("g")
      .attr("transform", `translate(0,${height - marginBottom})`)
      .call(xAxis);

  return Object.assign(svg.node(), {scales: {color: zScale}});
}