# 2023-11-28
# python -u
# Demucs Wrapper for AVAA Toolkit by CG
# Credits:
#    - demucs idea from https://github.com/EtienneAb3d/WhisperHallu
#    - Custom Progress Bar hook concept by aadnk https://github.com/openai/whisper/discussions/850

import os, sys
import argparse
import json
import time
import torch
import torchaudio
import demucs
from demucs.pretrained import get_model_from_args
from demucs.apply import apply_model
from demucs.separate import load_track
from torch._C import device

def load_demucs_model():
    return get_model_from_args(type('args', (object,), dict(name='htdemucs', repo=None))).cpu().eval()

def main(argv):
    try:
        parser = argparse.ArgumentParser(add_help=False)
        parser.add_argument('-i', '--input', dest='input') 
        parser.add_argument('-o', '--output', dest='output')        
        parser.add_argument('-s', '--source', dest='source')        
        parser.add_argument('-h', '--help', dest='help', action='store_true')
        parser.add_argument('-f', '--fake', dest='fake', action='store_true')
        parser.add_argument('-v', '--version', dest='version', action='store_true')
        parser.add_argument('-V', '--verbose', dest='verbose', action='store_true')
        args = parser.parse_args(argv)        
        if args.help:
            print ('avaa-demucs.py -i <input>')
            sys.exit(0)
            
        if args.version:
            print ('0.1'+str(args.version))
            sys.exit(0)
            
        if not args.input:
            print ('#E:missing input')
            sys.exit(4)           

        if not args.source: 
            args.source = 'vocals'            
        
    except Exception as ex:
        print ('#E:'+str(ex))
        print ('#E:invalid arguments?')
        print ('#E:Got '+ (' '.join(argv)))
        sys.exit(3)    
        
    target = args.input+"."+args.source+".wav" if not args.output else args.output;
        
    print ('#I:Args =', ' '.join(argv));
    print ('#I:Verbose =', 'yes' if args.verbose else 'no');
    print ('#F:Input =', args.input)
    #print ('#I:Model =', args.model)
    print ('#I:Source =', args.source)    
    #print ('#F:Output =', target)
    print ('#I:Loading model...')
    model = load_demucs_model()
    audio = load_track(args.input, model.audio_channels, model.samplerate)
    audio_dims = audio.dim()
    if audio_dims == 1:
        audio = audio[None, None].repeat_interleave(2, -2)
    else:
        if audio.shape[-2] == 1:
            audio = audio.repeat_interleave(2, -2)
        if audio_dims < 3:
            audio = audio[None]

    device = "cuda" if torch.cuda.is_available() else "cpu"
    print("#I:Demucs using device: "+device)
    
    if args.fake:        
        print ('#I:All good for preview mode!')
        print ('#W:For actual demucs separation, use "Convert" mode.')
        sys.exit(0)        
    
    print ('#I:Demucs processing...')

    bench = int(time.time());
    
    result = apply_model(model, audio, device=device, split=True, overlap=.25)
    
    if device != 'cpu':
        torch.cuda.empty_cache()
    
    for name in model.sources:
        if name==args.source: 
            source_idx=model.sources.index(name)
            source=result[0, source_idx].mean(0)            
            torchaudio.save(target, source[None], model.samplerate)
            print("#R:"+target)
    
if __name__ == "__main__":
    try:
        main(sys.argv[1:])
        sys.exit(0)
    except Exception as ex:        
        print("#E:"+str(ex))
        sys.exit(2)