# 2023-11-29
# python -u
# Silero VAD Wrapper for AVAA Toolkit by CG
# Credits:
#    - https://github.com/snakers4/silero-vad

import torch  
import os
import sys, getopt
import urllib.request 
import json
import time

def versiontuple(v):
    splitted = v.split('+')[0].split(".")
    version_list = []
    for i in splitted:
        try:
            version_list.append(int(i))
        except:
            version_list.append(0)
    return tuple(version_list)
    
def main(argv):
    inputFile = ''
    fake = False
    threshold=0.5
    minSilenceDuration=500
    minSpeechDuration=0
    samplingRate=16000
    
    try:
        opts, args = getopt.getopt(argv,"hi:t:s:e:r:vf",["input=","threshold=","silence=","speech=","rate=","version","fake"])
    except getopt.GetoptError:
        print ('#E:invalid arguments')
        print ('#E:Got '+ (' '.join(argv)))
        sys.exit(2)    
    for opt, arg in opts:
        if opt == '-h':
            print ('avaa-vad-silero.py -i <input>')
            sys.exit(0)
        elif opt in ("-i", "--input"):
            inputFile = arg
        elif opt in ("-t", "--threshold"):
            threshold = float(arg)
        elif opt in ("-s", "--silence"):
            minSilenceDuration = int(arg)            
        elif opt in ("-e", "--speech"):
            minSpeechDuration = int(arg)            
        elif opt in ("-r", "--rate"):
            samplingRate = int(arg)                 
        elif opt in ("--fake"):
            fake = True            
        elif opt in ("-v", "--version"):
            print("#I:0.1")
            sys.exit(0)
            
    if inputFile == '':
        print("#E:missing input file");
        sys.exit(2)
        
    print ('#I:Python =', sys.version);
    print ('#I:Torch =', torch.__version__);
    
    supported_version = '1.12.0'
    if versiontuple(torch.__version__) < versiontuple(supported_version):
        print(f'#E:Please install torch {supported_version} or greater ({torch.__version__} installed)')
        sys.exit(0)
    
    print ('#I:Args =', ' '.join(argv));
    print ('#F:Input =', inputFile)
    print ('#I:Threshold =', threshold)
    print ('#I:MinSilence =', str(minSilenceDuration)+' ms')
    print ('#I:MinSpeech =', str(minSpeechDuration)+' ms')
    print ('#I:Sampling =', str(samplingRate)+' Hz')
    print ('#I:Starting Silero...')
    from utils_vad import (init_jit_model,
                       get_speech_timestamps,
                       get_number_ts,
                       get_language,
                       get_language_and_group,
                       save_audio,
                       read_audio,
                       VADIterator,
                       collect_chunks,
                       drop_chunks,
                       Validator,
                       OnnxWrapper)
                       
    model_dir = os.path.dirname(__file__)
    model = init_jit_model(os.path.join(model_dir, 'silero_vad.jit'));
        
    if fake:        
        print ('#I:All good for preview mode!')
        print ('#W:For actual voice activity detection, use "Convert" mode.')
        sys.exit(0)        
    
    print ('#I:Reading audio file...')    
    wav = read_audio(inputFile, sampling_rate=samplingRate)    
    print ('#I:Detecting voice...')    
    speech_timestamps = get_speech_timestamps(wav, model,threshold=threshold,min_silence_duration_ms=minSilenceDuration, return_seconds=True, sampling_rate=samplingRate)        
    print('#R:'+json.dumps(speech_timestamps))
        
if __name__ == "__main__":
    
    try:
        main(sys.argv[1:])
        sys.exit(0)
    except Exception as ex:        
        print("#E:"+str(ex))
        sys.exit(2)