function ffmpegDurationToSeconds(s){
	if(!s) return 0;
	let a = s.trim().split(':');	
	if(a.length!==3) return 0;
	return a[0]*3600+a[1]*60+a[2]*1;
}
function secondsToDuration(s){
	return new Date(s * 1000).toISOString().slice(11, 19);
}
function ffmpegWrapper(step,ffargs,opt){			
	opt = opt || {};
	if(!opt.progress) opt.progress = (t,p)=>`Processing... ${t} - ${p}%`;
	if(!opt.download) opt.download = p=>`Downloading... ${p}`;
	if(!opt.trap) opt.trap = s=>false;	
	if(ffargs[0]!='ffmpeg'){
		ffargs = ['ffmpeg'].concat(ffargs);
	}
	let bench = Date.now();
	let aw = api.exec(ffargs);
	let progress = '';
	var inProgress;
	var gotError = false;
	var duration='';
	if(opt.expectedDuration) duration = secondsToDuration(Math.ceil(opt.expectedDuration/1000));
	var debugs = [];
	var lastchunk=0;
	var lastPercent=0;
	var completed; // needed for gyan build which outputs a last frame after the [out .....
	var lastFrame;
	var sentProgressOnce = false;
	var durationInSeconds= Math.round(opt.expectedDuration/1000) || 0;
	const isProgressLine = s => {
		return s.substr(0,6)=='frame=' ||
		(
			s.indexOf('time=')>-1 && s.indexOf('speed=')>-1
		)
	}
	aw.onError(s=>{	
		//step.info("onError:"+s)		
		if(!s) return;
		s = s.trim();
		if(!gotError){			
			gotError = s && s.substr(0,5).toLowerCase()=='error';
		}
		if(gotError){	
			
			if(s.substring(0,26)==`Error opening output file `){
				// generic ffmpeg error we don't need
			}
			else step.error(s);			
			if(debugs.length){
				debugs.forEach(s=>step.debug(s))
				debugs.length=0;
			}		
			return;
		}
		if(!duration && s.trim().substr(0,9).toLowerCase()=='duration:'){
			duration = s.trim().substr(10).trim().split(',')[0].split('.')[0];
			//TODO: duration:N/A 
			//step.warn('DURATION '+duration);
			durationInSeconds = ffmpegDurationToSeconds(duration);
			return;
		}
		if(isProgressLine(s)){
			if(completed){
				return;
			}
			let t = s.split('time=')[1].split(' ')[0].trim().split('.')[0].trim();			
			if(t[0]=='-') t = t.substring(1);
			
			let total = ffmpegDurationToSeconds(duration);
			let done = ffmpegDurationToSeconds(t);
			let percent = total<=0 || done<=0 ? 0 : Math.round((done/total)*100);
			let chunk = done - lastchunk;
			let percentChunk = percent - lastPercent;
			lastPercent = percent;
			lastchunk=done;
			lastFrame = {
				percent:percent,
				time:t,
				seconds:done,
				total:total,
				chunk:chunk*1000,
				percentChunk:percentChunk
			};
			if(opt.onProgress) opt.onProgress(lastFrame);			
			//if(opt.onProgressDone) opt.onProgresschunk*1000
			if(duration) t+=` / ${duration}`;			
			
			if(!inProgress) step.info(opt.progress(t,percent));
			else step.replace(opt.progress(t,percent));
			inProgress = true;
			sentProgressOnce = true;
			step.work();
			return;
		}		

		if(inProgress){
			if(s.trim().substr(0,7).toLowerCase()=='[parsed') return;			
			inProgress=false;
			completed=true;
			lastFrame.percentChunk = 100 - lastFrame.percent;
			lastFrame.chunk = (lastFrame.total - lastFrame.seconds)*1000;
			lastFrame.percent=100;
			lastFrame.time = lastFrame.total;
			if(opt.onProgress) opt.onProgress(lastFrame);
			step.replace(opt.progress(`${duration} / ${duration}`,100)); //^^
			step.ok();			
		}
		if(s[0]=='['){
			if(s.toLowerCase().indexOf('error')>-1 || s.toLowerCase().indexOf('could not')>-1){
				step.error(s);
			}
			if(!opt.trap(s)) debugs.push(s);
			return;
		}
		else{
			if(!opt.trap(s)) debugs.push(s);
			
		}
	})
	aw.onOutput(s=>aw.onError(s));
	step.monitor(aw);
	if(opt.env) for(var k in opt.env){
		debugs.push(`# set ${k}=${opt.env[k]}`);
		aw.env[k] = opt.env[k];
	}
	let exitStatus = aw.run();
	if(!exitStatus && opt.onProgress && !sentProgressOnce){
		opt.onProgress({
			percent:100,
			percentChunk:100,			
			chunk:durationInSeconds*1000,
			seconds:durationInSeconds		
		});
	}
	let benched = Math.ceil((Date.now()-bench)/1000)+1;
	if(!exitStatus){		
		step.info(`Completed in ${benched} seconds`);
		step.ok();
		return true;
	}else{
		step.error(`Process exited with error code ${exitStatus} after ${benched} seconds`);
		return false;
	}	
}
