function stepMock(){
	return {
		info:()=>{},
		error:()=>{},
		warn:()=>{},
		ok:()=>{},
		work:()=>{}
	}
}
function pythonCheck(step){
	step = step || stepMock();
	step.info(`Looking for Python...`)
	step.work()		

	let pyw = api.exec(['python','--version'])
	let pyv = ''
	pyw.onOutput(s=>{
		pyv = s.split(' ')[1]
	})
	pyw.onError(s=>{
		step.warn(s);
	});
	let pyx = pyw.run();
	if(pyx){
		step.error(`Python could not be started (exit code ${pyx}), is it properly installed?`);
		return;
	}
	if(!pyv){
		step.error(`Python version could not be determined (got "${pyv}")`);
		return;
	}
	step.append(" v"+pyv);
	step.ok();
	return pyv;
}
var pipCache = {};
function pipCheck(moduleName,step,giveup){		
	step = step || stepMock();
	step.info(`Looking for module ${moduleName}...`)
	
	if(pipCache[moduleName]){
		step.append(" v"+pipCache[moduleName]);
		step.ok();
		return pipCache[moduleName];
	}
	step.work()			
	
	let pyw = api.exec(['pip','list'])
	let pyv = '';
	pyw.onOutput(s=>{
		let a = s.split(' ');
		let v = a.pop();
		let mn = a[0];
		pipCache[mn]=v;
		if(mn==moduleName) pyv = v;		
	})
	pyw.onError(s=>{
		if(!s) return;
		if(s.substr(0,8)=='[notice]'){
			if(s.indexOf('new release of pip is available')>-1) return;
			if(s.indexOf('-m pip install --upgrade pip')>-1) return;
		}
		step.warn(s);
	});
	let pyax = pyw.run();
	if(pyax){
		step.error(`"pip list" failed (exit code ${pyax}), is pip properly installed?`);
		return;
	}
	if(!pyv){
		if(giveup){
			step.error(`${moduleName} was not found and can't be installed by AVAA.`);
			step.info(`Install it manually by running this command in a terminal: "pip install ${moduleName}"`);
			return;
		}else{
			step.warn(`${moduleName} was not found, attempting installation...`);
			step.work();
			let installLogs = pipInstallModule(moduleName,step);
			pyv = pipCheck(moduleName,step,true);			
			if(!pyv) installLogs.forEach(s=>step.warn(s));							
			return pyv;
		}
	}
	step.append(" v"+pyv);
	step.ok();
	return pyv;
}
function pipInstallModule(moduleName){
	let pyw = api.exec(['pip','install',moduleName]);
	let pyv = '';
	let logs = [];
	pyw.onOutput(s=>{
		if(!s) return;
		logs.push(s);
	})
	pyw.onError(s=>{
		if(!s) return;
		if(s.substr(0,8)=='[notice]'){
			if(s.indexOf('new release of pip is available')>-1) return;
			if(s.indexOf('-m pip install --upgrade pip')>-1) return;
		}
		logs.push(s);
	});
	let pyax = pyw.run();	
	return logs;
}

function pythonWrapper(name,step){
	step = step || stepMock();
	step.info(`Looking for ${name}...`)
	let awf = api.doc.atk.getDependencyFile(name+'.py');
	if(!awf.exists()){
		step.error(`Could not find ${name} at ${awf.getAbsolutePath()}`);
		return;
	}
	step.ok();
	step.info(`Checking ${name}...`)
	step.work()
	let aw = api.exec(['python',awf.getAbsolutePath(),'-v'])
	let r = aw.run();
	if(r>0){
		step.error(`${name} is not working properly (returned ${r})`);
		return;
	}	
	step.ok();
	return awf;
}