const TeiCorpo = {
	check:(log)=>{
		if(TeiCorpo.bridge) return true;		
		let path = api.atk.path;
		log.info(`Checking TeiCorpo availability... `);
		let filter = new org.apache.commons.io.filefilter.WildcardFileFilter("teicorpo*.jar", org.apache.commons.io.IOCase.INSENSITIVE);
        let list = (new java.io.File(path))['listFiles(java.io.FilenameFilter)'](filter);
		if(!list.length){
			log.error(`Can't find TeiCorpo library`);	
			log.warn(`Visit https://github.com/christopheparisse/teicorpo/blob/master/target/`)
			log.warn(`Download the teicorpo-x-SNAPSHOT.jar file and put it in AVAA's folder`);
			throw "TeiCorpo library not found";
		}else{
			log.append(`${list[0].getName()}`)
			log.ok();
		}
		let jar = list[0];
		let urls = java.lang.reflect.Array.newInstance(java.net.URL, 1);
		urls[0] = jar.toURL();
		child = new java.net.URLClassLoader(urls);
		let clanToTei = child.loadClass("fr.ortolang.teicorpo.ClanToTei");
		let praatToTei = child.loadClass("fr.ortolang.teicorpo.PraatToTei");
		let teiToElan = child.loadClass("fr.ortolang.teicorpo.TeiToElan");
		TeiCorpo.bridge = {
			clan:clanToTei,
			praat:praatToTei,
			tei:teiToElan
		}
		Object.keys(TeiCorpo.bridge).forEach(k=>{			
			let main = TeiCorpo.bridge[k].getMethods().filter(m=>{
				return m.getName()=="main";
			})[0];
			if(!main){
				log.error(`Can't find main() entry point in TeiCorpo jar file`);
				throw "invalid jar";
			}
			//log.debug(`Handling ${k} ${main}`)			
			TeiCorpo.bridge[k] = (a)=>{
				//log.debug(`Calling bridge ${k}.main() ${a}`);
				let args = java.lang.reflect.Array.newInstance(java.lang.String, a.length);
				for(let i=0;i<a.length;i++) args[i]=a[i];									
				let vargs = java.lang.reflect.Array.newInstance(java.lang.Object, 1);
				vargs[0]=args;				
				main.invoke(null,vargs);				
			}
		});
		return true;		
	},
	toEAF:(file,logger)=>{
		let log = logger||log;
		if(!TeiCorpo.check(log)){		
			log.error(`TeiCorpo is not installed`);
			throw "missing teicorpo JAR file";
		}
		let console = new org.avaatoolkit.util.Console();
		let ext = org.apache.commons.io.FilenameUtils.getExtension(file.getName()).toLowerCase();
		let path = org.apache.commons.io.FilenameUtils.separatorsToUnix(file.getAbsolutePath());
        let a = [path,"-o",path+ext];
        console.capture();
		switch (ext) {
			case "clan":
			case "cha":
				TeiCorpo.bridge.clan(a);
				break;
			case "textgrid":
			case "praat":
				TeiCorpo.bridge.praat(a);				
				break;
			case "tei":
				// no conv needed
				break;
			default:
				throw "Unhandled TeiCorpo format: " + ext;
		}
        let b = [path+ext,"-o",path+".eaf"];
        TeiCorpo.bridge.tei(b);
        console.release();
        //log.debug(`TeiCorpo Output: ${console.content.size()} bytes`);
		//log.debug(`${console.content}`);
		//TODO export console
        return new java.io.File(file.getAbsolutePath()+".eaf");
	}
}