/** 
@author CG
@log 2023-11-13 initial implementation
Transforms the page into a dynamic slideshow
**/
mod.slideshow = {
	hook:(event)=>{
		switch(event){
			case 'body':
				return `<div id="slides-container">`;
			case '/body':
				return `</div>`;
			case '/head':				
				return `
				<style>${mod.slideshow.css()}</style>
				<script>(${mod.slideshow.jstuff.toString().trim()})()</script>
				`
			case 'head':
				return '<!-- slideshow v0.1 -->';
		}
	},
	jstuff:function mod_slideshow(){		
		$(()=>{					
		gsap.registerPlugin(ScrollToPlugin)
		gsap.registerPlugin(CSSPlugin)
		var $window = $(window);
		var $document = $(document);
	//Only links that starts with #
	var $navButtons = $("nav a");//.filter("[href^=#]");
	var $navGoPrev = $(".go-prev");
	var $navGoNext = $(".go-next");
	var $slidesContainer = $("#slides-container");
	$('#slides-container > h1').wrap('<section></section>');
	var $slides = $("section");
	var $currentSlide = $slides.first();	
	var isAnimating = false;
	var pageHeight = $window.innerHeight();
	var keyCodes = {
		UP  : 38,
		DOWN: 40
	}	
	console.info($slides,$currentSlide);
	goToSlide($currentSlide);
	$window.on("resize", onResize).resize();
	$window.on("mousewheel DOMMouseScroll", onMouseWheel);
	$document.on("keydown", onKeyDown);
	$navButtons.on("click", onNavButtonClick);
	$navGoPrev.on("click", goToPrevSlide);
	$navGoNext.on("click", goToNextSlide);
	var pd = e => {
		try {
			e.preventDefault();
		}catch(ex){}
	}
	function onNavButtonClick(event)
	{	
		//The slide the button points to
		var $slide = $($(this).attr("href"));
		//If the slide exists, we go to it
		if($slide.length)
		{
			goToSlide($slide);
			pd(event);
		}
	}

	/*
	*   Getting the pressed key. Only if it's up or down arrow, we go to prev or next slide and prevent default behaviour
	*   This way, if there's text input, the user is still able to fill it
	* */
	function onKeyDown(event)
	{

		var PRESSED_KEY = event.keyCode;

		if(PRESSED_KEY == keyCodes.UP)
		{
			goToPrevSlide();
			pd(event);
		}
		else if(PRESSED_KEY == keyCodes.DOWN)
		{
			goToNextSlide();
			pd(event);
		}

	}

	/*
	*   When user scrolls with the mouse, we have to change slides
	* */
	function onMouseWheel(event)
	{
		//Normalize event wheel delta
		var delta = event.originalEvent.wheelDelta / 30 || -event.originalEvent.detail;

		//If the user scrolled up, it goes to previous slide, otherwise - to next slide
		if(delta < -1)
		{
			goToNextSlide();
		}
		else if(delta > 1)
		{
			goToPrevSlide();
		}

		pd(event);
	}

	/*
	*   If there's a previous slide, slide to it
	* */
	function goToPrevSlide()
	{
		if($currentSlide.prev().length)
		{
			goToSlide($currentSlide.prev());
		}
	}

	/*
	*   If there's a next slide, slide to it
	* */
	function goToNextSlide()
	{
		if($currentSlide.next().length)
		{
			goToSlide($currentSlide.next());
		}
	}

	/*
	*   Actual transition between slides
	* */
	function goToSlide($slide)
	{
		//If the slides are not changing and there's such a slide
		if(!isAnimating && $slide.length)
		{
			//setting animating flag to true
			isAnimating = true;
			$currentSlide = $slide;

			//Sliding to current slide
			let index =  $slides.toArray().indexOf($currentSlide[0]);
			//console.info('INDEX',index);			
			TweenLite.to($slidesContainer, 1, {scrollTo: {y: pageHeight * index }, ease: "out" , onComplete: onSlideChangeEnd});

			//Animating menu items
			//TweenLite.to($navButtons.filter(".active"), 0.5, {className: "-=active"});

			//TweenLite.to($navButtons.filter("[href=#" + $currentSlide.attr("id") + "]"), 0.5, {className: "+=active"});

		}
	}

	/*
	*   Once the sliding is finished, we need to restore "isAnimating" flag.
	*   You can also do other things in this function, such as changing page title
	* */
	function onSlideChangeEnd()
	{
		isAnimating = false;
	}

	/*
	*   When user resize it's browser we need to know the new height, so we can properly align the current slide
	* */
	function onResize(event)
	{

		//This will give us the new height of the window
		var newPageHeight = $window.innerHeight();

		/*
		*   If the new height is different from the old height ( the browser is resized vertically ), the slides are resized
		* */
		if(pageHeight !== newPageHeight)
		{
			pageHeight = newPageHeight;

			//This can be done via CSS only, but fails into some old browsers, so I prefer to set height via JS
			TweenLite.set([$slidesContainer, $slides], {height: pageHeight + "px"});

			//The current slide should be always on the top
			TweenLite.set($slidesContainer, {scrollTo: {y: pageHeight * $slides.toArray().indexOf($currentSlide[0]) }});
		}

	}
	})
	},
	css:function(){
		return `		
		#slides-container > h42,
		#slides-container > toc {
			display:none
		}
		
		
		body, div, p {
			margin: 0;
			padding: 0;
		}
		#slides-container {
			position: absolute;
			left: 0;
			top: 0;
			width: 100%;
			height: 100%;
			overflow: hidden;
			z-index: 10;
		}
		#slides-container .view-table > table {
			margin:auto;
		}
		#slides-container section:first-child h1 {
		    padding-top: 35vh;
			font-size: 3em;			
		}
		#slides-container > h1,
		#slides-container section {
			text-align:center;		
			position: relative;
			width: 100%;
			height: 100%;
			overflow: hidden;			
		}
		#slides-container .chart-legend-swatches > div {
			display: inline-flex !important;
		}
		`;
	}
}