/** 
@author CG
@experimental
@input Array<Annotation>
@attr group bool (false) Whether to separate annotations based on their group instead of their file
@attr sequence string Separate annotations based on another tier's annotations (used as sequences)
@attr time-offset bool (false) Whether to use files' time-offset
@attr time-between bool (false) Whether to show the time between separators
@attr time-collapse-threshold number (0) Number of seconds after which empty space is collapsed (0 = no collpase)
@attr time-collapse-display string (show-duration) [show-duration,compact,minimum,none] How the collapsed zone should look
@attr time-collapse-alt-display string (compact) [show-duration,compact,minimum,none] How the collapsed zone should look if it collapsed less than alt-display-threshold
@attr time-collapse-alt-display-threshold number (0) Number of seconds collapsed before which alt-display is used
@attr time-collapse-hide-first-start bool (false) Whether to hide the first (starting) collapse, if any
@attr time-collapse-hide-each-start bool (false) Whether to hide the first (starting) collapse of each group/column, if any
@attr time-collapse-hide-borders bool (false) Whether to hide the left and right borders of collapsed zones
@attr time-collapse-absolute-start bool (false) Whether to show the duration of the first collapse of each group, as an absolute duration (relative to corpus beginning when time-offset is active)
@todo @attr time-collapse-display-unit string [auto,seconds,minutes,hours,days] (auto) Unit for displaying time collapsed

@attr zoom-factor int (100) Milliseconds to pixel factor (how many milliseconds one pixel will represent)
@deprec @attr colors string A comma separated list of default colors, like "#c7f464,#ffa822"
@attr color-values bool (false) Whether to use CSS for specific values, colors defined in Style or as parameters
@attr color-values-opacity bool (false) Whether to toggle the opacity of the colored values
@deprec @attr show-alt-tiers bool (false) Whether to display alternative tiers in the main table
@attr show-alt-tiers-in-table bool (false) Whether to display alternative tiers in the main table
@attr show-alt-tier-in-empty-cells bool (false) Whether to display the alternative tier if the cell is empty
@attr show-value-on-hover bool (false) Whether to display the annotation value on mouse over
@attr show-snap-on-hover bool (false) Whether to display the media snapshot on mouse over
@attr show-media-on-click bool (false) Whether to display the media player on click
@attr @deprecated show-video-on-click bool (false) Whether to display the video player on click
@attr video-player-width string (300)
@attr video-player-height string ()
@param tier="id" name="" parent="" color="" alt-tier="" | Customize tier display
@param group="id" name="Custom Name" | Set a custom header name for each group
@param file="id" name="Custom Name" | Set a custom header name for each file
@param alt-tier="id" label="" css="" | Customize alternative tier display and legend
@param value="value" label="" color="" css="" | Customize specific value display and legend
@param !name
@param !label
@param !parent
@param !color
@param !css
@param !alt-tier
@log 2023-03-05 initial implementation
@log 2023-06-13 added customizable tier parent and color
@log 2023-08-28 rewrite for time collapse

The density view plots annotations as filled bars in a timeline, revealing interactions frequency and duration between tiers.
Density2 is a work in progress to handle time-collapse. Currently not compatible with alt-tiers.

@sample VIEW zoom-factor="500"
@image png
**/
view.density2 = {
	list:list=>{		
		var colors = ['#556270', '#4ecdc4', '#c7f464', '#ff6b6b', '#c44d58','#ffa822', '#134e6f', '#ff6150', '#1ac0c6', '#dee0e6'];
		var factor = attr['zoom-factor']*1 || attr.factor*1 || 100;
		if(attr.colors){			
			try {
				colors = JSON.parse(attr.colors.charAt(0)=='[' ? attr.colors : "['"+attr.colors.split(',').join("','")+"']");				
			}catch(ex){
				log.error('failed to parse `colors` attribute');
				log.error(c);
				log.error(ex.message);
			}
		}
		var addpx = (v,d) => v && (v*1===v*1) ? v+'px' : (d ? d+'px' : '');
		var defaultNoDataCSS = 'background:repeating-linear-gradient( 45deg, #000000, #000000 3px, #ffffff00 3px, #ffffff00 6px)';
		var color = 'black';		
		var group = !attr.group ? false : (attr.group == 'true'); 
		var showValueOnHover = !attr['show-value-on-hover'] ? false : (attr['show-value-on-hover'] == 'true'); 
		var showSnapOnHover = !attr['show-snap-on-hover'] ? false : (attr['show-snap-on-hover'] == 'true'); 
		var showVideoOnClick = !attr['show-video-on-click'] ? false : (attr['show-video-on-click'] == 'true'); 
		if(attr['show-video-on-click']) log.warn('show-video-on-click is deprecated, please use show-media-on-click');
		var showMediaOnClick = showVideoOnClick || !attr['show-media-on-click'] ? false : (attr['show-media-on-click'] == 'true'); 
		var showAltTiers = !attr['show-alt-tiers'] ? false : (attr['show-alt-tiers'] == 'true'); 
		if(attr['show-alt-tiers-in-table']=='true') showAltTiers=true;
		var showAltTierInEmptyCell = !attr['show-alt-tier-in-empty-cells'] ? false : (attr['show-alt-tier-in-empty-cells'] == 'true'); 
		var videoPlayerWidth = addpx(attr['video-player-width'],300);
		var videoPlayerHeight = addpx(attr['video-player-height']);
		var colorValues = attr['color-values'] == 'true';
		var colorValuesOpacity = attr['color-values-opacity'] == 'true';
		const timeOffset = attr['time-offset'] == 'true';
		const timeBetween = attr['time-between'] == 'true';
		var timeCollapseThreshold = attr['time-collapse-threshold']*1 || 0;
		var timeCollapseHideStart = attr['time-collapse-hide-start'] == 'true';
		var timeCollapseHideFirstStart = attr['time-collapse-hide-first-start'] == 'true';
		var timeCollapseHideEachStart = attr['time-collapse-hide-each-start'] == 'true';
		var timeCollapseAbsoluteStart = attr['time-collapse-absolute-start'] == 'true';
		var timeCollapseHideBorders = attr['time-collapse-hide-borders'] == 'true';
		var timeCollapseHideDuration = attr['time-collapse-hide-duration'] == 'true';
		var timeCollapseDisplay = attr['time-collapse-display'] || 'show-duration';
		var timeCollapseAltDisplay = attr['time-collapse-alt-display'] || 'show-duration';
		var timeCollapseAltDisplayThreshold = attr['time-collapse-alt-display-threshold']*1 || 0;
		var sequence = attr.sequence ? attr.sequence : '';		
		var tierNames = {}; // map of custom names from options
		var groupNames = {};
		var fileNames = {};
		var asort = (A,B)=>{ 
			let startA =  A.start;
			let startB = B.start;
			//return startA > startB ? 1 : -1;
			if(timeOffset){				
				startA += A.af.timeOffset;
				startB += B.af.timeOffset;
			}
			return startA > startB ? 1 : -1;
		};
		var tiers = {}; // map of tier id => tier's annotations			
		var files = {}; // map of file name => file's annotations
		var groups = {}; // ... 
		var sequences = {}; // ...
		var tierParents = {};
		var tierColors = {};
		var valueStyle = {};	
		var valueLegend = {};
		var tierAlt = {};
		var altTiers = {};
				
		params.forEach(opt=>{ 
			if(opt.hasAttribute('tier')){
				var tierid = opt.getAttribute('tier');
				tierNames[tierid] = opt.getAttribute('name');
				tiers[tierid] = []; // force order
				if(opt.hasAttribute('parent') && opt.getAttribute('parent')){
					tierParents[tierid] = opt.getAttribute('parent');
				}
				if(opt.hasAttribute('alt-tier') && opt.getAttribute('alt-tier')){
					tierAlt[tierid] = opt.getAttribute('alt-tier');
				}
				if(opt.hasAttribute('color') && opt.getAttribute('color')){
					tierColors[tierid] = opt.getAttribute('color');
				}
				if(!tierColors[tierid]){ // let's try loading from global colors
					let c = api.doc.style.colorOfTier(tierid);
					if(c) tierColors[tierid] = c;
				}
				return;
			}
			if(opt.hasAttribute('alt-tier') && opt.getAttribute('alt-tier')){
				var tierid = opt.getAttribute('alt-tier');
				var o = altTiers[tierid] = {id:tierid};
				if(opt.hasAttribute('color') && opt.getAttribute('color')) o.color = opt.getAttribute('color');
				if(opt.hasAttribute('css') && opt.getAttribute('css')) o.css = opt.getAttribute('css');
				if(opt.hasAttribute('name') && opt.getAttribute('name')) o.name = opt.getAttribute('name');
				if(opt.hasAttribute('label') && opt.getAttribute('label')) o.label = opt.getAttribute('label');
				return;
			}
			if(opt.hasAttribute('value') && opt.getAttribute('value')){
				let value = opt.getAttribute('value');
				let o = valueStyle[value] = {value:value};
				if(opt.hasAttribute('color') && opt.getAttribute('color')) o.color = opt.getAttribute('color');
				if(opt.hasAttribute('css') && opt.getAttribute('css')) o.css = opt.getAttribute('css');				
				if(opt.hasAttribute('label') && opt.getAttribute('label')) o.label = opt.getAttribute('label');
				return;
			}
			if(opt.hasAttribute('group')){
				groupNames[opt.getAttribute('group')+''] = opt.getAttribute("name")+'';
			}
			if(opt.hasAttribute('file')){
				fileNames[opt.getAttribute('file')+''] = opt.getAttribute("name")+'';
			}
		});
		
		if(sequence) group = false; // AF based		
		if(timeOffset){ // we must sort the files accordingly:
			let allFiles = [];
			for(let a of list) if(allFiles.indexOf(a.af)==-1) allFiles.push(a.af);			
			allFiles.sort((A,B)=> A.timeOffset > B.timeOffset ? 1 : -1);			
			for(let af of allFiles){
				//log.info()
				files[af.id] = [];
				if(!groups[af.group]) groups[af.group] = [];
			}
		}
		
		
		list.forEach(a=>{
			if(!tiers[a.tier.id]) tiers[a.tier.id] = [];
			tiers[a.tier.id].push(a)
			if(!tiers[a.tier.id].color) tiers[a.tier.id].color = tierColors[a.tier.id] ? tierColors[a.tier.id] : ( api.doc.style.colorOfTier(a.tier.id) ? api.doc.style.colorOfTier(a.tier.id) : (colors.length ? colors.shift() : 'black'));
			if(!files[a.af.id]) files[a.af.id] = [];			
			var	f = files[a.af.id];			
			f.push(a);
			if(f.start===undefined || a.start<f.start) f.start = a.start;
			if(!groups[a.af.group]) groups[a.af.group] = [];
			groups[a.af.group].push(a);

			if(sequence){
				a.af.getTier(sequence).annotations.forEach(seq=>{
					if(a.start>=seq.start && a.stop<=seq.stop){
						var sid = a.af.id+'/'+seq.value;
						if(!sequences[sid]){
							var s = sequences[sid] = [];
							s.id=sid;
							s.name = seq.value+'';
							s.af = seq.af;
							s.start = seq.start;
							s.stop = seq.stop;							
						}
						sequences[sid].push(a);
					}
				});
			}
		});
		var prev;		
		var aset = group ? groups : files;
		//log.info(Object.keys(aset));
		if(sequence) {
			aset = {};
			var a = [];
			for(var k in sequences) a.push(sequences[k]);
			a.sort(
				(a,b)=>{ return a.af.filename+"" == b.af.filename+"" ?  (a.start > b.start ? 1 : -1) : (a.af.filename+"" < b.af.filename+"" ? -1 : 1)}
			);
			a.forEach(seq=>aset[seq.id]=seq);
		}
		var names = group ? groupNames : fileNames;
		
		var hasParents = Object.keys(tierParents).length > 0;
		
		var didParent = {};
		var countParent = {};
		if(hasParents) for(tier in tiers){
			var par =  tierParents[tier] || '';
			if(!countParent[par]) countParent[par]=1;
			else countParent[par]++;
			// important for correct rowspan:
			if(!showAltTiers && altTiers[tier]) countParent[par]--;
		}
		
		
		
		//var isNoData = {}
		//for(var k in tierAlt){
		//	isNoData[tierAlt[k]] = true;
		//}
		
		var tree = {};
		
		var lastOffsets = {};	
		for(tier in tiers){
			if(!tree[tier]) tree[tier] = {id:tier,alist:{},alt:altTiers[tier]};
			var T = tree[tier];
			//if(isNoData[tier] && !showAltTiers) continue;//?
						
			for(key in aset){				
				if(!T.alist[key]) T.alist[key] = [];
				var finalw = 0;
				var last = 0;
				var opacity = 1;
				aset[key].sort(asort);				
				aset[key].forEach(a=>{
					if(a.tier.id+'' != tier) return;
					if(prev == a) return; // duplicates from possible OPerations???
					//log.info(a.id);
					prev = a;										
					var start = a.start - (aset[key].start || 0);
					var stop = a.stop - (aset[key].start || 0);
					if(timeOffset){
						//log.info('+'+a.af.timeOffset);
						start += a.af.timeOffset;
						stop += a.af.timeOffset;
						
						if(stop > (lastOffsets[key]||0)) lastOffsets[key] = stop;
					}
					var dur = stop-start;
					var F = Math.ceil;
					var w = F(dur/factor);
					var left = F(start/factor);
					if(isNaN(left)){
						log.error(aset[key].start);
						throw "game over.";
					}
						
					var blank = 0;
					if(last>0){
						blank = F((start-last)/factor);
						//T.alist[key].push({blank:true,width:b});//s+=`<div class="blank" style="width:${b}px"></div>`;
					
					}else if(start>0){
						blank = F((start)/factor);
						//T.alist[key].push({blank:true,width:b});//s+=`<div class="blank" style="width:${b}px"></div>`;
					}
					/*var b64v="";
					try {
						b64v = atob(esc(String(a.value)));
					}catch(ex){
						log.warn("failed b64 on "+a.value)
					}*/
					T.alist[key].push({annotation:a,width:w*1,opacity:opacity,left:left, blank:blank});
					
					last = stop;
					opacity = opacity == 1 ? 0.8 : 1;
					finalw = left + w;
				});
				
				//TODO:
				if(false){//tierAlt[tier]) {
					if((showAltTierInEmptyCell && !finalw) || finalw){
					//s+=`<i>${tierAlt[tier]}</i>`;
						s+=`<div class="alt-tier-nooverflow" sstyle="width:${finalw}px"><div>`;
						drawNoData(tierAlt[tier],aset[key]);
						s+='</div></div>'
					}
				}
				
				
			}				
		}
		// collapsing: #####################################################################
		if(timeCollapseThreshold>0){
		const threshold = Math.round((timeCollapseThreshold*1000/factor));
		const widths = {compact:30,minimum:10,none:1}		
		const getCollapseWidth = display => widths[display] || 100;		
		const compactAll = (list,index,width,pad)=>{
			for(let i = index ; i < list.length ; i++){
				let o = list[i];
				o.left -= width;				
				if(i==index) o.blank -= width;
				if(pad){
					o.left += pad;
					if(i==index) o.blank += pad;
				}
			}
		}
		const parseValue = milliseconds =>{
			let o = {};
			let s = Math.floor(milliseconds/1000);
			o.msec = milliseconds - (s*1000);
			o.days = Math.floor(s / (3600*24));
			s %= (3600*24);
			o.hours = Math.floor(s / 3600);
			s %= 3600;
			o.minutes = Math.floor(s / 60);
			o.seconds = s % 60;
			return o;
		}
		const valueToName = (v,first) => {
			let o = parseValue(v);
			let s = '';
			if(first && o.days>800) return `${Math.round(o.days/365)} years`;
			if(first && o.days>100) return `${Math.round(o.days/31)} months`;
			if(o.days) s += `${o.days} day${o.days>1?'s':''} `;
			if(o.days && first) return s + (o.hours ? '' : '');
			if(o.hours) s += `${o.hours} hour${o.hours>1?'s':''} `;
			if(o.hours && first) return s + (o.minutes ? '' : '');
			if(o.minutes) s += `${o.minutes} min`;
			if(o.minutes && first) return s + (o.seconds ? '' : '');
			if(o.seconds) s += `${o.seconds} sec`;  
			if(!o.seconds && o.msec && first) return `${o.msec} ms`
			return s;
		}
		const addCollapse = (key,o,offset)=>{			
			o.label = valueToName((o.size*factor)-(offset||0),true);			
			if(!aset[key].collapses) aset[key].collapses = [];
			aset[key].collapses.push(o);
		}		
		var firstOfAll = true;
		var lastKey;
		var currentKey;
		
		for(key in aset){ // for each "group" (column)						
			if(currentKey!=key){
				lastKey = currentKey
				currentKey = key;
			}
			var min = Number.MAX_SAFE_INTEGER; 
			var nextOf = {};
			var steps = 0;
			var frontier = [];
			var currentX = 0;
			var activeTiers = [];
			var lasts = [];
			var firstOfGroup = true;
			for(tier in tree){ // for each tier
				let T = tree[tier];
				//if(T.noData) continue;
				let L = T.alist[key];				
				if(!L.length) continue;
				L.index = 0;
				L.forEach(e=>e.list=L);
				if(activeTiers.indexOf(tier)<0) activeTiers.push(tier);
				let i = L.index = 0;
				frontier.push(L[i]);
			}
			//log.info('INITIAL FRONTIER ='+frontier.length);			
			while(true){	
				//frontier.forEach(e=>e.candidate=true)
				//if(steps>100) break;			
				if(steps++>100000) throw "steps exceeded"		
				if(!frontier.length){
					//log.info(key+' in '+steps+" steps");
					break;
				}				
				// LET'S FIND THE MOST-LEFT ELEMENT
				var mostLeft = Number.MAX_SAFE_INTEGER;
				var mostLeftElement = null;
				frontier.forEach(e=>{
					if(mostLeft>e.left){
						mostLeft = e.left;
						mostLeftElement = e;
					}
				});
				// DISTANCE BETWEEN MOST-LEFT ELEMENT AND CURRENT X
				let distance = mostLeft - currentX;
				//if(distance<0) throw "negative distance of "+distance;				
				if(distance>threshold){
					//log.info('COMPACTING '+distance)
					let skip = (firstOfAll && currentX==0 && timeCollapseHideFirstStart) || (firstOfGroup && currentX==0 && timeCollapseHideEachStart);
					let firstInside = firstOfGroup && !firstOfAll;
					firstOfAll = false;
					firstOfGroup = false;
					let display = timeCollapseAltDisplayThreshold > distance*factor / 1000 ?  timeCollapseAltDisplay : timeCollapseDisplay;
					frontier.forEach(fe=>{
						
						compactAll(fe.list,fe.list.indexOf(fe),distance,skip ? 0 : getCollapseWidth(display));
					})
					//lastOffsets
					let offset = 0;
					if(firstInside && !timeCollapseAbsoluteStart) {
						if(!lastKey) throw "no lastKey!?!?! can't be"
						offset =  lastOffsets[lastKey];
					}
					
					if(!skip) addCollapse(key, {size:distance,x:mostLeft-distance, display:display}, offset);
					continue;
				}else{
					//log.info('currentX='+mostLeft);
					// MOVE THE CURRENT X
					currentX = mostLeft + mostLeftElement.width;					
					// NEXT ELEMENT IN MOST-LEFT'S LIST
					mostLeftElement.candidate = false;
					frontier.splice(frontier.indexOf(mostLeftElement),1);
					//mostLeftElement.candidate=true;
					let L = mostLeftElement.list;
					if(L.index == L.length-1){					
						lasts.push(mostLeftElement);
						continue;
					}
					L.index++;
					frontier.push(L[L.index]);
					continue;
					
				}
				
			}
		}	
		} // # END OF COLLAPSING SCRIPT
			
		var s = `<table class="density-table ${timeCollapseHideBorders ? 'hide-borders' : ''} ${timeCollapseThreshold ? 'collapse' : ''}"><thead><tr>`;
		
		
		function drawNoData(tier,alist){
			if(!alist.length) return;
			//alist = 			
			var last = 0;
			var opacity = 1;			
			//alist.sort(asort);		
			var toDraw = [];			
			alist.map(o=>o.annotation).forEach(a=>{				
				if(a.tier.id+'' != tier) return;
				if(!toDraw.length){
					toDraw.push(a.clone());
					return;
				}
				var lastA = toDraw[toDraw.length-1];							
				
				if(lastA.stop==a.start){
					//log.info('COMBINE at '+a.start);
					lastA.stop = a.stop;
					return;
				}
				else{
					var diff = a.start-lastA.stop;
					if(diff<factor){
						//log.info('COMBINE SMALL DIFF '+diff);
						lastA.stop = a.stop;
						return;
					}
					
				}
				toDraw.push(a.clone());
			});
			if(!toDraw.length) return;
			log.info('toDraw'+toDraw.length)
			toDraw.forEach(a=>{
				var start = a.start - alist.start;
				var stop = a.stop - alist.start;				
				var dur = stop-start;
				var F = Math.ceil;
				var w = F(dur/factor);
				var left = F(start/factor);
				if(last>0){
					var b = F((start-last)/factor);
					s+=`<div class="blank" style="width:${b}px"></div>`;
				}else if(start>0){
					var b = F((start)/factor);
					s+=`<div class="blank" style="width:${b}px"></div>`;
				}
								
				var css = altTiers[tier] && altTiers[tier].css ? altTiers[tier].css : defaultNoDataCSS;
				
				s+=`<div class="annotation-pad" title="${esc(a.value)}" style="width:${w}px;background-color:${tiers[a.tier.id+''].color||'red'};"></div>`;
				s+=`<div class="annotation stripes" title="${esc(a.value)}" style="opacity:${opacity};left:${left}px;width:${w}px;${css}"></div>`;
				last = stop;				
			});
		}
		
		
		
		
		
		
		
		
		if(hasParents) s+= `<td class="corner" colspan="2"></td>`;
		else s+= `<td class="corner"></td>`;
		for(key in aset){
			var kname = sequence ? aset[key].name : names[key] || key;
			s += `<td class="af-separator sequence-separator"><span>${esc(kname)}</span>`;
			if(aset[key].collapses) aset[key].collapses.forEach(c=>{
				var seconds = (c.size * factor) * 1000;
				var tcdisplay = c.display;//timeCollapseDisplay ? 'collapse-'+timeCollapseDisplay:''}				
				s += `<div class="time-collapse collapse-${tcdisplay}" style="left:${c.x}px;width:${getCollapseWidth(tcdisplay)}px">`;
				
				
				
				
				s += `<b>${tcdisplay == 'show-duration' ? c.label : ''}</b>`;
				s += `</div>`;
			});
			s += `</td>`;			
		}
		
		s += `</tr></thead>`;
		
		s += `<tbody>`;	
		for(tier in tree){
			var T = tree[tier];
			//if(isNoData[tier] && !showAltTiers) continue;
			if(T.alt && !showAltTiers) continue;
			
			var tname = tierNames[tier] || api.doc.style.labelOfTier(tier) || tier;
			s += `<tr class="tier">`;
			if(hasParents){
				var par =  tierParents[tier] || '';
				if(!didParent[par]){
					didParent[par]=true;
					s += `<td rowspan="${countParent[par]}" class="parent-name">${esc(par)}</td>`;
				}
					
			}
			s += `<td class="tier-name"><span>${esc(tname)}</span></td>`;
			
			//s += `</td>`;
			for(key in T.alist){
				s += `<td class="tier-density ${showMediaOnClick ? 'clickable' : ''}">`;			
				T.alist[key].forEach(i=>{
					
					/*if(i.collapsed){
						s+=`<div class="time-collapse" style="width:${i.width}px;"></div>`;
					}
					else */
					
					if(i.annotation){
						if(!i.blank) i.blank=0;
						s+=`<div class="blank" style="width:${i.blank}px"></div>`;
						
						let a = i.annotation;
						let color = tiers[a.tier.id+''].color||'red';
						if(colorValues && valueStyle[a.value] && valueStyle[a.value].color){
							color = valueStyle[a.value].color;
						}else if(colorValues && api.doc.style.colorOfValue(a.value)){
							color = api.doc.style.colorOfValue(a.value);
						}
						let valueCSS = '';
						if(colorValues && valueStyle[a.value] && valueStyle[a.value].css) valueCSS = valueStyle[a.value].css;
						if(colorValues && api.doc.style.labelOfValue(a.value)){
							valueLegend[a.value] = {
								color:api.doc.style.colorOfValue(a.value),
								label:api.doc.style.labelOfValue(a.value)
							}
						}
						let itemOpacity = i.opacity;
						if(colorValues && !colorValuesOpacity) itemOpacity=1;
						s+=`<div class="annotation-pad" title="${esc(a.value)}" style="width:${i.width}px;background-color:${color};"></div>`;
						s+=`<div class="annotation ${i.candidate ? 'candidate' : ''} ${i.annoying ? 'annoying' : ''} ${i.reached ? 'reached' : ''}" title="${esc((a.value))}"`;
						if(showSnapOnHover){
							s+=` data-snap-src="${esc(a.snap())}"`;
						}
						if(showValueOnHover){
							s+=` data-value="${esc(a.value)}"`; //TODO b64?
						}
						if(showMediaOnClick){
							s+=` data-clip-src="${esc(a.clip())}"`;
						}					
						s+=` data-player-width="${videoPlayerWidth}" data-player-height="${videoPlayerHeight}"`;
						s+=` style="opacity:${itemOpacity};left:${i.left}px;width:${i.width}px;background-color:${color};${valueCSS}"></div>`;
					}
					if(tierAlt[tier]) {
						if((showAltTierInEmptyCell && !finalw) || finalw){
						//s+=`<i>${tierAlt[tier]}</i>`;
							s+=`<div class="alt-tier-nooverflow" sstyle="width:${finalw}px"><div>`;
							//log.info('drawNoData-'+tier);
							// START OF ALT-TIER DRAWING
							
							//drawNoData(tierAlt[tier],aset[key]);
							drawNoData(tierAlt[tier],T.alist[key]);
							
							s+=`</div></div>`;
						}
					}
				});
			}
			s += `</td>`;
			s += `</tr>`;
		}
		s += `</tbody></table>`;
		if(Object.keys(altTiers).length){
			s += `<div class="density-legend">`;
			for(var k in altTiers){
				var o = altTiers[k];
				var css = o.css ? o.css : defaultNoDataCSS;
				s += `<div class="legend-item alt-tier-legend"><span class="legend-block alt-tier-block" style="${css}"></span><label>${o.label||o.name||k}</label></div>`;
			}
			for(var k in valueStyle){
				var o = valueStyle[k];
				if(o.label){
					s += `<div class="legend-item value-legend"><span class="legend-block value-block" style="background-color:${o.color||'#000000'};${o.css}"></span><label>${o.label}</label></div>`;
				}
			}
			for(var k in valueLegend){								
				var o = valueLegend[k];
				s += `<div class="legend-item value-legend"><span class="legend-block value-block" style="background-color:${o.color||'#000000'};"></span><label>${o.label}</label></div>`;
			}
			
			s += `</div>`;
		}
		return s;		
	},
	js:s=>{
		function onResize(){
			$('thead .time-collapse').each(function(){
				$(this).height(($(this).parents('table').height()-24)+'px');				
			});
		}
		$(ready=>{
			$('.tier-density .annotation').each((i,e)=>{
				var $e = $(e);
				var snap = $e.data('snap-src');		
				var value = $e.data('value');
				//if(value) value = btoa(value);
				if(snap) Tipped.create(e,'<div><img style="max-width:700px" src="'+snap+'"/></div>');
				if(value) Tipped.create(e,'<div>'+value+'</div>');
				var clip = $e.data('clip-src');
				var pw = $e.data('player-width');
				var ph = $e.data('player-height');
				if(clip) Tipped.create(e,`<div class="density-popup-video" style="${pw?'width:'+pw+';':''}${ph?'height:'+ph+';':''}"><video preload="none" autoplay="true" poster="" controls onplay="Tipped.refresh(this)"><source src="${clip}" type="video/mp4"></video></div>`,
					{showOn:'click', hideOn:false, hideOnClickOutside: true,			
					 onShow: function (content, element) {
					  $(element).addClass("highlight");
					},
					afterHide: function (content, element) {
					  $(element).removeClass("highlight");
					}
					});
			});
			onResize();
			$(window).resize(onResize);
		});
		
	},
	css:s=>{
		return `
		
		
.density-popup-video {
	pointer-events:all;
}
.density-popup-video video {
	width:100%;
	height:100%;
}
#view .time-collapse.collapse-show-duration b {
    top: calc(50% - 22px);
    border-bottom: 1px dashed gainsboro;
}
#view table.density-table.collapse td.tier-density:not(:last-child),
#view table.density-table.collapse thead td:not(:last-child):not(:first-child)
 {
    border-right: 5px dashed gainsboro;
}
#view table.density-table.hide-borders .time-collapse {
	border-style:none;
}
#view table.density-table .time-collapse.collapse-compact b {
    width: 20px;
    border-bottom: 1px dashed #b3b3b3;
    position: relative;
    margin-top: -1px;
}

#view table.density-table .time-collapse.collapse-minimum b {
    visibility:hidden;
	width:10px;
}
#view table.density-table .time-collapse.collapse-none b {
    visibility:hidden;
	width:1px;
}
#view table.density-table .time-collapse.collapse-none {
    border-left:none;
}
#view .annotation.candidate {
    background: black !important;
}
#view .annotation.reached {
    background: pink !important;
}
#view td .annotation.annoying {
    background: cyan !important;
}
#view td {
    border: 1px solid gainsboro;
}
#view td.tier-density.clickable .annotation {
	cursor:pointer;
}
#view td.tier-density {
	position:relative;
}
#view td .annotation {
	position: absolute;
    top: 0px;
    height: 48px;    
}

#view td .annotation.highlight {
	border: 2px dotted;
}
#view .annotation-pad {
    height: 40px;    
    display: inline-block;	
	visibility:hidden;
}
#view tr.tier td.tier-name,
#view tr.tier td.parent-name {
    white-space: nowrap;
    padding: 10px;
}
#view .sequence-separator {
    text-align: center;
    font-weight: bold;
    padding-left: 10px;
    padding-right: 10px;
}
#view td.tier-density {
    padding-right: 20px;
    padding-top: 3px;
    padding-bottom: 0px;
    padding-left: 0px; 
}
#view td.corner {
	border-top:none;
	border-left:none;
}
#view table {
    border-collapse: collapse;
}
#view .tier-density {
    white-space: nowrap;
}
#view .blank {
    height: 40px;    
    display: inline-block;
}
#view thead td {
	position:relative;
}
#view table {
	margin-bottom:4px;
	margin-top:4px;
}
#view .time-collapse {
       height: 200px;
    width: 50px;
    position: absolute;
    top: 22px;
    baackground: linear-gradient(270deg, black, transparent);
    background: white;
    border-style: dashed;
    border-width: 1px;
    border-color: gainsboro;
	border-top:none;
	border-bottom:none;
    vertical-align: middle;
}
#view .time-collapse b {
	font-weight:normal;
    white-space: nowrap;    
    position: absolute;
    top: 50%;
    width: 90px;
    display: block;
    color: gray;
    font-size: 14px;
    lletter-spacing: -1px;
	margin:5px;
	border-bottom:2px dashed gainsboro;
}
#view tr.time-separator {	
    height: 50px;
	background: rgb(255,255,255);
	background: linear-gradient(0deg, rgba(255,255,255,0) 0%, rgba(0,0,0,0.08167016806722693) 50%, rgba(255,255,255,0) 100%);
	text-align:center;
}
#view tr.tier-header {	
    font-weight:bold;
	text-align:center;
}
#view tr.annotation {
    vertical-align: top;
	
}
#view tr.annotation td:first-child {
    color:gray;
	font-size: 14px;
    padding-right: 5px;
}
#view tr.af-separator td {
    background: black;
    color: white;
    font-weight: bold;
    padding: 5px;
    border-radius: 8px;
    text-align: center;
}
#view tr.annotation td {
    border-top: 1px dashed gainsboro;
    max-width: 320px;
	padding-top: 10px;

}
#view video {
    border-radius: 6px;
	display: block;
}
#view .alt-tier-nooverflow > div {
	position:relative;
}
#view .alt-tier-nooverflow {
    position: absolute;
    top: 0px;
    left: 0px;
    overflow: hidden;
	width: -webkit-fill-available;
	width: -moz-available;
	height: -webkit-fill-available;
	height: -moz-available;
	pointer-events: none;
}
#view .legend-block {
    height: -webkit-fill-available;
	height: -moz-available;
}
#view .legend-item {
	margin:5px;
}
#view .legend-item span {
    display: inline-block;
    width: 50px;
    height: 18px !important;
    margin-right: 5px;
	border-radius:6px;
}
#view .legend-item label {
	vertical-align: top;
	font-style:italic;
}

#view .density-legend {
    padding-top: 10px;
}
`

	}
}

function fdur(s){
    var sec_num = parseInt(s, 10);
    var hours   = Math.floor(sec_num / 3600);
    var minutes = Math.floor((sec_num - (hours * 3600)) / 60);
    var seconds = sec_num - (hours * 3600) - (minutes * 60);    
	if(!hours) {
		if(!minutes) return seconds+" seconds";
		return minutes+" minutes";		
	}
    return hours+' hours';
}

function ftime(s){
    var sec_num = parseInt(s, 10);
    var hours   = Math.floor(sec_num / 3600);
    var minutes = Math.floor((sec_num - (hours * 3600)) / 60);
    var seconds = sec_num - (hours * 3600) - (minutes * 60);
    //if (hours   < 10) {hours   = "0"+hours;}
    if (minutes < 10) {minutes = "0"+minutes;}
    if (seconds < 10) {seconds = "0"+seconds;}
    return hours+':'+minutes+':'+seconds;
}