/** 
@author CG
@oldname duration-by
@log 2024-08-08 migrated from old format, added modes
@log 2023-06-03 initial implementation
@input Array<Annotation>
@input Object<Array<Annotation>>
@attr ~ string () [group,tier,participant,value,start,duration,file,extra.*] ~
@attr mode select (sum) [sum,average,min,max,all] 
@attr factor int (1) Convert milliseconds to other units
@attr truncate int (-1) Truncate factored value to X decimal places

Calculates sum/average of annotations' duration (in milliseconds) by grouping on a property (value/tier/group/participant).
If the property is not specified, input object will be directly used as groups.

@sample OP duration-by="tier" _.factor="60000" _.truncate="2"
**/
function(prop, data){
	const factor = attr.factor*1 || 1;
	const truncate = isNaN(attr.truncate*1) ? -1 : attr.truncate*1;
	const modes = {
		'sum':'sum',
		'average':'avg',
		'median':'med',
		'min':'min',
		'max':'max',
		'all':'all'
	}
	const mode = attr.mode || 'sum';
	if(!modes[mode]){
		log.error(`"${mode}" is not a valid mode`);
		return data;
	}	
	if(!prop && Array.isArray(data)){
		log.error(`You must specify a property to group by, or provide an object as input`);
		return data;
	}
	const calcDuration = list=>{
		let all = {};		
		let sum = all['sum'] = {};
		let avg = all['avg'] = {};
		let med = all['med'] = {};
		let min = all['min'] = {};
		let max = all['max'] = {};
		let durations = {};
		let count = {};
		for(let a of list){
			let k = a.get(prop);
			let d = a.stop-a.start;
			if(!durations[k]) durations[k] = [];
			durations[k].push(d);
			sum[k] = (sum[k]||0)+d;			
			count[k] = (count[k]||0)+1;
			if(min[k]===undefined) min[k]=Number.MAX_VALUE;
			if(d<min[k]) min[k]=d;
			if(max[k]===undefined) max[k]=0;
			if(d>max[k]) max[k]=d;
		}
		for(let k in durations){
			let L = durations[k].sort(), D = L.length;
			//log.info(k+' : '+L.join(','));
			med[k] = D%2>0 ? L[(D+1)/2-1] : (L[D/2-1]+L[D/2])/2; // ^^'
		}
		for(let k in sum) for(let m in all) {
			all[m][k] /= factor;
		}
		for(let k in sum){
			avg[k] = sum[k]/count[k];
		}
		if(truncate>-1) for(let k in sum) for(let m in all) {			
			all[m][k] = truncateDecimals(all[m][k],truncate);			
		}
		if(!prop) for(let k in all){
			all[k] = all[k][''] || 0;
		}
		if(mode=='all') return all;
		else return all[modes[mode]];
	}

	if(Array.isArray(data)){		
		let o = calcDuration(data);
		if(mode!='all') return o;
		// special case we must invert object structure to be consistent
		let r = {};		
		for(let k in o){
			for(let kk in o[k]){
				if(!r[kk]) r[kk] = {};
				r[kk][k] = o[k][kk]
			}
		}
		return r;
	}	
	for(let key in data){
		let o = calcDuration(data[key])
		data[key] = o;	
	};
	return data;
	
}

function truncateDecimals(number, digits) {	
    var multiplier = Math.pow(10, digits),
        adjustedNum = number * multiplier,
        truncatedNum = Math[adjustedNum < 0 ? 'ceil' : 'floor'](adjustedNum);

    return truncatedNum / multiplier;
};