/** 
@author CG
@log 2024-08-09 initial implementation
@input Array<Annotation>
@input Object<Array<Annotation>>
@novalue
@attr mode string (sum) [sum,average,median,min,max,all] 
@attr factor int (1) Convert milliseconds to other units
@attr truncate int (-1) Truncate factored value to X decimal places
@attr min-threshold int (0) Min pause in milliseconds between 2 consecutive annotations (pauses shorter than this will not be counted)
@attr max-threshold int (500) Max pause in milliseconds between 2 consecutive annotations (pauses longer than this will not be counted)

Calculates sum/average of annotations' pause duration (in milliseconds).


@sample OP duration-of-pause _.factor="60000" _.truncate="2" _.mode="all"
**/
function(prop, data){
	const minThreshold = attr['min-threshold']*1 || 0;
	const maxThreshold = attr['max-threshold']*1 || 500;
	const factor = attr.factor*1 || 1;
	const truncate = isNaN(attr.truncate*1) ? -1 : attr.truncate*1;
	const modes = {
		'sum':'sum',
		'average':'avg',
		'median':'med',
		'min':'min',
		'max':'max',
		'all':'all'
	}
	const mode = attr.mode || 'sum';
	if(!modes[mode]){
		log.error(`"${mode}" is not a valid mode`);
		return data;
	}	
	if(!prop && Array.isArray(data)){
		log.error(`You must specify a property to group by, or provide an object as input`);
		return data;
	}
	/*let groups = {};
	for(let a of list){
		if(!a || !a.get) continue;
		let k = a.get(prop);
		if(!groups[k]) groups[k] = [];
		groups[k].push(a);
	}*/
	const calcDurationOfPause = list=>{
		let af = {};
		for(let a of list){
			if(!a || !a.get) continue;
			let k = a.af.id;
			if(!af[k]) af[k] = [];
			af[k].push(a);
		}		
		
		// todo: sort by start!?
		let all = {
			sum:0,
			avg:0,
			min:Number.MAX_VALUE,
			max:0,
			med:0			
		};		
		let durations = [];
		let counted=0,skipped=0;
		const asort = (A,B)=>{ return A.start > B.start ? 1 : -1};
		for(let afList of Object.values(af)){
			afList = afList.sort(asort);
			let previous = null;
			//log.info('L='+afList.length);
			
			for(let a of afList){
				if(!previous){
					previous=a;
					continue;
				}							
				let d = a.start-previous.stop
				previous = a;
				if(d>maxThreshold){
					skipped++;					
					continue;					
				}
				if(d<minThreshold){
					skipped++;					
					continue;					
				}				
				counted++;
				durations.push(d);
				all.sum+=d;				
				if(d<all.min) all.min=d;			
				if(d>all.max) all.max=d;
			}		
		}
		for(let k in durations){
			let L = durations.sort(), D = L.length;
			//log.info(k+' : '+L.join(','));
			all.med = D%2>0 ? L[(D+1)/2-1] : (L[D/2-1]+L[D/2])/2; // ^^'
		}
		for(let m in all) {
			all[m] /= factor;
		}
		all.avg = all.sum/counted;
		if(truncate>-1) for(let m in all) {
			all[m] = truncateDecimals(all[m],truncate);
		}
			/*if(!prop) for(let k in all){
				all[k] = all[k][''] || 0;
			}*/
		all.counted = counted;
		all.skipped = skipped;
		if(mode=='all') return all;			
		else return all[modes[mode]];
	}
	
	const calcDurationDeep = v => {		
		if(!v) return v;
		if(Array.isArray(v)) return calcDurationOfPause(v);
		if(typeof v === "object"){
			for(let k of Object.keys(v)){
				v[k] = calcDurationDeep(v[k]);
			}
		}
		return v;	
	}
	return calcDurationDeep(data);
	
	
	if(Array.isArray(data)){		
		return calcDurationOfPause(data);
	}	
	for(let key in data){
		if(!Array.isArray(data[key])){
			log.warn(`data[${key}] is not an array`);
			continue;
		}
		let o = calcDurationOfPause(data[key])
		data[key] = o;	
	};
	return data;
	
}

function truncateDecimals(number, digits) {	
    var multiplier = Math.pow(10, digits),
        adjustedNum = number * multiplier,
        truncatedNum = Math[adjustedNum < 0 ? 'ceil' : 'floor'](adjustedNum);

    return truncatedNum / multiplier;
};