/** 
@author CG
@log 2024-08-22 initial implementation
@input Array<Annotation>
@output Array<Annotation>
@novalue

Combine into one annotation all the overlapping annotations

@dependency group-annotations.js
@dependency sort-annotations.js

**/
function(s, v){	
	const enhanceSeparator = s=>{
		s = s.split("\\n").join("\n");
		s = s.split("\\r").join("\r");
		return s;
	}


	const valueSeparator = attr['value-separator'] || ', ';
	const sentenceSeparator = enhanceSeparator(attr['sentence-separator'] || '');	
	const punctuation = ['…','.',',','?','!'];
	const sentenceMarks = ['.','?','!'];

	const splitSentences = s => {
		let r = '';
		let p = '';
		for(let i=0;i<s.length;i++){
			let c = s.charAt(i);
			let n = s.charAt(i+1);
			r += c;
			if(i<s.length-1 && p!=c && c!=n && sentenceMarks.includes(c)){
				r += `${sentenceSeparator}`
				
				if(n == ' ') i++; // skip next space
			}
			p=c;
		}
		return r;
	}
	
	const combineAnnotations = (a,b)=>{
		a.stop = b.stop;
		let sep = valueSeparator;		
		let lastChar = a.value ? a.value.charAt(a.value.length-1) : '';
		if(punctuation.includes(lastChar)) sep = ' ';
		a.value += sep + b.value;
		return a;
	}
					
	const combineList = list=>{
		let previous;
		let ret = [];
		for(let a of list){
			if(!previous){
				previous = a.clone();
				continue;
			}
			if(previous.stop > a.start){
				previous = combineAnnotations(previous,a);
				continue;
			}
			ret.push(previous);
			previous = a.clone();
		}
		if(previous) ret.push(previous);		
		return ret;		
	}
	
	if(!Array.isArray(v)){		
		log.error(`input must be an array`);
		return v;
	}
	let ret = [];
	let groups = groupAnnotations(v,'file');
	for(let g of Object.values(groups)){
		g = combineList(sortAnnotations(g));
		for(let a of g) ret.push(a);
	}
	return ret;
}