/** 
@author CG
@log 2025-01-07 initial implementation
@input Array<Annotation>
@output Array<Annotation>
@novalue

@attr slice-duration number (0) The duration of each slice annotation, in seconds
@attr slice-count int (0) Alternative to slice-duration, the total amount of slices to create
@attr overlap-threshold number (0) todo

Takes each tier and slices its annotations to segments of a specific duration, aligning all the segments between tiers. The initial alignment starts with the earliest annotation found.

@dependency group-annotations.js
@dependency sort-annotations.js

**/
function(attribute, input){	
	
	if(!Array.isArray(input)){		
		log.error(`input must be an array`);
		return input;
	}
	const sliceDuration = attr['slice-duration']*1000 || 0;
	const sliceCount = attr['slice-count']*1 || 0;
	if(!sliceDuration && !sliceCount){
		log.error(`you must specify slice-duration or slice-count`);
		return input;
	}
	
	const sorted = sortAnnotations(input);
	const mainStart = sorted[0].start;
	const mainStop = sorted[sorted.length-1].stop; //FIXME with real most stop
	const mainDuration = mainStop - mainStart;
	const tiers = {};
	const outTiers = {};
	for(let a of sorted){
		tiers[a.tier.id] = tiers[a.tier.id] || [];
		outTiers[a.tier.id] = outTiers[a.tier.id] || [];
		tiers[a.tier.id].push(a);
	}
	const steps = sliceCount ? sliceCount : Math.ceil(mainDuration / sliceDuration);
	const stepDuration = Math.round(mainDuration / steps);
	log.info(`${steps} steps, slice=${stepDuration}ms`);
	for(let step = 0; step < steps; step++){
		for(let tier in tiers){
			let a = tiers[tier][0];
			if(!a) continue; // maybe tier has no annotation
			a = a.clone();
			a.start = mainStart + step*stepDuration;
			a.stop = mainStart + step*stepDuration + stepDuration;
			a.value = findBestValue(tiers[tier], a.start, a.stop);
			outTiers[tier].push(a);
		}
		
	}
	/*let ret = [];
	let groups = groupAnnotations(v,'file');
	for(let g of Object.values(groups)){
		g = combineList(sortAnnotations(g));
		for(let a of g) ret.push(a);
	}*/
	let ret = [];
	for(let ta of Object.values(outTiers)) ret = ret.concat(ta);
	return ret;
}

function findBestValue(list, start, stop){
	const potential = list.filter(a=>a.start <= stop && start <= a.stop);
	/*for(let a of list){
		if(a.stop<start) continue;
		if(a.start>stop) continue;
	}*/
	if(!potential.length) return '';
	let mostOverlaping, mostOverlap = 0;
	for(let a of potential){
		let overlap = Math.min(a.stop,stop) - Math.max(a.start,start);
		if(overlap>mostOverlap){
			mostOverlap=overlap;
			mostOverlaping=a;
		}
	}
	return (mostOverlaping||potential[0]).value;
	/*const center = (stop - start) / 2;
	let closest;
	let closestDist=Number.MAX_SAFE_INTEGER;
	for(let a of potential){
		let ac = (a.stop - a.start) / 2;
		let dist = Math.abs(center-ac);
		if(dist<closestDist){
			closestDist=dist;
			closest=a;
		}
	}
	return closest.value;
	*/
}