/** 
@author CG
@input Array<Array>
@input Object<Array>
@output Array<Integer>
@output Object<Integer>
@novalue
@attr depth int (0) How deep to go in the input tree before counting 
@attr remove-uncountable bool (false) Whether to remove the properties that are not countable
@attr remove-uncountable-elements bool (false) Whether to remove the uncountable elements from array, resulting in a shorter array
@attr count-strings bool (false) Whether to consider strings as arrays for counting

Counts the elements in each array element of the input array/object, replacing the array element itself with an integer value

@sample OP count=""
**/
function(s, v){

	var depth = attr.depth*1 || 0;
	var removeUncountable = attr['remove-uncountable']=='true';
	var removeUncountableElements = attr['remove-uncountable-elements']=='true';
	var countStrings = attr['count-strings']=='true';
	var isObject = o => o && o.constructor.name === "Object";
	var isString = o => typeof o === 'string' || o instanceof String;
	var deeper = function deeper(o,level,parent,key){		
		if(level === 0){
			let r=null;
			if(Array.isArray(o)){
				r = [];
				o.forEach(oo=>{
					//r.push(oo.length||0));
					if(Array.isArray(oo) || isObject(oo)){
						r.push(oo.length);
					}else if(countStrings && isString(oo)){
						r.push(oo.length);						
					}else if(!removeUncountable) {
						r.push(oo);
					}
				});
			}else if(isObject(o)){
				r = {};
				for(k in o) {
					if(Array.isArray(o[k]) || isObject(o[k])){
						r[k] = o[k].length;				
					}else if(countStrings && isString(o[k])){
						r[k] = o[k].length;						
					}else if(!removeUncountable) {
						r[k] = o[k];
					}
				}
			}
			if(r) parent[key] = r;
			else if(removeUncountable) delete parent[key];
		}else{
			if(Array.isArray(o)){				
				o.forEach((oo,i)=>deeper(oo,level-1,o,i));
			}else if(isObject(o)){
				for(k in o) deeper(o[k],level-1,o,k);	
			}
		}
		
	}
	var V = {k:v}
	deeper(v,depth,V,'k');
	if(removeUncountableElements && V.k.forEach){
		var a = [];		
		V.k.forEach(o=>{if(o!==null)a.push(o)});
		return a;
	}
	return V.k;	
	
	// old imp
	if(Array.isArray(v)){
		return `
			var a = [];
			data.forEach(o=>a.push(o.length||0));
			a;
		`
	}
	
	return `
		var o = {};
		for(k in data) o[k] = data[k].length;
		o;
	`
}