/** 
@author CG
@log 2023-07-25 initial implementation
@input Array<Annotation>
@output Array<Annotation>
@output Object<String,Array<Annotation>>

@attr range number (30) Minimum number of seconds between two annotations to consider as a new sequence
@attr to-tier string (sequences) The created tier wich will contain the sequences annotations
@attr output choices (array) [array,object]  The output type, which can be an Array of sequences' annotations, or an Object<Array> mapping a sequence to its list of annotations.
@novalue

Detects sequences by grouping annotations that are close to each other.

@sample OP detect-sequences="" _.range="15" _.output="object"
**/
function(s, v){		
	var range = attr.range*1 || 30;	
	var toTier = 'sequences';
	var toObject = attr.output=='object' ? {} : null;
	if(!v || !v.forEach) throw "invalid input, must be an array";
	if(!v.length) return v;
	var sequences = [];
	var tier;
	var seq;
	//TODO group and process by af
	data.sort((a,b)=>a.start>b.start?1:-1); // first order all input annotations by start time
	data.forEach(a=>{
		if(!tier) tier = a.af.addTier({id:toTier});
		//TODO problem if tier is shared accross different af?
		if(!seq || a.start>seq.stop+range*1000){			
			seq = a.clone();
			seq.tier = tier;			
			sequences.push(seq);
			seq.value = "sequence "+sequences.length;
			if(toObject) {
				if(!toObject[seq.value]) toObject[seq.value] = [];
				toObject[seq.value].push(a);
			}
		}else{
			if(toObject) toObject[seq.value].push(a);
			seq.stop = a.stop;
		}
	});
	return toObject ? toObject : sequences;
}