/** 
@author CG
@log 2023-07-25 initial implementation
@input Array<Annotation>
@output Array<Annotation>
@output Object<String,Array<Annotation>>

@attr range number (30) Minimum number of seconds between two annotations to consider as a new sequence
@attr to-tier string (sequences) The created tier wich will contain the sequences annotations
@attr output choices (array) [array,object]  The output type, which can be an Array of sequences' annotations, or an Object<Array> mapping a sequence to its list of annotations.
@novalue

Detects sequences by grouping annotations that are close to each other.

@dependency group-annotations.js
@dependency sort-annotations.js

@sample OP detect-sequences="" _.range="15" _.output="object"
**/
function(s, input){		
	const range = attr.range*1 || 30;	
	const toTier = attr['to-tier'] || 'sequences';
	const toObject = attr.output=='object' ? {} : null;
	if(!input || !input.forEach) throw "invalid input, must be an array";
	if(!input.length) return input;
	const sequences = [];
	
	let G = groupAnnotations(input,'file');
	for(let g of Object.values(G)){
		let tier=null,seq=null;
		for(let a of sortAnnotations(g)){
			if(!tier) tier = a.af.addTier({id:toTier});
			if(!seq || a.start>seq.stop+range*1000){			
				seq = a.clone();
				seq.tier = tier;			
				sequences.push(seq);
				seq.value = "sequence "+sequences.length;
				if(toObject) {
					if(!toObject[seq.value]) toObject[seq.value] = [];
					toObject[seq.value].push(a);
				}				
			}else{
				if(toObject) toObject[seq.value].push(a);
				seq.stop = a.stop;
			}
		}
	}
	
	
	return toObject ? toObject : sequences;
}