/** 
@author CG
@log 2023-06-03 initial implementation
@input Array<Annotation>
@input Object<String,Array<Annotation>>
@attr factor int (1) Convert milliseconds to other units
@attr truncate int (-1) Truncate factored value to X decimal places

Calculate sum of annotations' duration by grouping on a property (value/tier/group/participant)

@sample OP duration-by="tier" _.factor="60000" _.truncate="2"
**/
function(s, v){
	var factor = attr.factor*1 || 1;
	var truncate = isNaN(attr.truncate*1) ? -1 : attr.truncate*1;
	if(factor==0) throw "invalid factor value, can't be 0";	
	if(Array.isArray(v)){
		return `
			var o = {};
			data.forEach(a=>{
				var k = a.get('${s}')+'';
				if(!o[k]) o[k]=0;
				o[k] += (a.stop-a.start);
			});
			for(var k in o) {
				o[k] = o[k]/${factor};
				if(${truncate}>-1) o[k] = truncateDecimals(o[k],${truncate});
			}
			o;
		`;
	}
	
	return `
		Object.keys(data).map(key=>{
			var o = {};			
			data[key].forEach(a=>{
				var k = a.get('${s}')+'';
				if(!o[k]) o[k]=0;
				o[k] += (a.stop-a.start);
			});
			for(var k in o){
				o[k] = o[k]/${factor};
				if(${truncate}>-1) o[k] = truncateDecimals(o[k],${truncate});
			}
			data[key] = o;
		});
		data;
	`;
}

function truncateDecimals(number, digits) {	
    var multiplier = Math.pow(10, digits),
        adjustedNum = number * multiplier,
        truncatedNum = Math[adjustedNum < 0 ? 'ceil' : 'floor'](adjustedNum);

    return truncatedNum / multiplier;
};