/** 
@log 2024-08-08 initial implementation 
@author CG
@input Array<Annotation>
@output Object<String,Array>

Groups the annotations by specific file tags (comma separated or JSON array of strings).
Corpus files can have multiple tags, therefore an annotation could appear in multiple groups.
The result is an object whose keys are the chosen tags, mapped to arrays of annotations.

@sample OP group-by-file-tags="tag 1,tag 2"

**/
function(value, data){	

	if(!Array.isArray(data)){
		log.warn(`input is not an array`);
		return data;
	}
	
	var tags = [];
	try {
		tags = JSON.parse(value);
	}catch(ex){}
	if(!tags || !tags.length){
		tags = value.split(',');
	}
	if(!tags || !tags.length || !tags[0]){
		log.info(`no tag was specified, using all tags`);
		for(let a of data){
			if(!a || !a.af) continue;
			for(let t of a.af.tags) if(!tags.includes(t)) tags.push(t);
		}		
	}
		
	var o = {};	
	for(let t of tags) o[t] = [];	
	for(let a of data){
		if(!a || !a.af) continue;
		for(let t of tags){
			if(a.af.tags.contains(t)){
				o[t].push(a);
			}
		}		
	}
	return o;	
}