/** 
@author CG
@log 2024-08-16 initial implementation
@experimental
@input Array<Annotation>
@output Array<Annotation>
@novalue

@attr hallucination-char-factor int (1) todo
@attr hallucination-time-diff int (1000) Maximum pause time between 2 hallucinated annotations (in milliseconds)
@attr merge-comma-end bool (true) Merge 2 annotations if the first one ends with a comma
@attr merge-lowercase-start bool (true) Merge 2 annotations if the second one starts with a lower-case character

Improves transcribed content with various heuristic techniques.

**/
function(s, list, c){	
	if(!list || !list.length){
		log.error(`no annotation was provided`);
		return list;
	}
	
	const hallucinationTimeDiff = attr['hallucination-time-diff']==='0' ? 0 : attr['hallucination-time-diff']*1 || 1000;
	//const mergeCommaEnd = !(attr['merge-comma-end']=='false');
	//const mergeLowercaseStart = !(attr['merge-lowercase-start']=='false');
	const improvedAnnotations = [];

	const merge = (current,a)=>{
		current.value += ' '+a.value;	
		current.stop = a.stop;
	}
	const shouldMerge = (current,a)=>{
		let firstChar = a.value.charAt(0);
		if(firstChar.toLowerCase()===firstChar) return true;
		
		let v = current.value.trim();
		if(!v) return false;
		let lastChar = v.charAt(v.length-1);
		if(lastChar==',') return true;
		
	}
	const merges = [];
	const hallus = {};
	let current;
	for(let a of data){
		if(!current){ 
			current=a.clone();			
			continue;
		}
		if(hallucinationTimeDiff && current.tier.id == a.tier.id && current.value == a.value){
			if(a.start-hallucinationTimeDiff < current.stop){				
				hallus[a.value] = (hallus[a.value]||0)+1;
				//log.info(`removed hallucination "${a.value}"`)
				current = a.clone();
				continue;
			}
		}
		if(current.tier.id == a.tier.id && current.af == a.af && shouldMerge(current,a)){
			merge(current,a)
			merges.push(a);
		}else{
			improvedAnnotations.push(current);
			current = a.clone();			
		}
	}			
	for(let h of Object.keys(hallus)){		
		log.info(`Removed hallucination "${h}" x${hallus[h]}`)
	}
	if(merges.length){
		log.info(`Merged ${merges.length} annotations`);
	}
	if(current) improvedAnnotations.push(current);
	return improvedAnnotations;	
}