/** 
@author CG
@log 2023-20-03 initial implementation
@input None
@output Array<Annotation>

@attr dimension string The dimension to extract from the form
@attr exclude-from-intercoding bool (false) Whether to tag the extracted annotations so they are excluded from intercoding calculations
@attr extract-original-annotations bool (false) Whether to extract only the original annotations from the form, so the results are safe from corpus changes. When this attribute is specified, no dimension will be extracted: use again the operation to extract a dimension.

Loads JSON files resulting from forms, as virtual annotations.
The JSON files must be in the folder of the processed XML file.

@sample OP load-annotations-from-form="my form (*).json" 
**/
function(filter, v , c){
	var files = api.listFiles(filter);
	if(!files.length){
		log.error(`No files found with filter "${filter}"`);
		return `data;`
	}	
	var excludeFromIntercoding = attr['exclude-from-intercoding'] == 'true';
	var extractOriginalAnnotations = attr['extract-original-annotations'] == 'true';
	var addTag = attr['add-tag'] ? attr['add-tag']+'' : '';
	attr.dimension = attr.dimension ? attr.dimension+'':'';	
	var tiers = [];
	var finalTiers = [];
	var dimensions = {};
	//var tierNames = [];
	var legacyInput = false;
	var ret = [];
	let cleanFileName = s=>{
		s=(s+'').split('\\').pop().split('/').pop();
		filter.split('*').forEach(p=>s=s.split(p).join(''));
		return s;
	}
	if(extractOriginalAnnotations){		
		var listCount=-1;
		var listFile;		
		files.forEach(f=>{
			let json = JSON.parse(api.readFile(f));		;
			if(listCount<0){
				listCount = json.list.length;
				listFile=f;				
			}			
			else if(listCount!=json.list.length){
				log.warn(`Forms don't contain same amount of results: ${cleanFileName(f)}=${json.list.length} ${cleanFileName(listFile)}=${listCount}`);
			}
			if(listCount<json.list.length){ // we keep the one with most number of results
				listCount = json.list.length;
				listFile = f;
			}
		});
		let f = listFile || files[0]; //TODO: from a master form file?
		let json = JSON.parse(api.readFile(f));		;
		json.list.forEach(o=>{
			let afid = o.file || o.af || o.eaf; //legacy
			let af = api.getAF(afid);
			if(!af) return log.warn("Can't find annotation file `"+afid+"` in domain");					
			//log.info(af.id);
			let tier = af.getTier(o.tier);
			if(!tier) tier = af.addTier({id:o.tier});
			let a = af.addAnnotation({ //TODO .createAnnotation so it doesn't affect current domain state?
				tier:tier,
				id:o.tier+"_"+o.start+"_"+o.stop, //TODO harmonize
				start:o.start,
				stop:o.stop,
				value:o.value
			});				
			if(excludeFromIntercoding) a.addTag('exclude-from-intercoding');
			ret.push(a);
		})
		if(ret.length) log.info(`extracted ${ret.length} original annotations from ${cleanFileName(f.name)}`);
		else log.warn(`no original annotation found!`)
		return ret;
	}
	files.forEach(f=>{		
		var tierName = (f.getName()+"").split("(")[1].split(')')[0];
		tiers.push(tierName);
		finalTiers.push(tierName);
		log.debug('Loading '+tierName);
		try {
			var dim = {};
			var json = JSON.parse(api.readFile(f));
			if(!json.representation){
				json.representation = {"dimensions":[{"name":"choix","label":"Choisissez","type":"select","choices":"chef,map,locX,map-et-loc,chef-et-map,chef-et-loc"},{"name":"justification","label":"Justifiez","type":"text"}],"fields":[{"name":"nom","label":"Entrez votre nom","type":"text"}]};
				json.list.forEach(o=>{
					o.dimensions = {choix:o.input, justification:''}
				});
				log.warn('used special clo-fix');
			}
			if(json.representation && json.representation.dimensions){
				json.representation.dimensions.forEach(d=>{
					dimensions[d.name] = d;
					dim[d.name] = d;
				})
			}			
			
				
			
			
			if(!attr.dimension){
				attr.dimension = Object.keys(dimensions)[0];
				if(!attr.dimension){
					log.error(`no dimension found to use as default dimension`);
					return;
				}else{
					log.info(`using dimension "${attr.dimension}" because none was specified`);
				}
			}				
			
			if(attr.dimension.charAt(0)=='~'){
				//attr.dimension = attr.dimension.
			}
			if(!dim[attr.dimension]){
				log.warn(`dimension "${attr.dimension}" not found in form of tier "${tierName}"`);
				return;
			}
			json.list.forEach(o=>{
				var afid = o.file || o.af || o.eaf; //legacy
				var af = api.getAF(afid);
				if(!af){
					log.warn(`can't find annotation file "${afid}" in domain`);
					return;
				}
				let T = tierName+'';
				if(attr.dimension) T = T + ' - ' +attr.dimension;
				//log.warn(T);
				if(finalTiers.indexOf(T)==-1) finalTiers.push(T);
				let tier = af.getTier(T);
				if(!tier) tier = af.addTier({id:T});
				
				let value = o.dimensions ? o.dimensions[attr.dimension] : o.input;
				let a = af.addAnnotation({
					tier:tier,
					id:"",
					start:o.start,
					stop:o.stop,
					value:value
				});
				a.setExtra('form',true);
				if(excludeFromIntercoding) a.addTag('exclude-from-intercoding');
				ret.push(a);
			});
		}catch(ex){
			log.error(`FAILED TO READ FILE: ${f.getName()}`);			
			log.error(ex.message);
		}
	});
	log.info('found tiers: '+tiers.join(', '));
	log.info('found dimensions: '+Object.keys(dimensions).join(', '));	
	log.info(`extracted ${ret.length} results as new annotations`);
	return ret;
	
	
	return `
		data = [];
		${JSON.stringify(finalTiers)}.forEach(tier=>{
			api.doc.domain.forEach(af=>{
				var t = af.getTier(tier);
				if(!t) return;
				t.annotations.forEach(a=>data.push(a));	
			});
		});
		data;
	`;
}	
