/** 
@author CG
@log 2023-07-12 initial implementation
@input Array
@output Array
@novalue
@attr limit int (0) Maximum number of elements for the output array
@attr limit-per-file (0) Maximum number of annotations to take from one particular annotation file (if the input array contains annotations)
@attr seed int (1) The initial seed for reproducible randomness
@attr prng string (LCG) The algorithm for random number generation

Randomizes the input selection with a PRNG allowing reproducible results based on an initial seed.

@todo https://gist.github.com/blixt/f17b47c62508be59987b
@todo https://github.com/bryc/code/blob/master/jshash/PRNGs.md

**/
function(s, v){

	if(!Array.isArray(v)) throw "input is not an array";
	
	var limit = (attr.limit*1||0);
	var limitPerFile = (attr['limit-per-file']*1||0);
	var seed = (attr.seed*1||1);
	var prng = attr.prng ? attr.prng : 'LCG';
	if(!algo[prng]) throw `the PRNG algorithm "${prng}" does not exist`;
	var rand = algo[prng](seed);	
	var files = {};
	
	v.forEach(a=>{
		var fid = a.tier ? a.tier.af.id : '';
		if(!files[fid]) files[fid] = [];
		files[fid].push(a);		
	});
	var source = Object.values(files);	
	source.forEach(arr=>{
		shuffle(arr,rand);
	});	
	
	if(limitPerFile){
		source.forEach(arr=>arr.length=(limitPerFile<arr.length?limitPerFile:arr.length));	
	};
	
	var list = [];
	var done = false;	
	while(!done){
		var before = list.length;
		source.forEach(arr => {
			if(limit && list.length>=limit) return done=true;
			if(arr.length==0) return;
			list.push(arr.shift());
		});
		var after = list.length;
		if(before==after){
			//log.info("exhausted all");
			done=true;
		}		
	}
	return list;
}

var algo = {};
algo.LCG=s=>()=>((s=(48271*s)%2147483647))/2147483647;

//https://stackoverflow.com/questions/16801687/javascript-random-ordering-with-seed
function shuffle(array, rand) {
  var m = array.length, t, i;
  while (m) {
    i = Math.floor(rand() * m--);
    t = array[m];
    array[m] = array[i];
    array[i] = t;    
  }
  return array;
}