/** 
@author CG
@log 2024-08-28 added limit and reverse attributes
@log 2024-08-21 migrated to new op format, added final sort
@log 2023-02-24 range implementation
@log 2023-11-03 overlap fix!
@oldname select-annotations-in-sequence-from-tier
@attr ~ tier () ~
@attr overlap bool (true) Whether to include overlaping annotations (not fully contained in the sequence)
@attr range number (0) Number of seconds to add before and after the sequence, for considering annotations in sequence
@attr range-before number (0) Additional seconds to add before
@attr range-after number (0) Additional seconds to add after
@attr distinct bool (false) Whether to remove duplicate annotations from the resulting list
@attr combine bool (false) Whether to combine in one final annotation, all the annotations found in the sequence
@attr limit int (0) Max number of annotations to return (0 = all annotations found in sequence, 1 = first annotation found)
@attr reverse bool (false) Whether to reverse the list of annotations found in sequence before applying the limit. This can be used to return the last annotation found in sequence (via limit=1).
@attr separator string ( | ) A separator to insert when combining values of multiple annotations
@attr default-to-null bool (false) Whether to add a null element in the output array if no corresponding annotation is found for a given input annotation (default to false, or true if the operation runs in an EXTRA block)
@input Array<Annotation>
@output Array<Annotation>

Considers each input annotation as a sequence, and selects those from another tier (of the same file) that are included in the sequences.

@sample OP replace-with-annotations-in-sequence-from-tier="tier id" _.overlap="false"
**/
function(s, v){	
	const defaultToNull = context=='EXTRA' ? true : attr['default-to-null']=='true';	
	const overlap = attr.overlap=='false' ? false : true;
	const combine = attr.combine=='true';
	const reverse = attr.reverse=='true';
	const limit = attr.limit*1 || 0;
	const range = attr.range*1 || 0;
	const rangeBefore = isNaN(attr['range-before']*1) ? range : attr['range-before']*1;
	const rangeAfter = isNaN(attr['range-after']*1) ? range : attr['range-after']*1;	
	const distinct = attr.distinct === 'true';
	const separator = attr.separator || ' | ';
	const ret = [];				
	const failed = [];
	const asort = (A,B)=>{ return A.start > B.start ? 1 : -1};
	if(!Array.isArray(v)){		
		log.error(`input must be an array`);
		return v;
	}
	for(let seq of data){
		let count = ret.length;
		let seqA = [];
		let seqStart = seq.start;
		let seqStop = seq.stop;
		if(rangeBefore || rangeAfter){
			seqStart -= rangeBefore*1000;
			seqStop += rangeAfter*1000;
		}
		let seqAfTier = seq.af.getTier(s);
		if(!seqAfTier){
			log.warn(`could not find tier '${s}' in '${seq.af.filename}'`);
			if(defaultToNull) ret.push(null);
			failed.push(seq.toString());
			continue;
		}
		
		for(let a of seqAfTier.annotations.toArray().sort(asort)){
			if(!a) continue;
			if(overlap){
				//if(a.start >= seqStart && a.start <= seqStop
				//|| a.stop >= seqStart && a.stop <= seqStop
				if(a.start <= seqStop && seqStart <= a.stop) seqA.push(a);	
			}else{
				if(a.start >= seqStart && a.start <= seqStop && a.stop <= seqStop) seqA.push(a);
			}
		}
		if(!seqA.length){
			if(defaultToNull) ret.push(null);			
			failed.push(seq.toString());
		}else if(combine){
			let a = seqA[0].clone();
			a.start = seqA[0].start;
			a.stop = seqA[seqA.length-1].stop;				
			let values = seqA.sort(asort).map(A=>A.value);			
			a.value = values.join(separator);
			a.extra.combined = seqA.slice().sort(asort);
			ret.push(a);
		}else{
			let max = limit;
			if(reverse) seqA = seqA.reverse();
			for(let a of seqA){
				ret.push(a);
				max--;
				if(max===0) break;
			}
		}
	}
	if(failed.length){
		log.warn(`no annotation found in ${failed.length} sequence${failed.length>1?'s':''}`);
		log.report({title:`no annotation found in these sequences`,list:failed});
	}
	if(distinct){				
		for(let a of ret) if(uniques.indexOf(a)<0) uniques.push(a);
		return uniques;
	}
	
	return ret;	
}