/** 
@author CG
@oldname select-annotations-in-sequence-from-tier
@log 2023-02-24 range implementation
@log 2023-11-03 overlap fix!
@attr overlap bool (true) Whether to include overlaping annotations (not fully contained in the sequence)
@attr range number (0) Additional range in seconds to consider annotations in sequence
@attr combine bool (false) Whether to combine in one final annotation, all the annotations found in the sequence
@attr separator string ( | ) A separator to insert when combining values of multiple annotations
@attr default-to-null bool (false) Whether to add a null element in the output array if no corresponding annotation is found for a given input annotation (default to false, or true if the operation runs in an EXTRA block)
@input Array<Annotation>
@output Array<Annotation>

Considers each input annotation as a sequence, and selects those from another tier (of the same file) that are included in the sequences.

@sample OP replace-with-annotations-in-sequence-from-tier="tier id" _.overlap="false"
**/
function(s, v){	
	var overlap = !(attr.overlap=='false') ? 1 : 0;	
	var combine = attr.combine=='true' ? 1 : 0;	
	var defaultToNull = attr['default-to-null']=='true' ? 1 : 0;
	if(context=='EXTRA' && attr['default-to-null']==null) defaultToNull = true;	
	var range = attr.range*1 || 0;
	var separator = attr.separator || ' | ';
	//TODO migrate to inner js	
	return `var ret = [];				
		var failed = [];
		data.forEach(seq=>{
			var count = ret.length;
			var seqA = [];
			var seqStart = seq.start;
			var seqStop = seq.stop;
			if(${range}){
				seqStart -= ${range}*1000;
				seqStop += ${range}*1000;
			}
			var seqAfTier = seq.af.getTier("${s}");
			if(!seqAfTier){
				log.warn("could not find tier '${s}' in '"+seq.af.filename+"'");
				if(${defaultToNull}) ret.push(null);
				return;
			}
			seqAfTier.annotations.toArray().forEach(a=>{
				if(${overlap}){
					//if(a.start >= seqStart && a.start <= seqStop
					//|| a.stop >= seqStart && a.stop <= seqStop
					if(a.start <= seqStop && seqStart <= a.stop
					) seqA.push(a);	
				}else{
					if(a.start >= seqStart && a.start <= seqStop && a.stop <= seqStop) seqA.push(a);
				}
			});
			if(!seqA.length){
				if(${defaultToNull}) ret.push(null);
				log.warn("no annotation found in sequence");
				failed.push(seq.toString());
			}else if(${combine}){
				var a = seqA[0].clone();
				a.start = seqA[0].start;
				a.stop = seqA[seqA.length-1].stop;				
				var values = [];
				seqA.forEach(A=>values.push(A.value));
				a.value = values.join("${separator}");
				ret.push(a);
			}else{
				seqA.forEach(A=>ret.push(A));
			}
		});
		if(failed.length) log.report({title:'no annotations found in these sequences',list:failed});
		ret;
	`;
		
}