/** 
@author CG
@oldname select-annotations-from-tier-with-same-timecode
@log 2024-08-09 fixed selecting closest in-range annotation
@log 2023-02-26 range implementation
@todo @attr combine
@todo attr throw-error bool Whether to stop and report an error if a corresponding annotation can't be found
@attr ~ tier () ~
@attr multiple bool (false) Whether to select multiple annotations if more than one is in range
@attr range number (0) Acceptable range in seconds to consider 2 timecodes as equivalent
@attr default-to-null bool Whether to add a null element in the output array if no corresponding annotation is found for a given input annotation (default to false, or true if the operation runs in an EXTRA block)
@input Array<Annotation>
@output Array<Annotation>

Replaces each input annotation with one from another tier that has the same start time.

@sample OP replace-with-same-timecode-annotations-from-tier="tier id" _.range="0.5"
**/
function(s, data){
	const throwError = !(attr['throw-error']=='false') ? 1 : 0;
	const multiple = attr['multiple']=='true' ? 1 : 0;
	var defaultToNull = attr['default-to-null']=='true' ? 1 : 0;
	if(context=='EXTRA' && attr['default-to-null']==null) defaultToNull = true;	
	const range = attr.range*1 || 0;	
	const ret = [];		
	const failed = [];
	data = data.toArray ? data.toArray() : data;
	for(let a of data){
		let count = ret.length;
		let found = false;
		let inRange = [];
		for(let aa of a.tier.af.getTier(s).annotations){			
			if(!range){
				if(a.start == aa.start){
					ret.push(aa);
					found = true;
					break;
				}
				continue;
			}
			// range logic:
			if(Math.abs(aa.start - a.start) <= range*1000) {
				inRange.push(aa);
				found=true;
			}
		}
		if(inRange.length && multiple){
			for(let aa of inRange) ret.push(aa);
		}
		if(inRange.length && !multiple){
			let closest = inRange[0];
			for(let i=1;i<inRange.length;i++){
				let aa = inRange[i];
				let d = Math.abs(aa.start - a.start);
				if(d < Math.abs(closest.start - a.start)){
					closest = aa;
				}
			}
			ret.push(closest);
		}
		if(!found){
			if(defaultToNull) ret.push(null);
			
			failed.push(a.toString());		
		}
	}
	if(failed.length){
		log.warn(`no match for ${failed.length} input annotation${failed.length>1?'s':''}`);
		log.report({title:'no matching annotation',list:failed});	
	}
	return ret;
}