/** 
@author CG
@oldname select-annotations-from-tier-with-same-timecode
@log 2023-02-26 range implementation
@todo @attr combine
@todo attr throw-error bool Whether to stop and report an error if a corresponding annotation can't be found
@attr multiple bool (false) Whether to select multiple annotations if more than one is in range
@attr range number (0) Acceptable range in seconds to consider 2 timecodes as equivalent
@attr default-to-null bool Whether to add a null element in the output array if no corresponding annotation is found for a given input annotation (default to false, or true if the operation runs in an EXTRA block)
@input Array<Annotation>
@output Array<Annotation>

Replaces each input annotation with one from another tier that has the same start time.

@sample OP replace-with-same-timecode-annotations-from-tier="tier id" _.range="0.5"
**/
function(s, v){
	var throwError = !(attr['throw-error']=='false') ? 1 : 0;
	var multiple = attr['multiple']=='true' ? 1 : 0;
	var defaultToNull = attr['default-to-null']=='true' ? 1 : 0;
	if(context=='EXTRA' && attr['default-to-null']==null) defaultToNull = true;	
	var range = attr.window*1 || attr.range*1 || 0;
	v = v.toArray ? v.toArray() : v;
	//s = `a=>a.refs.get("${s}").value`;
	return `var ret = [];		
		var failed = [];
		data = data.toArray ? data.toArray() : data;
		data.forEach(a=>{
			var count = ret.length;
			var found = false;
			a.tier.af.getTier("${s}").annotations.toArray().forEach(aa=>{
				if(found && !${multiple}) return;
				if(${range}){
					if(Math.abs(aa.start - a.start) <= ${range}*1000) {
						ret.push(aa)
						found = true;
					}
				}else{
					if(a.start == aa.start){
						ret.push(aa);
						found = true;
					}
				}
				
			});			
			if(!found){
				if(${defaultToNull}) ret.push(null);
				log.warn("no matching annotation");
				failed.push(a.toString());				
			}
		})	
		if(failed.length) log.report({title:'no matching annotation',list:failed});	
		ret;
	`;
		
}