/**
@author CG
@log 2023-01-15 initial implementation
@log 2023-07-29 added attributes
@novalue
@attr group-by-attribute string () An optional attribute name to be used for grouping type-tokens together
@attr strip-punctuation bool (true) Whether to remove all punctuation before type-token processing
@attr case-sensitive bool (false) Whether to consider capital letters in words comparison
@attr replace-regex string () A regular expression to replace text before type-token processing
@attr replace-func string () A js function to replace text before type-token processing
@attr split-func string () A js function to use instead of the basic space splitting, for extracting words from strings
@input Array<Annotation>
@input Object<Array<Annotation>>
@input Object<Array<String>>
@input Object<String>
@output Object

Computes the type-token of input annotations.



**/
function(s, v){
	var data = v;
	var groupByAttribute = attr['group-by-attribute'] || s || ''; // legacy arg s before @novalue implementation
	var caseSensitive = attr['case-sensitive'] == true;
	var replaceRegex = attr['replace-regex'];
	var replaceFunc = attr['replace-func'];
	var splitFunc = attr['split-func'];
	replaceFunc = replaceFunc ? new Function(s, 'return ('+replaceFunc+')()') : s => s;
	splitFunc = splitFunc ? new Function(s, 'return ('+splitFunc+')()') : s => s.split(' ');
	if(Array.isArray(data) && data[0].tier){ // annotations array
		var tiers = {};
		data.forEach(a=>{
			var tid = a.tier.id;
			if(!tiers[tid]) tiers[tid] = [];
			tiers[tid].push(a);
		});
		data = tiers;
	}
	var K = groupByAttribute;//"${s}";
	Object.keys(data).map(key=>{
		var o = {};
		var A = data[key];
		A = A.toArray ? A.toArray() : A;
		A.forEach(a=>{
			var t = K ? (a.get ? a.get(K) : a[K]) : '_';
			if(!o[t]) o[t] = {total:0,words:[],annotations:0};
			o[t].annotations++;
			splitFunc(replaceFunc(a.value)).forEach(word=>{
				if(!caseSensitive) word = word.toLowerCase();
				if(word.length<2) return;
				o[t].total++;
				if(o[t].words.indexOf(word+'')==-1) o[t].words.push(word+'');
			});
		});		
		for(var t in o){
			o[t].unique = o[t].words.length;
			o[t].ratio = o[t].unique/o[t].total;
			o[t].percent = Math.round(o[t].ratio * 100);
			delete o[t].words;
		}
		data[key] = o;
		if(!K) data[key] = o['_'];
	});
	return data;
}