/** 
@author CG
@license @ext Hybrid Spectrogram and Waveform Source Separation https://github.com/facebookresearch/demucs | MIT
@log 2023-11-30 initial implementation
@cachable

@attr source string (vocals) [vocals,drums] The source type to separate from the rest

@dependency proc-exec-output-wrapper.js
@dependency proc-python-wrapper.js

Demucs can separate voice and instruments from an audio track

**/
proc['demucs-separation'] = {	
	pipe:pipe=>{		
		const isPreview = api.converter.getSetting('test-mode')=='true'; //todo api?		
		const isEstimate = api.converter.getSetting('estimate-mode')=='true'; //todo api?
		
		if(isEstimate) return;
		const attrSource = attr['source'] || 'vocals';
		
		let req = log.addStep({name:'Requirements'})
		if(!pythonCheck(req)) return false;
		if(!pipCheck("torch",req)) return false;
		if(!pipCheck("torchaudio",req)) return false;
		if(!pipCheck("demucs",req)) return false;		
		let wrapperFile = pythonWrapper("avaa-demucs",req);
		if(!wrapperFile) return false;
				
		var finalResult = undefined;
		for(let f of pipe.domain){
			let step = log.addStep(f);						
			if(!f.media.size()) {
				step.error(`No media file available in ${f.id}`);
				continue;
			}						
			for(let pmf of f.media){
				step.file(pmf,`Input = `);			
				let sourceFile = pmf;
				if(pmf.getExtension()!='wav'){
					step.info(`Converting to WAV...`)
					step.work();
					let wavFile = pipe.temp(pmf,'wav');
					let rx = api.exec(['ffmpeg','-i',pmf.getAbsolutePath(),wavFile.getAbsolutePath()]).run();
					if(rx){
						step.error(`Something went wrong converting to WAV file: ${wavFile.getAbsolutePath()}`);
						return false;
					}
					sourceFile = wavFile;
					step.ok();
					step.file(wavFile,`Converted to `)
				}						
				step.info(`Launching Demucs process`)
				step.work();
				let xargs = ['python','-u',wrapperFile.getAbsolutePath(),
					'-i',sourceFile.getAbsolutePath(),								
				];
				if(isPreview) xargs.push('--fake')			
				let aw = api.exec(xargs);
				let progress = '';
				aw.onError(s=>{
					step.error(`A process error occurred: ${s}`)
				})			
				aw.onOutput(execOutputWrapper(step,result=>{
					//step.info(`Completed:`);				
					//step.info(result.substring(0,100)+' (...)');
					step.file(result,`Output = `);					
					pmf.replaceWith(result)
					//pipe.domain[afid].media = af.pathToMediaFile(result);
					
					//step.info(`Parsing silero results`);
					//results = JSON.parse(result);
					//step.ok();
				},{progress:p=>`Processing... ${p}`}));
				step.monitor(aw);	
				let bench = Date.now();			
				let exitStatus = aw.run();			
				let benched = Math.ceil((Date.now()-bench)/1000)+1;
				if(!exitStatus){
					step.info(`Completed in ${benched} seconds`);
					step.ok();
				}else{
					step.error(`Process exited with error code ${exitStatus} after ${benched} seconds`);
					return false;
				}
				continue;
			}// end of media
		}		
		//log.info(`Generated ${createdAnnotations.length} annotations`);
		//return createdAnnotations;
		return finalResult;
	}
}