/** 
@author CG
@log 2024-03-21 initial implementation

@attr folder string (exported-corpus) Name of the folder to create and populate with the corpus media files
@attr overwrite bool (false) Whether to overwrite existing files in the folder, otherwise another folder will be created
@attr copy-temp bool (false) Whether to copy the files even if they come from the temp folder, otherwise temporary files are just moved to destination folder. If you use processors after export-corpus-media, you should activate this attribute so AVAA still finds the temporary files.

Exports current corpus media files to a folder.
The folder will be created in AVAA's temp directory.

**/
proc['export-corpus-media'] = {	
	pipe:pipe=>{		
		const isPreview = api.isPreviewMode();
		if(isPreview) return; //TODO
		const overwrite = attr['overwrite']=='true';
		const copyTempFiles = attr['copy-temp']=='true';
		const folderName = attr['folder']||'exported-corpus';
		let folderPath = api.atk.getTempPath()+'/'+folderName;
		let folderFile = new java.io.File(folderPath);
		let attempt = 0;
		if(!overwrite){
			while(folderFile.exists()){
				attempt++;
				folderPath = api.atk.getTempPath()+'/'+folderName+'-'+attempt;
				folderFile = new java.io.File(folderPath);
				if(attempt>1000){
					log.error(`Exceeded attempts to create a directory`);
					return false;
				}
			}
		}
		folderFile.mkdir();
		if(!folderFile.isDirectory()){
			log.error(`This folder could not be created: ${folderPath}`);
			return false;
		}				
		for(let f of pipe.domain){
			let step = log.addStep(f);
			if(!f.media.size()) {
				step.error(`No media file available in ${f.id}`);		
				continue;
			}
			for(let pmf of f.media){
				let targetFile = new java.io.File(folderPath+'/'+pmf.originalMediaFile.name);
				if(overwrite && targetFile.exists()){
					step.info(`The file already exists and will be overwritten`);
					targetFile.delete();
				}
				
				if(pmf.isTemp()){
					step.file(targetFile,`Moving temporary corpus file to `)								
					step.work();
					org.apache.commons.io.FileUtils.moveFile(pmf.getFile(), targetFile);	
				}else{
					step.file(targetFile,`Copying to `)								
					step.work();
					org.apache.commons.io.FileUtils.copyFile(pmf.getFile(), targetFile);	
				}
				step.ok();
			}
		}		
	}
}
