/** 
@author CG
@log 2024-11-12 initial implementation
@experimental

@attr folder string (exported-corpus) Name of the folder to create and populate with the corpus files. If the name contains a slash (/) it will be considered as an absolute folder path, otherwise the folder is created in AVAA's temp directory.
@attr overwrite bool (false) Whether to overwrite existing files in the folder, otherwise another folder will be created
@attr copy-temp bool (false) Whether to copy the files even if they come from the temp folder, otherwise temporary files are just moved to destination folder. If you use processors after export-corpus-standalone, you should activate this attribute so AVAA still finds the temporary files.

Exports current corpus media files together with a copy of the ORIGINAL corpus files edited to reference the exported media files. This produces a standalone corpus folder which can be easily shared because it does not contains absolute paths anymore.

**/
proc['export-corpus-standalone'] = {	
	pipe:pipe=>{		
		const isPreview = api.isPreviewMode();
		if(isPreview) return; //TODO
		const overwrite = attr['overwrite']=='true';
		const copyTempFiles = attr['copy-temp']=='true';
		const folderName = attr['folder']||'exported-corpus';
		let folderPath = api.atk.getTempPath()+'/'+folderName;
		if(folderName.indexOf('/')>-1) folderPath = folderName; // 
		let folderFile = new java.io.File(folderPath);
		let attempt = 0;
		if(!overwrite && folderName.indexOf('/')<0){
			while(folderFile.exists()){
				attempt++;
				folderPath = api.atk.getTempPath()+'/'+folderName+'-'+attempt;
				folderFile = new java.io.File(folderPath);
				if(attempt>1000){
					log.error(`Exceeded attempts to create a directory`);
					return false;
				}
			}
		}
		folderFile.mkdir();
		if(!folderFile.isDirectory()){
			log.error(`This folder could not be created: ${folderPath}`);
			return false;
		}				
		for(let f of pipe.domain){
			let step = log.addStep(f);
			if(!f.media.size()) {
				step.error(`No media file available in ${f.id}`);		
				continue;
			}
			let relativeAF = new java.io.File(folderPath+'/'+f.af.file.name);
			if(relativeAF.exists()){			
				if(overwrite) relativeAF.delete();
				else {
					step.warn(`Annotation File ${relativeAF.getAbsolutePath()} already exists, not overwriting, aborted.`)
					continue;
				}
			}			
			step.copyFile(f.af.toRelativePaths(), relativeAF, `Writing `);		
			
			for(let pmf of f.media){
				let targetFile = new java.io.File(folderPath+'/'+pmf.originalMediaFile.name);
				if(targetFile.exists()){
					if(overwrite) targetFile.delete();
					else {
						step.warn(`Media File ${targetFile.getAbsolutePath()} already exists, not overwriting, aborted.`)
						continue;
					}
				}						
				if(pmf.isTemp()){ //TODO copy-temp
					step.moveFile(pmf.getFile(),targetFile,`Moving temporary corpus file to `);
				}else{
					step.copyFile(pmf.getFile(),targetFile,`Copying to `);					
				}				
			}			
		}		
	}
}
