/** 
@author CG
@log 2024-01-21 initial implementation
@experimental

@attr copy-media bool (false) Whether to copy the associated media files next to EAF file, to use as relative path (good for sharing). Otherwise absolute path of media will be used (good for testing)
@attr export-empty-tiers bool (false) Whether to also exports the tiers that don't have any annotation


Exports current corpus to EAF format.

**/
proc['export-to-eaf'] = {	
	pipe:pipe=>{		
		const isPreview = api.isPreviewMode();
		if(isPreview) return; //TODO
		const copyMedia = attr['copy-media']=='true';
		const exportEmptyTiers = attr['export-empty-tiers']=='true';
		const author = 'AVAA Toolkit';		
		const date = (new Date()).toISOString();
		const docFile = api.doc.file;
		const folder = docFile.getParent()+'/'+api.doc.basename+" - EAF";
		const folderFile = new java.io.File(folder);
		folderFile.mkdir();
		if(!folderFile.isDirectory()){
			log.error(`This folder could not be created: ${folder}`);
			return false;
		}
		//throw folder;
		for(let f of pipe.domain){
			let step = log.addStep(f);
			if(!f.media.size()) {
				step.error(`No media file available in ${f.id}`);		
				continue;
			}
			let eaf = makeEAF(step,f);			
			let fname = f.af.id+".eaf";
			let outFile = api.writeFile(folder+'/'+fname,eaf);
			step.file(outFile.getAbsolutePath(),`Saved EAF `)
		}
		
		function makeEAF(step,domainFile){
			let pmf = domainFile.media.get(0);
			let mediaURL = 'file:///'+pmf.getAbsolutePath();
			let relativeMediaURL = './'+pmf.getFile().getName();			
			let timeSlots = [];
			let annotations = [];
			let tiers = {};
			let orderedTiers = domainFile.af.tiers.keySet();
			step.info(orderedTiers);
			for(let tier of orderedTiers){
				//step.info(tier);
				if(!tiers[tier]) tiers[tier] = [];
				let T = tiers[tier];
				for(let annotation of domainFile.af.tiers.get(tier).annotations){
					let ts = timeSlots.length+1;
					timeSlots.push(`		<TIME_SLOT TIME_SLOT_ID="ts${ts}" TIME_VALUE="${annotation.start}"/>`);
					timeSlots.push(`		<TIME_SLOT TIME_SLOT_ID="ts${ts+1}" TIME_VALUE="${annotation.stop}"/>`);
					let anEAF = `
		<ANNOTATION>
            <ALIGNABLE_ANNOTATION ANNOTATION_ID="a${annotations.length+1}"
                TIME_SLOT_REF1="ts${ts}" TIME_SLOT_REF2="ts${ts+1}">
                <ANNOTATION_VALUE>${esc(annotation.value)}</ANNOTATION_VALUE>
            </ALIGNABLE_ANNOTATION>
        </ANNOTATION>`;
					T.push(anEAF);
					annotations.push(anEAF);				
				}
				if(!exportEmptyTiers && !T.length){
					delete tiers[tier];
				}
					
					
			}				
			step.info(`Exported ${Object.keys(tiers).length} tiers`);
			step.info(`Exported ${annotations.length} annotations`);
			
			if(copyMedia){
				let path = folder+'/'+domainFile.af.id+'.'+pmf.getExtension();
				step.file(path,'Copying media ');
				let copiedFile = pmf.copyTo(path)				
				mediaURL = relativeMediaURL = './'+copiedFile.getName();
			}else{
				step.file(pmf,'Using media ');
			}				
			
			let template = `<?xml version="1.0" encoding="UTF-8"?>
<ANNOTATION_DOCUMENT AUTHOR="${author}" DATE="${date}"
    FORMAT="3.0" VERSION="3.0"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="http://www.mpi.nl/tools/elan/EAFv3.0.xsd">
    <HEADER MEDIA_FILE="" TIME_UNITS="milliseconds">
        <MEDIA_DESCRIPTOR MEDIA_URL="${mediaURL}"
            MIME_TYPE="video/mp4" RELATIVE_MEDIA_URL="${relativeMediaURL}"/>        
    </HEADER>
    <TIME_ORDER>
${timeSlots.join("\n")}
    </TIME_ORDER>
    <TIER LINGUISTIC_TYPE_REF="default-lt" TIER_ID="default"/>    
	${Object.keys(tiers).map(k=>'<TIER LINGUISTIC_TYPE_REF="default-lt" TIER_ID="'+esc(k)+'">'+tiers[k].join('')+"\n</TIER>").join("\n")}    
    <LINGUISTIC_TYPE GRAPHIC_REFERENCES="false"
        LINGUISTIC_TYPE_ID="default-lt" TIME_ALIGNABLE="true"/>
    <CONSTRAINT
        DESCRIPTION="Time subdivision of parent annotation's time interval, no time gaps allowed within this interval" STEREOTYPE="Time_Subdivision"/>
    <CONSTRAINT
        DESCRIPTION="Symbolic subdivision of a parent annotation. Annotations refering to the same parent are ordered" STEREOTYPE="Symbolic_Subdivision"/>
    <CONSTRAINT DESCRIPTION="1-1 association with a parent annotation" STEREOTYPE="Symbolic_Association"/>
    <CONSTRAINT
        DESCRIPTION="Time alignable annotations within the parent annotation's time interval, gaps are allowed" STEREOTYPE="Included_In"/>
</ANNOTATION_DOCUMENT>`;
		return template;
		}
		//<PROPERTY NAME="URN">urn:nl-mpi-tools-elan-eaf:d2000c9b-af5a-413e-82c7-a7886bcc4983</PROPERTY>
	}
}
