/** 
@author CG
@log 2024-01-29 initial implementation
@experimental
@input Array<Annotation>

@attr copy-media bool (false) Whether to copy the associated media files next to SRT file

Exports a selection of annotations to SRT format

@dependency milliseconds-to-time.js

**/
proc['export-to-srt'] = {	
	pipe:pipe=>{		
		const isPreview = api.isPreviewMode();
		if(isPreview) return; //TODO
		const copyMedia = attr['copy-media']=='true';
		const exportEmptyTiers = attr['export-empty-tiers']=='true';
		const author = 'AVAA Toolkit';		
		const date = (new Date()).toISOString();
		const docFile = api.doc.file;				
		const folder = docFile.getParent()+'/'+api.doc.basename+" - SRT";
		const folderFile = new java.io.File(folder);
		
		if(!annotations.size()){
			log.error(`you must provide a selection of annotations`)
		}
		
		folderFile.mkdir();
		if(!folderFile.isDirectory()){
			log.error(`This folder could not be created: ${folder}`);
			return false;
		}
		
		for(let f of pipe.domain){
			let step = log.addStep(f);
			let pmf = f.media.get(0);
			let fileAnnotations = annotations.toArray().filter(a=>a.af==f.af);
			if(!fileAnnotations.length){
				step.info(`No annotation found in the selection for this file`);
				continue;
			}
			if(!f.media.size()) {
				step.error(`No media file available in ${f.id}`);		
				continue;
			}			
			let srt = makeSRT(step,f,fileAnnotations);
			let fname = pmf.getFile().getName()+".srt";
			let outFile = api.writeFile(folder+'/'+fname,srt);
						
			step.file(outFile,`Exported ${fileAnnotations.length} annotations to`);
			
			if(copyMedia){
				let path = folder+'/'+pmf.getFile().getName();
				step.file(path,'Copying media ');
				let copiedFile = pmf.copyTo(path)				
				mediaURL = relativeMediaURL = './'+copiedFile.getName();
			}else{
				step.file(pmf,'Using media ');
			}				
		}
		function formatTimeSRT(ms){
			let a = millisecondsToTime(ms).split('.');
			while(a[1].length<3) a[1] = a[1] + '0';
			return a.join(',');
		}
		function makeSRT(step,domainFile,annotations){
			let i = 0;
			let s = '';
			for(a of annotations){				
				s+=`${++i}\n${formatTimeSRT(a.start)} --> ${formatTimeSRT(a.stop)}\n${a.value}\n\n`;
			}
				//f.write(f"{start_segment},000 --> {end_segment},000\n")
            return s;
		}
      
	}
}
