/** 
@author CG
@log 2025-01-25 initial implementation
@experimental

@todo attr export-empty-tiers bool (false) Whether to also export the tiers that don't have any annotation
@attr mode select (TEI corpus) [TEI corpus,TEI divisions,TEI files] The method used to export the annotations into TEI
@attr structure select (speaker) [speaker,utterance] The structure used to represent annotations
@attr corpus-name string (tei-corpus) The name of the exported corpus
@attr corpus-subtitle string () Optional subtitle for exported corpus
@attr extension string (.tei.xml) The extension of exported TEI files
@attr author string () The author to add to TEI file description

@dependency sort-annotations.js

Exports selection of annotations to TEI format.
(Synchronization markers not implemented yet)


**/
PROC = {
	pipe:pipe=>{		
		const isPreview = api.isPreviewMode();
		if(isPreview || api.isEstimateMode()) return; //TODO
		const xmlHeader = '<?xml version="1.0" encoding="UTF-8"?>\n';
		const corpusName = attr['corpus-name'] || 'tei-corpus';
		const corpusSubtitle = attr['corpus-subtitle'] || '';
		const exportMode = attr['mode'] || 'TEI corpus';
		const exportStructure = attr['structure'] || 'speaker';
		const fileExtension = attr['extension'] || '.tei.xml';
		const copyMedia = attr['copy-media']=='true';
		const exportEmptyTiers = attr['export-empty-tiers']=='true';
		const author = attr['author'] || '';
		const date = (new Date()).toISOString();
		const makeTEI = exportStructure=='speaker' ? makeTEI_speaker : makeTEI_utterance;
		if(!annotations.size()){
			log.error(`you must provide a selection of annotations`);
			return false;
		}
		const folderName = attr['folder']||'Exported TEI';
		let folder = api.getTempFolder(folderName);
		let attempt = 0;
		while(folder.exists()){
			attempt++;
			folder = api.getTempFolder(folderName+'-'+attempt);				
			if(attempt>1000){
				log.error(`Exceeded attempts to create a directory`);
				return false;
			}
		}
		folder.mkdir();
		if(!folder.isDirectory()){
			log.error(`This folder could not be created: ${folderPath}`);
			return false;
		}		
		const teiHeader = `<teiHeader>
	<fileDesc>
		<titleStmt>
			<title type="main">${corpusName}</title>
			<title type="subtitle">${corpusSubtitle}</title>
			<author>${author}</author>
		</titleStmt>
		<publicationStmt>
			<p>Corpus generated by <ref target="https://avaa-toolkit.org">AVAA Toolkit</ref> ${date}</p>
		</publicationStmt>
	</fileDesc>
</teiHeader>
`;
		var xmlCorpus = xmlHeader + xmlCorpus + `<teiCorpus xmlns="http://www.tei-c.org/ns/1.0">\n` + teiHeader;		
		var xmlDivisions = xmlHeader + '<TEI>\n' + teiHeader + '<text><body>\n';
		for(let f of pipe.domain){
			let step = log.addStep(f);
			let fileAnnotations = annotations.toArray().filter(a=>a.af==f.af);
			if(!fileAnnotations.length){
				step.info(`No annotation found in the selection for this file`);
				continue;
			}			
			let teiXML = makeTEI(step,fileAnnotations,f,{author:author,subtitle:corpusSubtitle});	
			if(exportMode=='TEI files'){
				let fname = f.af.id+fileExtension;
				let outFile = api.writeFile(folder.getAbsolutePath()+'/'+fname,xmlHeader+teiXML);
				step.file(`Saved TEI file`,outFile);
			}
			if(exportMode=='TEI corpus'){
				xmlCorpus+=teiXML+"\n";
			}
			if(exportMode=='TEI divisions'){
				let cxml = teiXML.split('<body>')[1].split('</body>')[0].trim();				
				xmlDivisions+=`\t<div xml:id="${f.af.id}">\n${cxml}\n\t</div>\n`;
			}
		}		
		if(exportMode=='TEI corpus'){
			xmlCorpus+=`</teiCorpus>\n`;
			let step = log.addStep("TEI Corpus");			
			let fname = corpusName+fileExtension;
			let outFile = api.writeFile(folder.getAbsolutePath()+'/'+fname,xmlCorpus);
			step.file(`Saved TEI corpus`,outFile);			
			api.export(outFile);
		}
		if(exportMode=='TEI divisions'){			
			xmlDivisions+=`</body></text></TEI>`;
			let step = log.addStep("TEI Divisions File");			
			let fname = corpusName+fileExtension;
			let outFile = api.writeFile(folder.getAbsolutePath()+'/'+fname,xmlDivisions);
			step.file(`Saved TEI file`,outFile);			
			api.export(outFile);
		}		
		if(exportMode=='TEI files'){
			api.export(folder);
		}
		
	}
}
	

function makeTEI_utterance(step,fileAnnotations,domainFile,opt){
	opt = opt || {};
	const author = '';
	const date = (new Date()).toISOString();
	const tiers = {};
	let body = '';
	fileAnnotations = sortAnnotations(fileAnnotations);
	let previousTier;
	for(let annotation of fileAnnotations){
		let tier = annotation.tier.id;
		if(!tiers[tier]) tiers[tier] = 1;
		//tiers[tier].push(annotation);// relevant?
		let tierChanged = tier!=previousTier;
		if(tierChanged){
			if(previousTier) body+=`\n\t\t</u>\n`;
			previousTier = tier;
			body+=`\t\t<u who="#${esc(tier)}">`; //\n\t\t\t<speaker>${tier}</speaker>
		}
		body+=`\n\t\t\t<seg>${esc(annotation.value)}</seg>`;
	}				
	if(previousTier) body+=`\n\t\t</u>`;
	step.info(`Exported ${Object.keys(tiers).length} tiers`);
	step.info(`Exported ${fileAnnotations.length} annotations`);
	return `<TEI xmlns="http://www.tei-c.org/ns/1.0">
<teiHeader>
	<fileDesc>
		<titleStmt>
			<title type="main">${opt.title || domainFile.af.id}</title>
			<title type="subtitle">${opt.subtitle||''}</title>
			<author>${opt.author||''}</author>
		</titleStmt>
		<publicationStmt>
			<p>Generated by <ref target="https://avaa-toolkit.org">AVAA Toolkit</ref> ${date}</p>
		</publicationStmt>
	</fileDesc>
</teiHeader>
<listPerson type="tiers">
	${Object.keys(tiers).map(k=>'<person xml:id="'+esc(k)+'" />').join("\n\t")}
</listPerson>
<text>
	<body>
${body}
	</body>
</text>
</TEI>`;
}

function makeTEI_speaker(step,fileAnnotations,domainFile,opt){
	opt = opt || {};
	const date = (new Date()).toISOString();
	const tiers = {};
	let body = '';
	fileAnnotations = sortAnnotations(fileAnnotations);
	let previousTier;
	for(let annotation of fileAnnotations){
		let tier = annotation.tier.id;
		if(!tiers[tier]) tiers[tier] = 1;
		//tiers[tier].push(annotation);// relevant?
		let tierChanged = tier!=previousTier;
		if(tierChanged){
			if(previousTier) body+=`\n\t\t</sp>\n`;
			previousTier = tier;
			body+=`\t\t<sp who="#${esc(tier)}">`; //\n\t\t\t<speaker>${tier}</speaker>
		}
		body+=`\n\t\t\t<p>${esc(annotation.value)}</p>`;
	}				
	if(previousTier) body+=`\n\t\t</sp>`;
	step.info(`Exported ${Object.keys(tiers).length} tiers`);
	step.info(`Exported ${fileAnnotations.length} annotations`);
	return `<TEI xmlns="http://www.tei-c.org/ns/1.0">
<teiHeader>
	<fileDesc>
		<titleStmt>
			<title type="main">${opt.title || domainFile.af.id}</title>
			<title type="subtitle">${opt.subtitle||''}</title>
			<author>${opt.author||''}</author>
		</titleStmt>
		<publicationStmt>
			<p>Generated by <ref target="https://avaa-toolkit.org">AVAA Toolkit</ref> ${date}</p>
		</publicationStmt>
	</fileDesc>
</teiHeader>
<list>
	<head>Tiers</head>
	${Object.keys(tiers).map(k=>'<item xml:id="'+esc(k)+'"><name>'+k+"</name></item>").join("\n\t")}
</list>
<text>
	<body>
${body}
	</body>
</text>
</TEI>`;
}