/** 
@author CG
@license Noise Removal Neural Network Models https://github.com/GregorR/rnnoise-models | No
@log 2023-12-04 initial implementation
@experimental
@cachable

@attr method string () [FFT,RNN,NLM] The denoise method to use
@attr rnn-model string () [beguiling-drafter,conjoined-burgers,leavened-quisling,marathon-prescription,somnolent-hogwash,xiph-std] 
@attr rnn-mix number (1) How much to mix filtered samples into final output. Allowed range is from -1 to 1. Default value is 1. Negative values are special, they set how much to keep filtered noise in the final filter output
This processor calls ffmpeg's denoise feature.
- FFT: Denoises audio with FFT.
- NLM: Reduces broadband noise using a Non-Local Means algorithm.
- RNN: Reduces noise from speech using Recurrent Neural Networks model.

Learn more about the RNN models at https://github.com/GregorR/rnnoise-models

@credits https://superuser.com/questions/733061/reduce-background-noise-and-optimize-the-speech-from-an-audio-clip-using-ffmpeg

@dependency proc-ffmpeg-wrapper.js

**/
proc['ffmpeg-denoise'] = {	
	pipe:pipe=>{		
		const isPreview = api.converter.getSetting('test-mode')=='true'; //todo api?		
		const attrMethod = attr['method'] || 'FFT';
		const attrModel = attr['rnn-model'] || 'beguiling-drafter';
		
		if(attrMethod!='FFT'){
			log.error(`Sorry, only FFT method is implemented for now`);
			//TODO;
			return false;
		}		
		if(data && data.length){			
			log.warn(`Denoise currently does not handle input annotations, only pipe domain`);
		}			
		for(let f of pipe.domain){			
			let step = log.addStep(f);			
			if(!f.media.size()) {
				step.error(`No media file available in ${f.id}`);		
				continue;
			}			
			for(let pmf of f.media){
				if(api.isEstimateMode()){
					api.converter.progressTodo(100*1000);
					continue;
				}
				step.file(pmf,`Input = `);			
				if(api.isPreviewMode()){
					step.info(`Not denoising because preview mode.`)
					continue;
				}			
				let outputFile = pipe.temp(pmf);
				step.file(outputFile,`Output = `)
				//let filter = `arnndn="${api.doc.atk.path}/scripts/dependencies/rnnoise/${attrModel}.rnnn"`
				let filter = `afftdn`;//TODO
				//step.debug(filter);
				let ffargs = [
					'-i',pmf.getAbsolutePath(),
					'-af',`"${filter}"`,
					outputFile.getAbsolutePath()
				]			
				step.info(`Running denoise...`)
				step.work();
				
				let rs = ffmpegWrapper(step,ffargs,{
					onProgress:o=>api.converter.progressDone(o.percentChunk*1000),
				});
				if(!rs) return false;
				pmf.replaceWith(outputFile);			
			} // end of media
		} // end of domain file
	}
}
