/** 
@author CG
@log 2023-11-30 initial implementation

@cachable

@attr filter-complex string () The filter expression

@dependency proc-ffmpeg-wrapper.js

This processor calls ffmpeg with a user defined filter-complex.
Learn more about what filters can do at https://www.ffmpeg.org/ffmpeg-filters.html

**/
proc['ffmpeg-filter-complex'] = {	
	pipe:pipe=>{		
		const isPreview = api.isPreviewMode();
		const attrFilter = attr['filter-complex'];		
		const attrFormat = attr['format']; //TODO?
		if(!attrFilter){
			log.error(`you must specify the filter-complex attribute`);
			return false;
		
		for(let f of pipe.domain){
			let step = log.addStep(f);
			if(!f.media.size()) {
				step.error(`No media file available in ${eafid}`);		
				continue;
			}			
			for(let pmf of f.media){
				if(api.isEstimateMode()) {
					api.converter.progressTodo(100*1000);
					continue;
				}
				step.file(pmf,`Input = `);	
				if(isPreview){
					step.info(`Not processing because preview mode.`)
					continue;
				}				
				let targetFile = pipe.temp(pmf);
				step.file(targetFile,`Output = `);
				let ffargs = ['-y',
					'-i',pmf.getAbsolutePath(),
					'-cf','"'+attrFilter+'"'				
				]
				if(attrFormat){
					ffargs.push(attrFormat);				
				}
					
				ffargs.push(targetFile.getAbsolutePath())						
				step.info(`Running audio filter...`)
				step.work();
				let rs = ffmpegWrapper(step,ffargs,{
					onProgress:o=>api.converter.progressDone(o.percentChunk*1000),
				});
				if(!rs) return false; // abort pipe
				pmf.replaceWith(targetFile);
				continue;	
			} // end of media
 		} // end of domain file		
	}
}
