/** 
@author CG
@license Video filters by https://github.com/dyne/frei0r | GPL-2.0
@license Frei0r DLL pack for Windows by Gyan Doshi https://www.gyan.dev/ffmpeg | CC BY-NC-ND 4.0
@log 2023-12-11 initial implementation
@experimental
@cachable

@attr filter string () [,3dflippo,addition,addition_alpha,aech0r,alpha0ps_alpha0ps,alpha0ps_alphagrad,alpha0ps_alphaspot,alphaatop,alphain,alphainjection,alphaout,alphaover,alphaxor,B,balanc0r,baltan,bgsubtract0r,bluescreen0r,brightness,burn,bw0r,c0rners,cartoon,cluster,colgate,coloradj_RGB,colordistance,colorhalftone,colorize,colortap,color_only,composition,contrast0r,curves,d90stairsteppingfix,darken,defish0r,delay0r,delaygrab,denoise_hqdn3d,difference,distort0r,dither,divide,dodge,edgeglow,elastic_scale,emboss,equaliz0r,flippo,G,gamma,glitch0r,glow,grain_extract,grain_merge,hardlight,hue,hueshift0r,IIRblur,invert0r,ising0r,kaleid0sc0pe,keyspillm0pup,lenscorrection,letterb0xed,levels,lighten,lightgraffiti,lissajous0r,luminance,mask0mate,measure_pr0be,measure_pr0file,medians,multiply,ndvi,nervous,nois0r,normaliz0r,nosync0r,onecol0r,overlay,partik0l,perspective,pixeliz0r,pixs0r,plasma,posterize,premultiply,primaries,R,RGB,rgbnoise,rgbparade,rgbsplit0r,saturat0r,saturation,scale0tilt,scanline0r,screen,select0r,sharpness,sigmoidaltransfer,sleid0r_push-down,sleid0r_push-left,sleid0r_push-right,sleid0r_push-up,sleid0r_slide-down,sleid0r_slide-left,sleid0r_slide-right,sleid0r_slide-up,sleid0r_wipe-barn-door-h,sleid0r_wipe-barn-door-v,sleid0r_wipe-circle,sleid0r_wipe-down,sleid0r_wipe-left,sleid0r_wipe-rect,sleid0r_wipe-right,sleid0r_wipe-up,sobel,softglow,softlight,sopsat,spillsupress,squareblur,subtract,tehroxx0r,test_pat_B,test_pat_C,test_pat_G,test_pat_I,test_pat_L,test_pat_R,threelay0r,three_point_balance,threshold0r,timeout,tint0r,transparency,twolay0r,uvmap,value,vectorscope,vertigo,vignette,xfade0r] The frei0r filter to use

@attr param-1 string () A parameter to pass to the filter
@attr param-2 string () A parameter to pass to the filter
@attr param-3 string () A parameter to pass to the filter
@todo @attr param-4 string () A parameter to pass to the filter
@todo @attr param-5 string () A parameter to pass to the filter

@todo api.isLinux() ? https://github.com/dyne/frei0r/releases extract the .so files in /usr/lib/frei0r-1  

@dependency proc-ffmpeg-wrapper.js

Applies a frei0r filter on each corpus media file.

**/
proc['ffmpeg-frei0r'] = {	
	pipe:pipe=>{		
	
		const FREI0R_PATH = api.atk.path+'/scripts/dependencies/frei0r';
		if(!attr.filter) return log.error(`you must specify the "filter" attribute`);
		
		
		const attrFilter = attr['filter'];		
		const attrParams = [];
		for(let i=1;i<=5;i++){
			let v = attr[`param-${i}`];
			if(v) attrParams.push(v);
		}
		for(let f of pipe.domain){
			let step = log.addStep({name:f.id});
			if(!f.media.size()) {
				step.error(`No media file available in ${f.id}`);		
				continue;
			}			
			for(let pmf of f.media){
			    if(!pmf.exists()){
			        step.error(`The file does not exist: ${pmf.getAbsolutePath()}`);
			        continue;
			    }
				if(api.isEstimateMode()){
					api.converter.progressTodo(pmf.mediaFile.getDetails().getDurationMS());		
					continue;
				}
				step.file(pmf,`Input = `);	
				if(api.isPreviewMode()){
					step.info(`Not applying because preview mode.`)
					continue;
				}			
				let outputFile = pipe.temp(pmf);
				step.file(outputFile,`Output = `)
				//TODO fix multiple params
				let fparams = '';
				if(attrParams.length){
					fparams += attrParams.join('|');
				}
				
				let ffFilter = api.isWindows() ?
				`"frei0r=filter_name=${attrFilter}:filter_params=${fparams},format=yuv420p"` :
				`frei0r=filter_name=${attrFilter}:filter_params=${fparams},format=yuv420p`;
				
				let ffargs = [
					'-i',pmf.getAbsolutePath(),										
					'-vf',ffFilter,					
					outputFile.getAbsolutePath()
				];
				
				step.info(ffargs.join(' '));
				step.info(`Starting filter ${attrFilter}...`)
				step.work();				
				
				let rs = ffmpegWrapper(step,ffargs,{
					env:{FREI0R_PATH:FREI0R_PATH},
					onProgress:o=>api.converter.progressDone(o.chunk),
					trap:s=>{
						if(s.substr(0,14)=='configuration:'){
							if(s.indexOf('--enable-frei0r')==-1){
								step.error(`ffmpeg is not built with frei0r enabled`);
								step.error(`please install a frei0r compatible ffmpeg version`);							
							}else{
								//step.info(`frei0r is enabled`);
							}					
						}
					}
				});
				if(!rs){
					step.error(`Media file "${pmf.getAbsolutePath()}" could not be processed`);
					continue;
					//pipe.abort
				}
				pmf.replaceWith(outputFile);				
			}
		}		
	}
}
