/** 
@author CG
@log 2023-11-30 initial implementation
@experimental
@cachable

@attr file file () .R file to run
@attr source script () Plain R source code to run
@todo Add to source R functions to help interface with AVAA
@dependency proc-exec-output-wrapper.js
@dependency proc-r-utils.js

This processor calls R with user provided data, and integrates the resulting data into the final HTML page.
Resulting R output can be graphic files (jpg, png, gif, svg) or tabular text data.
Input provided to R as arguments are in order:
- temp directory path to work with and create result files
- path to a JSON file consisting of the selection (annotations) or data provided to the processor

R scripts must follow a specific input/output syntax to be compatible with AVAA (see "Calling R" in the scripting guide).


**/
proc['r-script'] = {	
	pipe:pipe=>{		
		const isPreview = api.isPreviewMode();
		const attrFile = attr['file'];		
		const attrSource = attr['source'];
		if(!attrFile){
			log.error(`you must specify the file attribute`);
			return false;
		}
		
		const step = log.addStep({name:'R'})
		//step.info(dataJSON)
		if(!RCheck(step)) return;
				//step.file(pmf,`Input = `);
				let tempdir = api.atk.getTempPath();
				let xargs = ['Rscript',`"${attrFile}"`,`"${tempdir}"`];
				//if(isPreview) xargs.push('--fake')				
				let aw = api.exec(xargs);
				step.monitor(aw);
				let progress = '';
				aw.onError(s=>{		
					if(!s) return;
					step.error(`Process Error: ${s}`)
				})
				aw.onOutput(execOutputWrapper(step,result=>{
					step.info(`Copying result to assets...`);
					//step.info(result);										
					let src = result.trim();
					let srcFile = new java.io.File(src);
					let target = "assets/" + srcFile.getName();										
					let targetFile = new java.io.File(api.doc.atk.path + "/" + target);
					let ext = org.apache.commons.io.FilenameUtils.getExtension(src).toLowerCase();
					api.doc.dependencies.put(target,"asset");
					org.apache.commons.io.FileUtils.copyFile(srcFile,targetFile);
					api.doc.html.append(`<img src="${api.toBaseURL(target)}"/>\n`)
					step.ok();					
					
				}));						
				let bench = Date.now();
				let exitStatus = aw.run();			
				let benched = Math.ceil((Date.now()-bench)/1000)+1;
				if(!exitStatus){
					step.info(`Completed in ${benched} seconds`);
					step.ok();
				}
				else {
					step.error(`Process exited with error code ${exitStatus} after ${benched} seconds`);
					return false;
				}				
				//continue;	
			//} // end of media
 		//} // end of domain file
		
		return true;
	}
}

