/** 
@author CG
@log 2024-11-17 initial implementation

@attr exclude bool (false) Wheter to exclude the filtered files instead of keeping them
@attr file regexp () Which media file from the corpus to keep (regular expression)
@attr filter js () A custom filter function called for each file that should return true to keep the file, like (mf)=>mf.extension.includes('mp4')

Filters media files from the corpus, keeping only files that match a specific critera.
The "exclude" attribute can be used to alternatively exclude these files from the corpus.
Useful to work on a subset of the corpus media files without modifying the corpus itself.

**/
PROC = {
	pipe:pipe=>{		
		if(!attr.file && !attr.filter){
			log.warn(`Please specify a criteria`)
			return;
		}
		
		const exclude = attr.exclude==='true';
		const filterFunc = attr.filter ? new Function('return '+attr.filter)() : null;		
		const step = log.addStep({name:`Reducing Corpus Media`});		
		
		
		for(let f of pipe.corpus){			
			let mediaFilesToKeep = [];
			for(let mf of f.media){
				let keep=false;				
				if(attr.file && mf.getFile().getName().match(attr.file)) keep = true;			
				if(filterFunc && filterFunc(f.mediaFile)) keep = true;
				if(!keep) continue;			
				mediaFilesToKeep.push(mf);			
			}
			if(exclude){
				for(let mf of mediaFilesToKeep){
					step.info(`Removing ${mf.getFile().getName()}`);
					f.media.remove(mf);
				}
			}else{
				f.media.clear();
				for(let mf of mediaFilesToKeep){
					step.info(`Keeping ${mf.getFile().getName()}`);
					f.media.add(mf);
				}
			}
		}	
				
		
	}
}
