/** 
@author CG
@log 2024-01-22 initial implementation

@input Array<Annotation>

Takes a selection of annotations and modifies the corpus, using the input annotations as sequences, each sequence being removed from the corpus file, with its associated media segment and all the annotations included in that sequence.

@dependency proc-ffmpeg-wrapper.js
@dependency milliseconds-to-time.js
**/
proc['remove-sequences-from-corpus'] = {	
	pipe:pipe=>{		
		const isPreview = api.isPreviewMode();		
		const author = 'AVAA Toolkit';		
		const date = (new Date()).toISOString();
		
		const saveID = attr['save-id'] || '';
		const doc = api.doc;
		
		if(!annotations.size()){
			log.error(`No annotation was provided`);
			return false;
		}
		
		const asort = (A,B)=>{ return A.start > B.start ? 1 : -1};
		
		for(let f of pipe.domain){
			let list = [];
			
			for(let a of annotations){
				if(a.af.id == f.id){
					list.push(a);
				}
			}
			if(!list.length) continue;
			let pmf = f.media[0];
			list.sort(asort);
			let segmentFilter = '';
			let step = log.addStep(f);
			//,format=yuv420p
			step.info(`Found ${list.length} sequences to remove`);
			let pos = 0;
			let mediaEnd = pmf.mediaFile.getDetails().duration*1000;
			// basically, we want to cut and join the other segments
			let segments = [];
			let shiftAfter = {}; // keep info to shift or remove annotations accordingly
			for(let a of list){
				shiftAfter[a.start] = (a.stop - a.start);
				segments.push({start:pos,stop:a.start});
				pos = a.stop;
			}			
			if(pos<mediaEnd){
				segments.push({start:pos,stop:mediaEnd});
			}							
			var count=0;
			for(let a of segments){
				//Input1:
				let start = a.start/1000;
				let end = a.stop/1000;
				step.info(`Keeping segment from ${msToTime(a.start)} to ${msToTime(a.stop)}`);	
				segmentFilter+=`[0:v]trim=start=${start}:end=${end},setpts=PTS-STARTPTS[${count}v];`;
				segmentFilter+=`[0:a]atrim=start=${start}:end=${end},asetpts=PTS-STARTPTS[${count}a];`;
				count++;
			}
			// now let's remove/shift/resize cropped annotations
			let removed = [];
			let shifted = [];				
			let resized = [];
			var currentCutShift;
			for(let a of f.af.annotations.values()){
				currentCutShift = 0;
				for(let cutStart in shiftAfter){
					cutStart = cutStart*1 + currentCutShift;
					let cutDuration = shiftAfter[cutStart];
					currentCutShift += cutDuration;
					let cutStop = cutStart + cutDuration;
					if(a.stop<=cutStart) continue; // annotation is completely before the cut, skip.
					if(a.start>=cutStart && a.stop<=cutStop){ // annotation is completely in the cut, remove.
						if(removed.indexOf(a)<0) removed.push(a);							
						continue;
					}
					if(a.start>=cutStop){ // annotation is completely after the cut
						a.start -= cutDuration; // shift both start and stop
						a.stop -= cutDuration;
						if(shifted.indexOf(a)<0) shifted.push(a);
						continue;
					}
					if(a.start < cutStart){ // annotation starts before the cut,
						if(a.stop<=cutStop) { // but stops before the cut end
							a.stop = cutStart; // so let's reduce it													
						}else{ // but stops after the cut
							a.stop -= cutDuration; // reduce from the whole cut							
						}
						if(resized.indexOf(a)<0) resized.push(a);
						continue;
					}
					// annotation starts in the cut but stops after the cut, only this case left:					
					a.start = cutStop - cutDuration; // shift start
					a.stop -= cutDuration; // shift stop
					if(shifted.indexOf(a)<0) shifted.push(a);
					
				}				
			}
			for(let a of removed){
				f.af.removeAnnotation(a);
			}
			step.info(`Adapting annotations: ${removed.length} removed, ${shifted.length} shifted, ${resized.length} resized`);
			
			if(isPreview){
				step.info(`That's all for preview mode.`);
				continue;	
			}
			
			
			let filter = segmentFilter;
			for(let i=0;i<count;i++) filter+=`[${i}v][${i}a]`;
			filter+=`concat=n=${count}:v=1:a=1[outv][outa]`
			//out.mp4
			//step.warn(filter);
			let croppedFile = pipe.temp(f.media[0]);
			let ffargs = [				
				'-i',f.media[0].getAbsolutePath(),
				'-filter_complex',`"${filter}"`,
				'-map','[outv]',
				'-map','[outa]',
				croppedFile.getAbsolutePath()
			]
			step.file(croppedFile,'Cropping to ');			
			let rs = ffmpegWrapper(step,ffargs,{
				//expectedDuration:a.getDuration(),
				//onProgress:o=>api.converter.progressDone(o.chunk)
			});
			if(!rs) pipe.abort();
			pmf.replaceWith(croppedFile);
		}		
	}
}
