/** 
@author CG
@log 2024-08-21 added file attribute
@log 2024-02-06 initial implementation

@attr file regexp () Restrict file(s) on which to rename tiers
@attr from string () The tier name to be replaced
@attr to string () The replacement name to use
@attr map js () A JSON object mapping the names to replace (keys) to the new names (values), like {"OLD NAME":"NEW NAME"}
@todo attr name-template js (`${name}`) The template function to build the name of each tier, called with the original tier name and object, like (name,tier) => `${name.toUpperCase()}}`

Rename tiers in all or specific corpus files.

**/
proc['rename-tiers'] = {	
	pipe:pipe=>{		
		
		const isPreview = api.isPreviewMode();
		const copyMedia = attr['copy-media']=='true';
		const restrictFile = attr.file;
		const author = 'AVAA Toolkit';		
		const date = (new Date()).toISOString();
		
		const saveID = attr['save-id'] || '';
		const doc = api.doc;
		
		var map = attr.map ? JSON.parse(attr.map) : null;
		
		if(!map && !attr.from){
			log.error(`must specify the "from" attribute`);
			return false;
		}
		if(!map && !attr.to){
			log.error(`must specify the "to" attribute`);
			return false;
		}
		if(!map) map = {};
		if(attr.from && attr.to) map[attr.from] = attr.to;
		
		let nameTemplate = attr['name-template'] || '';
		if(!nameTemplate) nameTemplate = (name,tier) => `${name}`;
		else {
			if(nameTemplate.charAt(0)=='`') nameTemplate = '(name,tier)=>'+nameTemplate;
			nameTemplate = new Function('return '+nameTemplate)();
		}
		
		for(let f of pipe.domain){
			if(restrictFile){
				if(!f.af.id.match(restrictFile)) continue;
			}
			let step = log.addStep(f);			
			for(let k in map){
				let tier = f.af.getTier(k);
				if(!tier){
					step.warn(`Tier "${k}" not found`);
					continue;
				}		
				//TODO nameTemplate
				tier.id = map[k];
				f.af.tiers.remove(k);
				f.af.tiers.put(tier.id,tier);
				step.info(`Renamed "${k}" to "${map[k]}"`);
				step.ok();
			}
		}
		
	}
}
