/** 
@author CG
@log 2025-01-05 added template attribute
@log 2024-08-21 added file attribute
@log 2024-02-06 initial implementation

@attr file regexp () Restrict file(s) on which to rename tiers
@attr from string () A single tier name to be replaced
@attr to string () The replacement name to use with the "from" attribute
@attr map js () A JSON object mapping the names to replace (keys) to the new names (values), like {"OLD NAME":"NEW NAME"}
@attr template js (`${name}`) A template function to build the name of each tier, called with the original tier name and object, like (name,tier) => `${name.toUpperCase()}`. When this attribute is specified all corpus tiers will be affected.

Rename tiers in all or specific corpus files.

**/
proc['rename-tiers'] = {	
	pipe:pipe=>{		
		
		const isPreview = api.isPreviewMode();
		const copyMedia = attr['copy-media']=='true';		
		const restrictFile = attr.file;
		const author = 'AVAA Toolkit';		
		const date = (new Date()).toISOString();
		
		const saveID = attr['save-id'] || '';
		const doc = api.doc;
		
		var map = attr.map ? JSON.parse(attr.map) : null;
		
		
		if(!map && !attr.from && !attr.template){
			log.error(`must specify the "from" attribute`);
			return false;
		}
		if(!map && !attr.to && !attr.template){
			log.error(`must specify the "to" attribute`);
			return false;
		}
		if(!map) map = {};
		if(attr.from && attr.to) map[attr.from] = attr.to;
		
		let nameTemplate = attr['template'] || '';
		if(!nameTemplate) nameTemplate = (name,tier) => `${name}`;
		else {
			if(nameTemplate.charAt(0)=='`') nameTemplate = '(name,tier)=>'+nameTemplate;
			nameTemplate = new Function('return '+nameTemplate)();
		}
		
		for(let f of pipe.domain){
			if(restrictFile){
				if(!f.af.id.match(restrictFile)) continue;
			}
			let step = log.addStep(f);			
			if(nameTemplate && !Object.keys(map).length){
				let allTiers = f.af.tiers.values().toArray();
				let renamed = 0;
				for(let t of allTiers){
					let newID = nameTemplate(t.id,t);
					if(t.id==newID) continue;
					f.af.tiers.remove(t.id);					
					t.id = newID;
					f.af.tiers.put(newID,t);
					renamed++;
				}
				step.info(`Renamed ${renamed} tiers`);
				continue;
			}
			for(let k in map){
				let tier = f.af.getTier(k);
				if(!tier){
					step.warn(`Tier "${k}" not found`);
					continue;
				}						
				tier.id = map[k];
				f.af.tiers.remove(k);
				f.af.tiers.put(tier.id,tier);
				step.info(`Renamed "${k}" to "${map[k]}"`);
				step.ok();
			}
		}
		
	}
}
