/** 
@author CG
@log 2024-01-22 initial implementation

@input Array<Annotation>
@attr name-template js (`sequence-${i}-${a.af.id}`) The template function to build the name of each corpus file, provided with the annotation and its index, like (a,i) => `${i} - ${a.value}`

Takes a selection of annotations and recreates the corpus, using the input annotations as sequences, each sequence being transformed into one corpus file with its associated media file and all the annotations/tiers included in that sequence.

@dependency proc-ffmpeg-wrapper.js

**/
proc['sequences-to-corpus'] = {	
	pipe:pipe=>{		
		const isPreview = api.isPreviewMode();
		const copyMedia = attr['copy-media']=='true';
		const author = 'AVAA Toolkit';		
		const date = (new Date()).toISOString();
		
		const saveID = attr['save-id'] || '';
		const doc = api.doc;
		
		let nameTemplate = attr['name-template'] || '';
		if(!nameTemplate) nameTemplate = (a,i) => `sequence-${i}-${a.af.id}`;
		else {
			if(nameTemplate.charAt(0)=='`') nameTemplate = '(a,i)=>'+nameTemplate;
			nameTemplate = new Function('return '+nameTemplate)();
		}
		/*if(!doc.saves.containsKey(saveID)){
			log.error(`No saved slot with that id: ${saveID}`);
			return;
		}
		const save = doc.saves.get(saveID);
		if(!save.annotations.size()){
			log.error(`No annotation was found in the saved slot`);
			return;
		}*/		
		if(!annotations.size()){
			log.error(`No annotation was provided`);
			return false;
		}
		let names = [];
		let idx = 0;
		for(let a of annotations){
			idx++;
			let fname = nameTemplate(a,idx);
			if(names.indexOf(fname.toLowerCase())>-1){
				log.error(`There is a domain filename collision, this name is already in the list: "${fname}"`);
				log.warn(`You must adapt the "name-template" attribute`);
				return false;
			}
			names.push(fname.toLowerCase());
		}		
		pipe.domain.clear();
		idx=0;
		for(let a of annotations){
			//let fname = a.value+'@'+a.start;
			idx++;
			let fname = nameTemplate(a,idx);
			let step = log.addStep({name:fname});
			let media = a.tier.af.getPipedMedia();			
			let mediaCut = pipe.temp(media);						
			let ffargs = [
				'-ss',a.start/1000,
				'-i',media.file.getAbsolutePath(),
				'-c','copy',							
				'-t',(a.stop-a.start)/1000,
				mediaCut.getAbsolutePath()
			]			
			//blackmagic internal stuff
			let af = new org.avaatoolkit.formats.AnyMediaFile(doc);	
			af.id = fname;
			af.file = mediaCut;
			let mediaFile = new org.avaatoolkit.model.MediaFile(af, mediaCut.getAbsolutePath());
			af.mediaFiles.add(mediaFile);			
			//step.info(a.tier.af.annotations.size())
			
			for(let aa of a.tier.af.annotations.values()){
				if(aa.start >= a.start && aa.start <= a.stop && aa.stop <= a.stop){
					aa = af.addAnnotation(aa.clone());
					aa.start -= a.start;
					aa.stop -= a.start;					
					//step.info('FOUND ANNOTATION '+aa)
				}
			}
			step.info(`Imported ${af.tiers.size()} tiers and ${af.annotations.size()} annotations`);
			step.file(mediaCut,'Cut to ');
			
			let domainFile = new org.avaatoolkit.script.PipeDomainFile(af);
			pipe.domain.add(domainFile);
			let rs = ffmpegWrapper(step,ffargs,{
				expectedDuration:a.getDuration(),
				onProgress:o=>api.converter.progressDone(o.chunk)
			});
			if(!rs) pipe.abort();
			
		}		
	}
}
