/** 
@author CG
@log 2023-12-10 initial implementation

@input Domain
@input Pipe<Files>
@output Domain
@output Pipe<Annotations>

@dependency proc-exec-output-wrapper.js
@dependency proc-python-wrapper.js

The face recognition detects faces coordinates

**/
proc['face-recognition'] = {	
	pipe:pipe=>{		
		const isPreview = api.isPreviewMode();
		const isEstimate = api.isEstimateMode();
		
		if(isEstimate) return;
		const attrSource = attr['source'] || 'vocals';
		
		let req = log.addStep({name:'Requirements'})
		if(!pythonCheck(req)) return false;
		if(!pipCheck("face_recognition",req)) return false;
		let wrapperFile = pythonWrapper("avaa-face-recognition",req);
		if(!wrapperFile) return false;
		
		log.info(pipe);
		log.info(pipe.domain);
		for(let afid in pipe.domain){
			pipe.domain[afid].step = log.addStep({name:afid})			
		}
		var finalResult = undefined;
		for(let afid in pipe.domain){
			let f = pipe.domain[afid];
			let af = f.af;
			let step = f.step;			
			var media = f.media; // get first media file referenced in af			
			if(!media) {
				step.error(`No media file available in pipe domain ${eafid}`);
				continue;
			}			
			
			let sourceFile = api.cleanPath(media.file.getAbsolutePath());
			step.file(sourceFile,`Input = `);
			
			if(media.extension!='wav'){
				step.info(`Converting to WAV...`)
				step.work();
				let wavFile = api.doc.atk.path+"/assets/temp-demucs-"+Date.now()+".wav";				
				let rx = api.exec(['ffmpeg','-i',sourceFile,wavFile]).run();
				if(rx){
					step.error(`Something went wrong converting to WAV file: ${wavFile}`);
					return false;
				}
				sourceFile = wavFile;
				step.ok();
				step.file(wavFile,`Converted to `)
			}
			
			
			step.info(`Launching Demucs process`)
			step.work();
			let xargs = ['python','-u',wrapperFile.getAbsolutePath(),
				'-i',sourceFile,								
			];
			if(isPreview) xargs.push('--fake')			
			let aw = api.exec(xargs);
			let progress = '';
			aw.onError(s=>{
				step.error(`A process error occurred: ${s}`)
			})			
			aw.onOutput(execOutputWrapper(step,result=>{
				//step.info(`Completed:`);				
				//step.info(result.substring(0,100)+' (...)');
				step.file(result,`Output = `);
				finalResult = result;
				pipe.domain[afid].media = af.pathToMediaFile(result);
				//step.info(`Parsing silero results`);
				//results = JSON.parse(result);
				//step.ok();
			},{progress:p=>`Processing... ${p}`}));
			step.monitor(aw);	
			let bench = Date.now();			
			let exitStatus = aw.run();			
			let benched = Math.ceil((Date.now()-bench)/1000)+1;
			if(!exitStatus){
				step.info(`Completed in ${benched} seconds`);
				step.ok();
			}else{
				step.error(`Process exited with error code ${exitStatus} after ${benched} seconds`);
				return false;
			}
			continue;
		}		
		//log.info(`Generated ${createdAnnotations.length} annotations`);
		//return createdAnnotations;
		return finalResult;
	}
}