/** 
@author CG
@log 2024-03-11 initial implementation
@label add multiple media files 
@attr files files () The files to load
@attr name string (untitled bulk) The name of this corpus 
@attr group string () The group to associate to all the files
@attr tags tags () A list of tags to associate to this corpus
@attr separate bool (false) 

Simplify bulk processing by group-loading many media files 

**/
provider.bulk = {
	load:function(){
		attr.name = attr.name || ('untitled bulk')
		const step = log.addStep({name:attr.name});
		if(!attr.files){
			step.error(`No files were specified`);
			return;
		}
		let j;
		try {
			j = JSON.parse(attr.files);
		}catch(ex){
			step.warn(ex.message);
		}
		if(!j){
			step.error(`Could not parse files attribute: "${attr.files}"`)
			return;
		}          
		if(!j.length){
			step.error(`No files selected`)
			return;
		}
							
		let af = new org.avaatoolkit.formats.Generic(api.doc);		
		af.group = attr.group || '';	
		if(attr.tags) af.tags.addAll(attr.tags.split(','));
		af.id = attr.name;
		af.domainKey = domainKey; // must pass current special domainKey variable //TODO refactor possible?
		api.doc.domain.add(af);				
		for(let file of j){
			if(!file) continue;
			if(!file.path) continue;
			let f = new java.io.File(file.path);
			if(!f.exists()){
				step.warn(`File "${file.path}" does not exist`);
				continue;
			}					
			af.addMediaFile(f);				
		}
		step.info(`Loaded ${af.mediaFiles.size()} files`);
		step.ok();
		return af;
	}
}