/** 
@author CG
@log Provides corpus mapping for the online annotation platform https://celluloid.huma-num.fr/ 
@log 2024-01-25 initial implementation
@label add a Celluloid project
@attr project string () The celluloid project id, to load annotations and media from
@attr download-media bool (true) Whether to download the associated video file
@attr resolution string (720p) [720p,480p,144p] The resolution of the downloaded project media videos
@attr group string () The group to be associated with this corpus file and all its annotations
@attr tags tags () A list of tags associated to this corpus file
@dependency human-file-size.js

**/

provider.celluloid = {
	load:function(){
		const downloadMedia = attr['download-media']=='false' ? false : true;
		const projectID = attr.project;
		const group = attr.group || '';
		const resolution = attr.resolution || '720p';
		if(!projectID) throw `missing "project" attribute`;
		
		//log.info('PROJECT ID '+projectID);
		const details = this.getProjectDetails(projectID);
		const step = log.addStep({name:details.title});
		
		const annotations = this.getAnnotations(projectID);				
		let af = new org.avaatoolkit.formats.PeerTubeFile(api.doc,{
			instance:'celluloid-media.huma-num.fr',
			instanceName:'celluloid',
			url:'https://celluloid-media.huma-num.fr/w/'+details.videoId,
			resolution:resolution
		});	
		/*
		 "id": "4878b2b9-706b-46b7-af9b-b2a0eb23eae7",
                "videoId": "5GPbYsxX97Kxo8uLviWG4i",
                "userId": "215a5717-9024-4241-8411-f4216bc1a236",
                "title": "L1 Humanités numériques 4/4",
                "description": "Découverte de nouveaux outils dans le cadre du programme \"territoires du numérique »",
                "host": "celluloid-media.huma-num.fr",
                "publishedAt": "2023-02-16T20:47:30.320Z",
                "public": true,
                "collaborative": true,
                "shared": true,
                "shareCode": null,
                "shareExpiresAt": null,
                "extra": {},
                "playlistId": null,
                "duration": 294,
                "thumbnailURL": "https://celluloid-media.huma-num.fr/lazy-static/thumbnails/4f32d39b-cb35-4412-8bfe-2821e7f9534f.jpg",
                "keywords": [],
                "user": {
                    "id": "215a5717-9024-4241-8411-f4216bc1a236",
                    "username": "LoTessier",
                    "role": "Admin",
                    "avatar": null,
                    "initial": "L",
                    "color": "#4af9a8"
                },
				*/
		af.id = projectID;
		af.group = group;
		if(attr.tags) af.tags.addAll(attr.tags.split(','));
		af.filename = details.title.split('/').join('-');
		af.domainKey = domainKey;
		af.file = af.originalFile = new java.io.File(api.atk.path + '/corpus/providers/celluloid/'+projectID+'_'+resolution+".mp4");
		for(let a of annotations) af.addAnnotation(a);
		api.doc.domain.add(af);
		let numTiers = af.tiers.size();
		step.info(`Loaded ${annotations.length} annotations from ${numTiers} tier${numTiers>1 ? 's' : ''}`);
		if(!af.file.exists()){			
			if(downloadMedia){
				step.file(af.file,`Downloading media to `);
				step.info(`Downloading 0b`);					
				af.onDownloadProgress = p=>{
					step.replace(`Downloading ${humanFileSize(p.downloaded)}b ...`);
				}
				af.download();
				step.ok();
			}else{
				step.info(`Not downloading media.`)
			}
		}else{
			step.info(`Media file already downloaded`);
		}
		af.addMediaFile(af.file);
	},
	domain:(domain)=>{
		
	},
	getProjectDetails:function(project){
		const url = `https://celluloid.huma-num.fr/trpc/project.byId?batch=1&input=%7B%220%22%3A%7B%22id%22%3A%22`+project+`%22%7D%7D`;
		const content = api.fetch(url);
		if(!content || content.charAt(0)!='[') throw `invalid content @ ${url}`;
		let r = JSON.parse(content);
		if(!r) throw `failed decoding JSON content`;
		return r[0].result.data;
	},
	getAnnotations:function(project){
		const url = `https://celluloid.huma-num.fr/trpc/annotation.byProjectId?batch=1&input=%7B%220%22%3A%7B%22id%22%3A%22`+project+`%22%7D%7D`;
		const content = api.fetch(url);
		if(!content || content.charAt(0)!='[') throw `invalid content @ ${url}`;
		let r = JSON.parse(content);
		if(!r) throw `failed decoding JSON content`;
		const list = [];
		for(let o of r[0].result.data){
			let annotation = {
				id:o.id,
				start:o.startTime*1000,
				stop:o.stopTime*1000,
				value:o.text,
				tier:o.user ? o.user.username || o.user.id : o.userId
			}
			list.push(annotation);
		}
		return list;
	}
	
}


