/** 
@author CG
@log 2024-08-07 initial implementation
@label add a PRAAT TextGrid file
@hidden
@attr file file () The .TextGrid file to load
@attr group string () The group to associate to this corpus file and all its annotations
@attr tags tags () A list of tags associated to this corpus file

Provides corpus mapping for PRAAT annotation format

@dependency tei-corpo.js
@register extension textgrid
**/
provider.praat = {
	load:function(){
		const step = log.addStep({name:`Loading TextGrid`});
		if(!attr.file){
			step.error(`No .TextGrid file was specified`);
			return;
		}
				
		const file = api.getFile(attr.file);
		if(!file.exists()){
			step.error(`File "${attr.file}" was not found`);
			return;
		}
		
		// we use TeiCorpo to convert to AVAA native EAF format
		const eafFile = TeiCorpo.toEAF(file,step);		
		step.ok();
				
		// now we just build an EAF annotation file instance
		step.info(`Loading...`);		
		step.work();
		const af = api.createAnnotationFile(org.avaatoolkit.formats.EAF);
		af.setSourceFile(eafFile);
		af.parse();
        step.ok();
				
		if(attr.group) af.group = attr.group; // associate group
		if(attr.tags) af.tags.addAll(attr.tags.split(',')); // associate tags
		
		api.doc.corpus.add(af); // finally add annotation file to corpus
		
		step.info(`Found ${af.tiers.size()} tiers and ${af.annotations.size()} annotations`);
		
		return af;		
	}
}