/** 
@author CG
@input Array<Annotation>
@attr show-header bool (false) Whether to display table header
@attr show-index bool (true) Whether to display index column
@attr show-value bool (true) Whether to display annotation's text value
@attr show-video bool (true) Whether to display annotation's video clip
@attr show-tier bool (true) Whether to display tier's name
@attr show-file bool (false) Whether to display annotation file name
@attr cotext int (0) Number of cotext annotations to show before and after (0 = no cotext)
@attr cotext-range number (0) Max number of seconds to consider a neighbor annotation as a valid cotext annotation (0 = no limit)
@attr cotext-media bool (false) Whether to extend the clip to include the cotext annotations (make sure to use a reasonable cotext-range to avoid super long clips)
@attr extend-duration number (0) Number of seconds to add before and after when cutting the clip
@attr label-save string (SAVE) Save button label
@deprec @attr cv string () optional controlled vocabulary for form input
@deprec @attr choices string () optional comma separated values to use as form inputs
@attr name string () a name to identify this form, will be used as filename and accept field parameters interpolation
@param +dimension="name" label="label" cv="controlled vocabulary id"
@param +dimension="name" label="label" choices="list,of,choices"
@param +dimension="name" label="label" type="text"
@param +field="name" label="label" type="text"
@param .label
@param .type
@param .cv
@param .choices

The form view renders HTML input forms, allowing easy online sharing for collecting external data.
Form results are simple JSON files which can then be imported back as virtual tiers for further analysis.

**/
view.form = {
	list:list=>{
		var s = `<table>`;
		var showIndex = !(attr['show-index']=='false');
		var showHeader = !(attr['show-header']=='false');
		var showValue = !(attr['show-value']=='false');
		var showVideo = !(attr['show-video']=='false');
		var showTier = !(attr['show-tier']=='false');
		var showFile = attr['show-file']=='true';
		var cotext = attr['cotext']*1 || 0;
		var cotextRange = attr['cotext-range']*1 || 0;
		var cotextMedia = !(attr['cotext-media']=='false');
		var extendDuration = attr['extend-duration']*1 || 0;
		//var cv = attr.cv ? attr.cv+'' : '';
		
		
		
		var names = {};
		var fields = [];
		var dimensions = [];
		params.forEach(opt=>{ 
			if(opt.hasAttribute("field")){
				var f = {name:opt.getAttribute("field")}								
				f.label = opt.hasAttribute("label") ? opt.getAttribute("label")+'' : '';
				f.type = opt.hasAttribute("type") ? opt.getAttribute("type")+'' : 'text';
				fields.push(f);
			}
			if(opt.hasAttribute("dimension")){
				var f = {name:opt.getAttribute("dimension")}								
				f.label = opt.hasAttribute("label") ? opt.getAttribute("label")+'' : '';
				if(opt.hasAttribute("cv")){
					f.type = 'select';
					f.cv = opt.getAttribute('cv');
				}
				else if(opt.hasAttribute("choices")){
					f.type = 'select';
					f.choices = opt.getAttribute('choices');
				}
				else {
					f.type = opt.hasAttribute("type") ? opt.getAttribute("type")+'' : 'text';
				}
				dimensions.push(f);
			}
		});		
		
		if(showHeader){
			s += `<thead><tr>`;
			if(showIndex) s += `<td class="th-index"></td>`;
			if(showFile) s += `<td class="th-af"></td>`;
			if(showTier) s += `<td class="th-tier"></td>`;
			if(showValue) s += `<td class="th-value"></td>`;
			if(showVideo) s += `<td class="th-video"></td>`;
			dimensions.forEach(d=>{
				s += `<td class="th-dimension">${d.label || d.name}</td>`;
			});
			s += `</tr></thead>`;
		}

		s += `<tbody>`;
		
		
		var representation = {
			options:{
				cotext:cotext,
				cotextRange:cotextRange,
				cotextMedia:cotextMedia,
				extendDuration:extendDuration
			},
			dimensions:dimensions,
			fields:fields
		}
		const asort = (A,B)=>{ return A.start > B.start ? 1 : -1};
		function getCotext(a,dist,dir,range){			
			let list = a.tier.annotations.toArray();
			list.sort(asort);
			let index = list.indexOf(a);
			if(index==-1){
				log.warn(`could not find annotation for getCotext(${a.id})`)
				return;
			}
			index = index + dist*dir;
			//log.info('getCotext indexOf "'+a.id+'" "'+a.value+'" = '+index+' of '+list.length);
			
			//if(a.start < aa.start) ret.push(aa);
			if(index<0) return;
			if(index>=list.length) return;
			if(range && (Math.abs(a.start - list[index].start)>range*1000)) return;
			return list[index];
		}
		
		var idx=0;
		//FIX: 
		list.forEach(a=>{			
			//FIX: 
			idx++;
			var placeholder = ''; //TODO
			var ref = {				
				eaf:a.af.file.getName(), //legacy
				af:a.af.file.getName(),
				tier:a.tier.id,
				annotation:a.id,
				value:a.value,
				start:a.start,
				stop:a.stop
			};
			ref = JSON.stringify(ref);
			//ref.d = cv;			
			s+=`<tr data-ref="${btoa(ref)}">`;			
			if(showIndex) s += `<td class="td-index">${idx}</td>`;
			if(showFile) s += `<td class="td-af">${a.af.id}</td>`;
			if(showTier) s += `<td class="td-tier">${esc(a.tier.id)}</td>`;
			if(cotext){
				s += `<td class="td-value">`;
				for(let i=cotext;i>0;i--){
					let coa = getCotext(a,i,-1,cotextRange)
					if(!coa) continue;
					a.extra.formCotextStart = coa.start;
					s += `<span class="cotext cotext-before cotext-${i}" title="Cotext -${msToTime(a.start-coa.start)}">${esc(coa.value)}</span>`;
				}
				s+=`<span class="cotext cotext-current cotext-0">${esc(a.value)}</span>`;
				for(let i=1;i<=cotext;i++){
					let coa = getCotext(a,i,1,cotextRange);
					if(!coa) break;
					a.extra.formCotextStop = coa.stop;
					s += `<span class="cotext cotext-after cotext-${i}" title="Cotext +${msToTime(coa.start-a.start)}">${esc(coa.value)}</span>`;
				}					
				s += `</td>`;
			}else{
				if(showValue) s += `<td class="td-value">${esc(a.value)}</td>`;
			}
			if(showVideo) {
				let aa = a.clone();
				if(cotext && cotextMedia){
					if(a.extra.formCotextStart) aa.start = a.extra.formCotextStart;
					if(a.extra.formCotextStop) aa.stop = a.extra.formCotextStop;
				}
				if(extendDuration){									
					let d = Math.round(extendDuration*1000);
					aa.start -= d;
					if(aa.start<0) aa.start = 0;
					aa.stop += d;
				}				
				s += `<td class="td-video">`
				s += `<video preload="none" poster="${aa.snap()}" controls width="300">`;
				s += `<source src="${aa.clip()}" type="video/mp4"></video>`;
				s += `</td>`;
			}
			
			
			dimensions.forEach(f=>{
				s+=`<td class="user-input" data-dimension="${f.name}">`;
				switch(f.type){
					case 'select':						
						var choices = f.choices ? (f.choices+'').split(",") : [];
						var allowed = choices.length ? choices : a.af.vocabulary.values().toArray().filter(cve=>cve.cv.equals(f.cv)).map(o=>o.value);			
						s+=`<select name="userinput"><option selected value="" disabled> - </option>`;
						if(allowed.length){				
							allowed.forEach(o=>s+=`<option value="${esc(o)}">${esc(o)}</option>`);
						}else{
							s+=`<input type="text" placeholder="${placeholder}" name="userinput" />`
						}
						break;
					default:
						s+=`<input type="text" placeholder="${placeholder}" name="userinput" />`						
				}				
				s+=`</td>`;
			});
			
			
			
			s+=`</tr>`;
		});
		s+=`</tbody></table>`;
		var name = attr.name || 'untitled form';
		s+=`<form name="${btoa(name)}" data-rep="${btoa(JSON.stringify(representation))}">`;
		fields.forEach(f=>{
			s+=`<label>`;
			s+=`<input name="${f.name}" placeholder="${f.label}" type="text" />`;
			s+=`</label>`;
		})
				
		s+=`</form>`
		s+=`<button>${attr['label-save'] || 'SAVE'}</button>`
		
		return s;
	},
	js:s=>{
		$(()=>{
			$('.view-form button').click(function(){
				var list = [];
				var parent =$(this).parent();
				parent.find('table').each((i,table)=>{					
					$(table).find('tbody tr').each((i,tr)=>{
						var ref = $(tr).data('ref');
						ref = JSON.parse(atob(ref));						 
						ref.dimensions = {};
//						ref.input = $(tr).find('td *[name="userinput"]')[0].value || '';
						$(tr).find('td.user-input').each(function(){
							var dimension = $(this).data('dimension');
							var value = $(this).find('*[name="userinput"]')[0].value || '';
							console.info(dimension,value);
							ref.dimensions[dimension]=value;							
						});
						list.push(ref);
					});
				});	
				var fields = [];
				parent.find('form input').each((i,input)=>{					
					var field = {
						name:input.name,
						value:input.value
					}
					fields.push(field);
					
				});	
				console.info(fields);		
				var rep = parent.find('form').data('rep');
				rep = rep ? JSON.parse(atob(rep)) : {};
				var agent='';
				try {
					agent = navigator.userAgent;
				}catch(exc){
				}
				var r = JSON.stringify({
					list:list,
					fields:fields,
					representation:rep,
					agent:agent,
					now:Date.now()
				});
				var name = parent.find('form').attr('name');
				if(name) name=atob(name);
				else name = 'results';				
				fields.forEach(f=>{
					name = name.replace('$'+f.name,f.value || '');
				});
				if(!window.saveAs) return alert("filesaver.js is missing, can't save file.");
				//var b = new Blob([decodeURIComponent(atob($(ev.currentTarget).attr('data-json')))], {type: "text/plain;charset=utf-8"});
				var b = new Blob([r], {type: "text/plain;charset=utf-8"});
				window.saveAs(b, name+'.json', {type:'application/json'});
			});
		});		
	},
	css:s=>{
		return `	
#view video {
    border-radius:4px;
}
#view table {
	border-collapse: collapse;	
}
#view thead {
	font-weight:bold;
}
#view td {
	padding: 5px;
    border: 1px solid gainsboro;
}
#view form {
	padding: 20px;
}
#view button {
	margin-left:20px
}

#view .td-value .cotext.cotext-0 {
    font-weight: bold;
    font-style: normal;
}
#view .td-value .cotext:hover {
	text-decoration: underline;
}
#view .td-value .cotext {
    margin-left: 10px;
	display:block;
}

`
		
	}
}


function msToTime(duration) {
  var milliseconds = Math.floor((duration % 1000) / 100),
    seconds = Math.floor((duration / 1000) % 60),
    minutes = Math.floor((duration / (1000 * 60)) % 60),
    hours = Math.floor((duration / (1000 * 60 * 60)) % 24);

  hours = (hours < 10) ? "0" + hours : hours;
  minutes = (minutes < 10) ? "0" + minutes : minutes;
  seconds = (seconds < 10) ? "0" + seconds : seconds;
//hours + ":" + 
  return minutes + ":" + seconds;
}