/** 
@author CG
@input Array<Annotation>
@attr show-value bool (true) Whether to display annotation's text value
@attr show-media bool (true) Whether to display annotation's media clip
@todo @attr @deprecated show-video bool (true) Whether to display annotation's video clip
@attr show-snap bool (false) Whether to display annotation's snapshot
@attr show-tier bool (true) Whether to display tier's name
@attr class string (list) [list,grid] Alternative display style
@attr file-header html () Raw html to insert as a header before each file
@attr file-separator html () Raw html to insert as a separator between each file
@attr file-footer html () Raw html to insert as a footer after each file
@param file="" name="" | Set a custom name for each file

The list view simply renders annotations one after another. It is possible to specify a custom class for switching display to grid mode.
@sample VIEW show-tier="false" style="grid"
@image grid.jpg
**/
view.list = {
	list:list=>{
		var showValue = !(attr['show-value']=='false');
		var showMedia = !(attr['show-media']=='false');
		var showTier =  !(attr['show-tier']=='false');
		var showSnap =  (attr['show-snap']=='true');
		var fileSeparator = attr['file-separator'] || '';
		var fileHeader = attr['file-header'] || '';
		var fileFooter = attr['file-footer'] || '';
		var fileNames = {};
		params.forEach(opt=>{ 
			if(opt.hasAttribute('file')){
				fileNames[opt.getAttribute('file')+''] = opt.getAttribute("name")+'';
			}
		});
		var s = "";
		var prev;
		list.forEach(a=>{
			var p = prev;
			if(p!=a.af){
				if(p) s += fileFooter;
				if(p) s += fileSeparator;				
				s += fileHeader.split('${file}').join(fileNames[a.tier.af.filename]||a.tier.af.filename);
			}
			s += `<div class="list-item">`;
			if(showMedia) {
				var aClip = a.clip();
				var aSnap = a.snap();
				s += `<video preload="none" poster="${aSnap}" controls width="300">`;
				s += `<source src="${aClip}" type="video/mp4"></video>`;
			}
			if(showSnap) s+= `<img src="${a.snap()}" />`;
			if(showTier) s += `<label class="tier">${esc(a.tier.id)}</label>`;
			if(showValue) s += `<label class="annotation">${esc(a.value)}</label>`;
			
			s += `</div>`;
			if(p!=a.af){			
				//if(p) s += fileFooter;				
			}
			prev = a.af;	
		});
		s+=fileFooter;
		return s;
	},
	css:s=>{
		return `
.view-list video {
    border-radius:4px;
}
.view-list label {
    border-top: 1px dashed gainsboro;
    vertical-align: top;
    padding-left: 10px;
}
.view-list label.tier {
    font-weight: bold;
}
.view-list.no-tier label.tier {
	display:none;
}
.view-list.grid .list-item {
	display:inline-block;
	margin:5px;
}
.view-list.grid video {
	display:block;
`;		
	}
}