/** 
@author CG
@input Array<Annotation>
@input Array<Object>
@deprec @param index="Index"
@deprec @param value="Annotation Value"
@deprec @param video="Video"
@deprec @param snap="Snapshot"
@deprec @param group="Group"
@deprec @param tier="Tier"
@deprec @param participant="Participant"
@deprec @param start="Start timecode"
@deprec @param stop="Stop timecode"
@param +column="index" name="Index"
@param +column="tier" name="Tier"
@param +column="value" name="Annotation Value"
@param +column="video" name="Video"
@param +column="snap" name="Snapshot"
@param +column="start" name="Start timecode"
@param +column="stop" name="Stop timecode"
@param +column="group" name="Group"
@param +column="participant" name="Participant"
@param +column="file" name="File"
@param +column="extra" extra-id="" name="" as="[value,video]"
@param +column="property" property="" name="" as="[text,link]"
@param !name
@param !as
@param !extra-id
@param !property
@todo param-attr
@deprec param *extra
@log 2023-01-03 initial implementation
@log 2023-11-15 now can display Array<Object>
@attr header-name-replace js () A custom function to replace the columns header name

The table view can display various annotations' (or objects) properties into table columns.
It also supports the Extra protocol allowing user-defined properties to be displayed as columns.


@sample VIEW
@sample SET column="tier" name="Actor"
@sample SET column="value" name="Transcript"
@sample SET column="video" name="Video Clip"

**/
const VIEW = {	
	data:input=>{
		
		const headerNameReplace = attr['header-name-replace'] ? new Function('return '+attr['header-name-replace'])() : null;
		
		//var h = ['index','snap','video','clip','group','tier','participant','value','start','stop','eaf'];		
		var combined = {};
		var columns = [];
		var names = {};
		var extras = {};			
		for(let opt of params){
			/*var id, extraid;
			if(opt.hasAttribute("extra")){
				id = opt.getAttribute("extra");
				extras[id] = { 
					name: opt.getAttribute("name") || id,
					as: (opt.getAttribute("as")+'') || ''
				}				
				td.push('extra-'+id);
				names['extra-'+id] = extras[id];
				return;
			}*/
			if(opt.hasAttribute("column")){
				var id = opt.getAttribute("column")+'';
				var c = {
					id:id,
					as:'',
					name: (opt.hasAttribute("name") ? opt.getAttribute("name") : '') || id
				}
				if(id=="extra"){
					c.extra = opt.getAttribute("extra-id");
					c.as = (opt.getAttribute("as")+'') || 'value';
					if(['video','value'].indexOf(c.as)<0) c.as = 'value'; 					
				}
				if(id=="property"){
					c.prop = opt.getAttribute("property");
					c.as = (opt.getAttribute("as")+'') || 'text';
					//if(['text','link'].indexOf(c.as)<0) c.as = 'text'; 
				}
				columns.push(c);				
				continue;
			}
			for(var i = 0; i < opt.getAttributes().getLength();i++){
                var att = opt.getAttributes().item(i).getNodeName()+'';
				columns.push({
					id:att,
					as:'',
					name:(opt.getAttributes().item(i).getTextContent() || att)+""
				});                
			}				
		}			
		if(!input || !input.forEach){
			log.error(`input is not an array`);			
			let keys = Object.keys(input);
			if(keys.length){
				log.warn(`input is an object, use "flatten" operation to convert to array`);
				//log.warn(keys.join(','));
			}
			return false;
		}
		
		if(!columns.length){
			log.info(`no columns were specified, using automatic columns`);
			if(input && input.forEach){//Array.isArray(input)){
				let o = input[0];
				if(!o) throw `input is an empty array?`
				//log.info(o.toString());
				if(o.getExtra){
					columns.push({id:'index',name:'Index'});
					columns.push({id:'file',name:'File'});
					columns.push({id:'tier',name:'Tier'});
					columns.push({id:'start',name:'Start'});
					columns.push({id:'value',name:'Value'});
				}else{
					let allKeys = {};
					for(let oo of input){
						for(let k of Object.keys(oo)){
							if(allKeys[k]) continue;
							let v = oo[k];							
							if(typeof v === 'string'){
								allKeys[k] = {id:'property',prop:k,as:'text',name:k};
							}else if(typeof v === 'number'){
								allKeys[k] = {id:'property',prop:k,as:'text',name:k};
							}else if(Array.isArray(v)){
								allKeys[k] = {id:'property',prop:k,as:'text',name:k};
							}else if(Object.keys(v).length){
								for(let kk of Object.keys(v)){
									allKeys[k+'.'+kk] = {id:'property',prop:k+'.'+kk,as:'text',name:k+'.'+kk};
								}
							}
						}
					}
					let keys = Object.keys(allKeys);
					keys.sort();
					for(let k of keys) {
						allKeys[k].name = headerNameReplace ? headerNameReplace(allKeys[k].name) : allKeys[k].name;
						columns.push(allKeys[k]);
					}
				}
			}else{
				
			}
		}
		//log.info(JSON.stringify(input));
		var list = input;
		
		
		
		//log.info('TDS = '+td.join(','));
		var s = `<table><thead><tr>`;
		for(let td of columns) s+=`<td class="td-${td.id} td-${td.id}-${td.as}">${td.name}</td>`;
		s+=`</tr></thead><tbody>`;
		var idx=0;
		var out = function out(a,att,td){
			if(!a) return; // for eventual empty/null extra
			switch(att){		
					case 'i':
					case 'index':
						s+=idx;
						break;					
					case 'file':
					case 'eaf':					
					case 'af':
						s+=(a.af.filename+"").split('\\').pop();
						break;
					case 'start':						
						s+=ftime(Math.round(a.start/1000));
						break;
					case 'stop':
						s+=ftime(Math.round(a.stop/1000));
						break;						
					case 'snap':
						s+='<img src="'+a.snap()+'" />';
						break;
					case 'clip':
					case 'video':
						s+='<video preload="none" poster="'+a.snap()+'" controls width="300">' +
							'<source src="'+a.clip()+'" type="video/mp4"></video>';
						break;					
					case 'extra':
						//log.info("RENDERING extra "+td.extra + " AS "+td.as+(a.getExtra(td.extra)))
						if(('hasExtra' in a) && !a.hasExtra(td.extra)){
							log.warn(`missing "${td.extra}" extra property`);
						}else{
							out(a.getExtra(td.extra), td.as, td);
						}
						break;
					case 'property':
						//log.info("RENDERING extra "+td.extra + " AS "+td.as+(a.getExtra(td.extra)))
						let k = td.prop;
						let tree = k && k.split ? k.split('.') : [k];
						let v = a[tree.shift()];
						while(tree.length) v = v[tree.shift()];
						if(td.as && td.as.charAt && td.as.charAt(0)=='('){
							let f = new Function('return '+td.as)();
							s+=f(v);
						}
						else if(td.as=='img'){							
							s+=`<img src="${v}" />`;
						}else{
							if(Array.isArray(v)){
								s+=`<ol>`;
								for(let x of v) s+=`<li>${esc(x)}</li>`;
								s+=`</ol>`;
							}else{
								if(v) s+=esc(v)
							}
						}
						//out(a, td.as, td);
						break;						
					case 'value':						
						if(a.extra.combined && a.extra.combined.length){
							s+=`<span class="combined-annotations">`;
							a.extra.combined.forEach(ca=>{								
								s+=`<span data-annotation-id="${ca.af.id+'_'+ca.id}" class="combined-annotation">${esc(ca.value)}</span><span class="combined-annotation-separator">&middot;</span>`;
							});
							s+=`</span>`;
						}else{
							s+=esc(a.get('value'));	
						}
						break;
					case 'text': //data
					case 'link': //data
						s+=esc(a);
						break;						
					default:
						let t = '?'+att+'?';
						if(a && a.get) t = a.get(att);
						else if(a) t=a[att]||'';						
							
						s+=esc(t);
				}
		}
		list.forEach(a=>{
			idx++;			
			s+=`<tr>`;
			columns.forEach(td=>{
				s+=`<td class="td-${td.id} td-${td.id}-${td.as}">`;
				//s+='<td class="td-'+att.split(':')[0]+'">';
				out(a, td.id, td);
				s+=`</td>`;
			});
			s+=`</tr>`;
		});
		s+=`</tbody></table>`;
		return s;
	},
	js:()=>{		
		
		$(()=>{
			$('.view-table').each((i,table)=>{				
				let total = {};
				$(table).find('td .combined-annotations').each((i,td)=>{					
					$(td).find('.combined-annotation').each((i,ca)=>{
						let $ca = $(ca);
						let caid = $ca.data('annotation-id');						
						
						if(!total[caid]) total[caid]=0;
						total[caid]++;
						if(total>5) total = 5;
						$ca.addClass('f-'+total[caid]);									
						$ca.attr('title',caid+' -> '+total[caid]);						
						$ca.mouseover(()=>{
							$(table).find('.combined-annotation[data-annotation-id='+caid+']').toggleClass('combined-annotation-highlighted',true);
						})
						$ca.mouseout(()=>{
							$(table).find('.combined-annotation[data-annotation-id='+caid+']').toggleClass('combined-annotation-highlighted',false);
						})
					})
				});
				for(let caid in total){
					if(total[caid]<2){
						$(table).find('.combined-annotation[data-annotation-id='+caid+']').addClass('combined-annotation-single');
					}
				}
			})
		});
	},
	css:s=>{
		return `
/** test **/		
#view .combined-annotation-single {
	border-bottom: none !important;
}	
#view .combined-annotation-highlighted.f-1 {
	border-bottom: 2px dashed green; 
}
#view .combined-annotation-highlighted {
	border-bottom:2px dashed orange; 
}
#view .combined-annotation.f-1 {
	opacity:1;	
}
#view .combined-annotation.f-2 {
	opacity:0.8;
}
#view .combined-annotation.f-3 {
	opacity:0.6;
}
#view .combined-annotation.f-4 {
	opacity:0.4;
}
#view .combined-annotation.f-5 {
	opacity:0.3;
}
		
#view table {
	border-collapse: collapse;	
}
#view thead {
	font-weight:bold;
}
#view td {
	padding: 5px;
    border: 1px solid gainsboro;
}
#view td.td-snap img {
	max-height:200px;
}
#view span.combined-annotation-separator {
    margin-left: 5px;
    margin-right: 5px;
}
#view span.combined-annotation-separator:last-child {
    display: none;
}
`
	}
}

function ftime(s){
    var sec_num = parseInt(s, 10);
    var hours   = Math.floor(sec_num / 3600);
    var minutes = Math.floor((sec_num - (hours * 3600)) / 60);
    var seconds = sec_num - (hours * 3600) - (minutes * 60);
    if (hours   < 10) {hours   = "0"+hours;}
    if (minutes < 10) {minutes = "0"+minutes;}
    if (seconds < 10) {seconds = "0"+seconds;}
    return hours+':'+minutes+':'+seconds;
}