/** 
@author CG
@input Array<Annotation>
@input Array<Object>
@deprec @param index="Index"
@deprec @param value="Annotation Value"
@deprec @param video="Video"
@deprec @param snap="Snapshot"
@deprec @param group="Group"
@deprec @param tier="Tier"
@deprec @param participant="Participant"
@deprec @param start="Start timecode"
@deprec @param stop="Stop timecode"
@param +column="index" name="Index"
@param +column="tier" name="Tier"
@param +column="value" name="Annotation Value"
@param +column="video" name="Video"
@param +column="snap" name="Snapshot"
@param +column="start" name="Start timecode"
@param +column="stop" name="Stop timecode"
@param +column="group" name="Group"
@param +column="participant" name="Participant"
@param +column="file" name="File"
@param +column="extra" extra-id="" name="" as="[value,video]"
@param +column="property" property="" name="" as="[text,link]"
@param !name
@param !as
@param !extra-id
@param !property
@todo param-attr
@deprec param *extra
@log 2023-01-03 initial implementation
@log 2023-11-15 now can display Array<Object>

The table view can display various annotations' (or objects) properties into table columns.
It also supports the Extra protocol allowing user-defined properties to be displayed as columns.


@sample VIEW
@sample SET column="tier" name="Actor"
@sample SET column="value" name="Transcript"
@sample SET column="video" name="Video Clip"

**/
view.table = {	
	list:list=>{
		//var h = ['index','snap','video','clip','group','tier','participant','value','start','stop','eaf'];		
		var columns = [];
		var names = {};
		var extras = {};			
		params.forEach(opt=>{ 		
			/*var id, extraid;
			if(opt.hasAttribute("extra")){
				id = opt.getAttribute("extra");
				extras[id] = { 
					name: opt.getAttribute("name") || id,
					as: (opt.getAttribute("as")+'') || ''
				}				
				td.push('extra-'+id);
				names['extra-'+id] = extras[id];
				return;
			}*/
			if(opt.hasAttribute("column")){
				var id = opt.getAttribute("column")+'';
				var c = {
					id:id,
					as:'',
					name: (opt.hasAttribute("name") ? opt.getAttribute("name") : '') || id
				}
				if(id=="extra"){
					c.extra = opt.getAttribute("extra-id");
					c.as = (opt.getAttribute("as")+'') || 'value';
					if(['video','value'].indexOf(c.as)<0) c.as = 'value'; 					
				}
				if(id=="property"){
					c.prop = opt.getAttribute("property");
					c.as = (opt.getAttribute("as")+'') || 'text';
					//if(['text','link'].indexOf(c.as)<0) c.as = 'text'; 
				}
				columns.push(c);				
				return;
			}
			for(var i = 0; i < opt.getAttributes().getLength();i++){
                var att = opt.getAttributes().item(i).getNodeName()+'';
				columns.push({
					id:att,
					as:'',
					name:(opt.getAttributes().item(i).getTextContent() || att)+""
				});                
			}				
		});		
		//log.info('TDS = '+td.join(','));
		var s = `<table><thead><tr>`;
		columns.forEach(td=>s+=`<td class="td-${td.id} td-${td.id}-${td.as}">${td.name}</td>`);	
		s+=`</tr></thead><tbody>`;
		var idx=0;
		var out = function out(a,att,td){
			if(!a) return; // for eventual empty/null extra
			switch(att){		
					case 'i':
					case 'index':
						s+=idx;
						break;					
					case 'file':
					case 'eaf':					
					case 'af':
						s+=(a.af.filename+"").split('\\').pop();
						break;
					case 'start':						
						s+=ftime(Math.round(a.start/1000));
						break;
					case 'stop':
						s+=ftime(Math.round(a.stop/1000));
						break;						
					case 'snap':
						s+='<img src="'+a.snap()+'" />';
						break;
					case 'clip':
					case 'video':
						s+='<video preload="none" poster="'+a.snap()+'" controls width="300">' +
							'<source src="'+a.clip()+'" type="video/mp4"></video>';
						break;					
					case 'extra':
						//log.info("RENDERING extra "+td.extra + " AS "+td.as+(a.getExtra(td.extra)))
						out(a.getExtra(td.extra), td.as, td);
						break;
					case 'property':
						//log.info("RENDERING extra "+td.extra + " AS "+td.as+(a.getExtra(td.extra)))
						let k = td.prop;
						let tree = k && k.split ? k.split('.') : [k];
						let v = a[tree.shift()];
						while(tree.length) v = v[tree.shift()];
						if(td.as && td.as.charAt && td.as.charAt(0)=='('){
							let f = new Function('return '+td.as)();
							s+=f(v);
						}
						else if(td.as=='img'){							
							s+=`<img src="${v}" />`;
						}else s+=esc(v)
						//out(a, td.as, td);
						break;						
					case 'value':
						s+=esc(a.get('value'));
						break;
					case 'text': //data
					case 'link': //data
						s+=esc(a);
						break;						
					default:
						s+=esc(a.get(att));
				}
		}
		list.forEach(a=>{
			idx++;			
			s+=`<tr>`;
			columns.forEach(td=>{
				s+=`<td class="td-${td.id} td-${td.id}-${td.as}">`;
				//s+='<td class="td-'+att.split(':')[0]+'">';
				out(a, td.id, td);
				s+=`</td>`;
			});
			s+=`</tr>`;
		});
		s+=`</tbody></table>`;
		return s;
	},
	css:s=>{
		return `
#view table {
	border-collapse: collapse;	
}
#view thead {
	font-weight:bold;
}
#view td {
	padding: 5px;
    border: 1px solid gainsboro;
}
#view td.td-snap img {
	max-height:200px;
}`
	}
}

function ftime(s){
    var sec_num = parseInt(s, 10);
    var hours   = Math.floor(sec_num / 3600);
    var minutes = Math.floor((sec_num - (hours * 3600)) / 60);
    var seconds = sec_num - (hours * 3600) - (minutes * 60);
    if (hours   < 10) {hours   = "0"+hours;}
    if (minutes < 10) {minutes = "0"+minutes;}
    if (seconds < 10) {seconds = "0"+seconds;}
    return hours+':'+minutes+':'+seconds;
}