/** 
@author CG
@input Array<Annotation>
@attr collapse int (60) Max number of seconds between 2 annotations beyond which a separator indicating collapsed duration is displayed
@attr show-value bool (true) Whether to display the annotation's text value 
@attr show-video bool (true) Whether to display the annotation's video clip 
@attr show-timestamp bool (true) Whether to display the timestamp
@attr group bool (false) Whether to separate annotations based on their group instead of their file
@attr sequence string Separate annotations based on another tier's annotations (used as sequences)
@todo sequence-relative-time bool (false) Whether to display time relative to sequence start
@param tier="id" name="Custom Name" | Set a custom header name for each tier
@param group="id" name="Custom Name" | Set a custom header name for each group
@param file="id" name="Custom Name" | Set a custom header name for each file
@param column="tier id" | Set a custom order for each tier
@param !name
@log 2023-02-25 added sequence attribute
@log 2023-02-11 initial implementation
@todo handle offsets and group times
@todo times and duration localization
The timeline view displays annotations vertically with time markers, and using one column per tier.
Ideal for dialogs between 2+ participants.

@sample VIEW collapse="60"
@image jpg
**/
view.timeline = {
	list:list=>{		
		var collapse = attr.collapse ? attr.collapse*1 || 60 : 60; 
		var showTimestamp = attr['show-timestamp'] ? attr['show-timestamp'] == 'true' : true;
		var video = attr['show-video'] ? attr['show-video'] == 'true' : true;
		var text = attr['show-value'] ? attr['show-value'] == 'true' : true; 		
		var group = !attr.group ? false : (attr.group == 'true'); 
		var sequence = attr.sequence ? attr.sequence : '';		
		var tierNames = {}; // map of custom names from options
		var groupNames = {};
		var fileNames = {};
		var columns = [];
		params.forEach(opt=>{ 
			if(opt.hasAttribute('tier')){
				tierNames[opt.getAttribute('tier')+''] = opt.getAttribute("name")+'';
			}
			if(opt.hasAttribute('group')){
				groupNames[opt.getAttribute('group')+''] = opt.getAttribute("name")+'';
			}
			if(opt.hasAttribute('file')){
				fileNames[opt.getAttribute('file')+''] = opt.getAttribute("name")+'';
			}
			if(opt.hasAttribute('column')){
				columns.push(opt.getAttribute('column'));
			}
		});
		var asort = (A,B)=>{ return A.start > B.start ? 1 : -1};
		var tiers = {}; // map of tier id => tier's annotations		
		var files = {}; // map of file name => file's annotations
		var groups = {}; // ... 
		var sequences = {}; // ...
		if(sequence){
			log.debug("USING TIMELINE SEQUENCE: "+sequence);
			group = false; // EAF based
			
		}				
		list.forEach(a=>{
			if(!tiers[a.tier.id]) tiers[a.tier.id] = [];
			tiers[a.tier.id].push(a);
			if(!files[a.af.id]) files[a.af.id] = [];
			files[a.af.id].push(a);
			if(!groups[a.af.group]) groups[a.af.group] = [];
			groups[a.af.group].push(a);
			if(sequence){
				a.af.getTier(sequence).annotations.forEach(seq=>{
					if(a.start>=seq.start && a.stop<=seq.stop){
						var sid = a.af.id+'/'+seq.value;
						if(!sequences[sid]){
							var s = sequences[sid] = [];
							s.id=sid;
							s.name = seq.value+'';
							s.af = seq.af;
							s.start = seq.start;
							s.stop = seq.stop;							
						}
						sequences[sid].push(a);
					}
				});
			}
		});
		var eaf, prev;
		var s = `<div class="timeflow-container">`;
        s += `<table><tbody>`;		
		var aset = group ? groups : files;
		if(sequence) {
			aset = {};
			var a = [];
			for(var k in sequences) a.push(sequences[k]);
			a.sort(
				(a,b)=>{ return a.af.filename+"" == b.af.filename+"" ?  (a.start > b.start ? 1 : -1) : (a.af.filename+"" < b.af.filename+"" ? -1 : 1)}
			);
			a.forEach(seq=>aset[seq.id]=seq);
		}
		var names = group ? groupNames : fileNames;
		for(key in aset){
			// if there is more than one EAF/group, display a separator with its name
			var kname = sequence ? aset[key].name : names[key] || key;						
			if(Object.keys(aset).length>1) s += `<tr class="af-separator sequence-separator"><td colspan="100"><i>${esc(kname)}</i></td></tr>`;
			aset[key].sort(asort);
			var atiers = {};
			columns.forEach(c=>atiers[c]=1);
			aset[key].forEach(a=>{
				if(!atiers[a.tier.id]) atiers[a.tier.id] = 1;
			});
			// show tier names in header
			s += `<tr class="tier-header"><td></td>`;
			for(tier in atiers) s += `<td class="tier-name">`+esc(tierNames[tier] || tier)+`</td>`;			
			s += `</tr>`;				
			
			var first = true;
			aset[key].forEach(a=>{				
				if(prev == a) return; // duplicates from possible OPerations
				if(prev && (a.start - collapse*1000  > prev.stop)){
					var diff = Math.round((a.start - prev.stop)/1000);					
					if(!first) s += `<tr class="time-separator"><td colspan="100"><i>${fdur(diff)}</i></td></tr>`;
				}
				prev = a
				first = false;
				s += `<tr class="annotation">`;
				s+=`<td>`+(showTimestamp ? ftime(a.start/1000) : '')+`</td>`;
				for(tier in atiers){
					s += `<td>`;
					if(a.tier.id.equals(tier)){
						if(video){
							s+=`<video preload="none" poster="${a.snap()}" controls width="300"><source src="${a.clip()}" type="video/mp4"></video>`;
						}
						if(text){
							s+=`<label class="annotation">${esc(a.value)}</label>`;
						}
					}
					s += `</td>`;
				}			
				s += `</tr>`;
			});
		};
		s+=`</tbody></table></div>`;
		return s;
	},
	css:s=>{
		return `
#view tr.time-separator {	
    height: 50px;
	background: rgb(255,255,255);
	background: linear-gradient(0deg, rgba(255,255,255,0) 0%, rgba(0,0,0,0.08167016806722693) 50%, rgba(255,255,255,0) 100%);
	text-align:center;
}
#view tr.tier-header {	
    font-weight:bold;
	text-align:center;
}
#view tr.annotation {
    vertical-align: top;
	
}
#view tr.annotation td:first-child {
    color:gray;
	font-size: 14px;
    padding-right: 5px;
}
#view tr.af-separator td {
    background: black;
    color: white;
    font-weight: bold;
    padding: 5px;
    border-radius: 8px;
    text-align: center;
}
#view tr.annotation td {
    border-top: 1px dashed gainsboro;
    max-width: 320px;
	padding-top: 10px;

}
#view video {
    border-radius: 6px;
	display: block;
}
`
	}
}

function fdur(s){
    var sec_num = parseInt(s, 10);
    var hours   = Math.floor(sec_num / 3600);
    var minutes = Math.floor((sec_num - (hours * 3600)) / 60);
    var seconds = sec_num - (hours * 3600) - (minutes * 60);    
	if(!hours) {
		if(!minutes) return seconds+" seconds";
		return minutes+" minutes";		
	}
    return hours+' hours';
}

function ftime(s){
    var sec_num = parseInt(s, 10);
    var hours   = Math.floor(sec_num / 3600);
    var minutes = Math.floor((sec_num - (hours * 3600)) / 60);
    var seconds = sec_num - (hours * 3600) - (minutes * 60);
    //if (hours   < 10) {hours   = "0"+hours;}
    if (minutes < 10) {minutes = "0"+minutes;}
    if (seconds < 10) {seconds = "0"+seconds;}
    return hours+':'+minutes+':'+seconds;
}